@@ -83,7 +83,6 @@
 
 #include <compat/strl.h>
 #include <retro_inline.h>
-#include <retro_assert.h>
 #include <retro_miscellaneous.h>
 #include <retro_timers.h>
 
@@ -224,7 +223,6 @@ static INLINE int rsnd_format_to_samplesize ( uint16_t fmt )
 
 int rsd_samplesize( rsound_t *rd )
 {
-   retro_assert(rd != NULL);
    return rd->samplesize;
 }
 
@@ -1315,7 +1313,6 @@ int rsd_stop(rsound_t *rd)
 {
    const char buf[] = "RSD    5 STOP";
 
-   retro_assert(rd != NULL);
    rsnd_stop_thread(rd);
 
    /* Do not really care about errors here.
@@ -1329,7 +1326,6 @@ int rsd_stop(rsound_t *rd)
 size_t rsd_write( rsound_t *rsound, const void* buf, size_t size)
 {
    size_t max_write, written = 0;
-   retro_assert(rsound != NULL);
    if ( !rsound->ready_for_data )
       return 0;
 
@@ -1354,21 +1350,13 @@ size_t rsd_write( rsound_t *rsound, const void* buf, size_t size)
 
 int rsd_start(rsound_t *rsound)
 {
-   retro_assert(rsound != NULL);
-   retro_assert(rsound->rate > 0);
-   retro_assert(rsound->channels > 0);
-   retro_assert(rsound->host != NULL);
-   retro_assert(rsound->port != NULL);
-
    if ( rsnd_create_connection(rsound) < 0 )
       return -1;
-
    return 0;
 }
 
 int rsd_exec(rsound_t *rsound)
 {
-   retro_assert(rsound != NULL);
    RSD_DEBUG("[RSound] rsd_exec().\n");
 
    /* Makes sure we have a working connection */
@@ -1419,8 +1407,6 @@ int rsd_exec(rsound_t *rsound)
 /* ioctl()-ish param setting :D */
 int rsd_set_param(rsound_t *rd, enum rsd_settings option, void* param)
 {
-	retro_assert(rd != NULL);
-	retro_assert(param != NULL);
 	int retval = 0;
 
 	switch(option)
@@ -1517,27 +1503,18 @@ void rsd_delay_wait(rsound_t *rd)
 
 size_t rsd_pointer(rsound_t *rsound)
 {
-   retro_assert(rsound != NULL);
-   int ptr;
-
-   ptr = rsnd_get_ptr(rsound);
-
-   return ptr;
+   return rsnd_get_ptr(rsound);
 }
 
 size_t rsd_get_avail(rsound_t *rd)
 {
-   retro_assert(rd != NULL);
-   int ptr;
-   ptr = rsnd_get_ptr(rd);
-   return rd->buffer_size - ptr;
+   return rd->buffer_size - rsnd_get_ptr(rd);
 }
 
 size_t rsd_delay(rsound_t *rd)
 {
-   retro_assert(rd != NULL);
    int ptr = rsnd_get_delay(rd);
-   if ( ptr < 0 )
+   if (ptr < 0)
       ptr = 0;
 
    return ptr;
@@ -1545,16 +1522,12 @@ size_t rsd_delay(rsound_t *rd)
 
 size_t rsd_delay_ms(rsound_t* rd)
 {
-   retro_assert(rd);
-   retro_assert(rd->rate > 0 && rd->channels > 0);
-
    return (rsd_delay(rd) * 1000) / ( rd->rate * rd->channels * rd->samplesize );
 }
 
 int rsd_pause(rsound_t* rsound, int enable)
 {
-   retro_assert(rsound != NULL);
-   if ( enable )
+   if (enable)
       return rsd_stop(rsound);
 
    return rsd_start(rsound);
@@ -1567,8 +1540,6 @@ int rsd_init(rsound_t** rsound)
    if (*rsound == NULL)
       return -1;
 
-   retro_assert(rsound != NULL);
-
    (*rsound)->conn.socket = -1;
    (*rsound)->conn.ctl_socket = -1;
 
@@ -1629,17 +1600,10 @@ int rsd_simple_start(rsound_t** rsound, const char* host, const char* port, cons
 
 void rsd_set_callback(rsound_t *rsound, rsd_audio_callback_t audio_cb, rsd_error_callback_t err_cb, size_t max_size, void *userdata)
 {
-   retro_assert(rsound != NULL);
-
    rsound->audio_callback = audio_cb;
    rsound->error_callback = err_cb;
-   rsound->cb_max_size = max_size;
-   rsound->cb_data = userdata;
-
-   if (rsound->audio_callback)
-   {
-      retro_assert(rsound->error_callback);
-   }
+   rsound->cb_max_size    = max_size;
+   rsound->cb_data        = userdata;
 }
 
 void rsd_callback_lock(rsound_t *rsound)
@@ -1654,7 +1618,6 @@ void rsd_callback_unlock(rsound_t *rsound)
 
 int rsd_free(rsound_t *rsound)
 {
-   retro_assert(rsound != NULL);
    if (rsound->fifo_buffer)
       fifo_free(rsound->fifo_buffer);
    if (rsound->host)
