@@ -34,38 +34,10 @@
 
 #include "configuration.h"
 
-RETRO_BEGIN_DECLS
-
 #define MAX_CMD_DRIVERS              3
 #define DEFAULT_NETWORK_CMD_PORT 55355
 
-struct cmd_map
-{
-   const char *str;
-   unsigned id;
-};
-
-struct command_handler;
-
-typedef void (*command_poller_t)(struct command_handler *cmd);
-typedef void (*command_replier_t)(struct command_handler *cmd, const char * data, size_t len);
-typedef void (*command_destructor_t)(struct command_handler *cmd);
-
-struct command_handler
-{
-   /* Interface to poll the driver */
-   command_poller_t poll;
-   /* Interface to reply */
-   command_replier_t replier;
-   /* Interface to delete the underlying command */
-   command_destructor_t destroy;
-   /* Underlying command storage */
-   void *userptr;
-   /* State received */
-   bool state[RARCH_BIND_LIST_END];
-};
-
-typedef struct command_handler command_t;
+RETRO_BEGIN_DECLS
 
 enum event_command
 {
@@ -92,6 +64,12 @@ enum event_command
    CMD_EVENT_SAVE_STATE,
    CMD_EVENT_SAVE_STATE_DECREMENT,
    CMD_EVENT_SAVE_STATE_INCREMENT,
+   /* Replay hotkeys. */
+   CMD_EVENT_PLAY_REPLAY,
+   CMD_EVENT_RECORD_REPLAY,
+   CMD_EVENT_HALT_REPLAY,
+   CMD_EVENT_REPLAY_DECREMENT,
+   CMD_EVENT_REPLAY_INCREMENT,
    /* Save state actions. */
    CMD_EVENT_SAVE_STATE_TO_RAM,
    CMD_EVENT_LOAD_STATE_FROM_RAM,
@@ -110,6 +88,8 @@ enum event_command
    CMD_EVENT_REWIND_DEINIT,
    /* Initializes rewind. */
    CMD_EVENT_REWIND_INIT,
+   /* Reinitializes rewind (primarily if the state size changes). */
+   CMD_EVENT_REWIND_REINIT,
    /* Toggles rewind. */
    CMD_EVENT_REWIND_TOGGLE,
    /* Initializes autosave. */
@@ -131,8 +111,8 @@ enum event_command
    CMD_EVENT_STATISTICS_TOGGLE,
    /* Initializes overlay. */
    CMD_EVENT_OVERLAY_INIT,
-   /* Deinitializes overlay. */
-   CMD_EVENT_OVERLAY_DEINIT,
+   /* Frees or caches overlay. */
+   CMD_EVENT_OVERLAY_UNLOAD,
    /* Sets current scale factor for overlay. */
    CMD_EVENT_OVERLAY_SET_SCALE_FACTOR,
    /* Sets current alpha modulation for overlay. */
@@ -186,11 +166,14 @@ enum event_command
    CMD_EVENT_MENU_TOGGLE,
    /* Configuration saving. */
    CMD_EVENT_MENU_RESET_TO_DEFAULT_CONFIG,
+   CMD_EVENT_MENU_SAVE_CONFIG,
    CMD_EVENT_MENU_SAVE_CURRENT_CONFIG,
    CMD_EVENT_MENU_SAVE_CURRENT_CONFIG_OVERRIDE_CORE,
    CMD_EVENT_MENU_SAVE_CURRENT_CONFIG_OVERRIDE_CONTENT_DIR,
    CMD_EVENT_MENU_SAVE_CURRENT_CONFIG_OVERRIDE_GAME,
-   CMD_EVENT_MENU_SAVE_CONFIG,
+   CMD_EVENT_MENU_REMOVE_CURRENT_CONFIG_OVERRIDE_CORE,
+   CMD_EVENT_MENU_REMOVE_CURRENT_CONFIG_OVERRIDE_CONTENT_DIR,
+   CMD_EVENT_MENU_REMOVE_CURRENT_CONFIG_OVERRIDE_GAME,
    /* Applies shader changes. */
    CMD_EVENT_SHADERS_APPLY_CHANGES,
    /* A new shader preset has been loaded */
@@ -257,10 +240,14 @@ enum event_command
    CMD_EVENT_RECORDING_TOGGLE,
    /* Toggle streaming. */
    CMD_EVENT_STREAMING_TOGGLE,
-   /* Toggle BSV recording. */
-   CMD_EVENT_BSV_RECORDING_TOGGLE,
    /* Toggle Run-Ahead. */
    CMD_EVENT_RUNAHEAD_TOGGLE,
+   /* Toggle Preemtive Frames. */
+   CMD_EVENT_PREEMPT_TOGGLE,
+   /* Deinitialize or Reinitialize Preemptive Frames. */
+   CMD_EVENT_PREEMPT_UPDATE,
+   /* Force Preemptive Frames to refill its state buffer. */
+   CMD_EVENT_PREEMPT_RESET_BUFFER,
    /* Toggle VRR runloop. */
    CMD_EVENT_VRR_RUNLOOP_TOGGLE,
    /* AI service. */
@@ -273,17 +260,18 @@ enum event_command
    CMD_EVENT_PRESENCE_UPDATE,
    CMD_EVENT_OVERLAY_NEXT,
    CMD_EVENT_OSK_TOGGLE,
-
+#ifdef HAVE_MICROPHONE
+   /* Stops all enabled microphones. */
+   CMD_EVENT_MICROPHONE_STOP,
+   /* Starts all enabled microphones */
+   CMD_EVENT_MICROPHONE_START,
+   /* Reinitializes microphone driver. */
+   CMD_EVENT_MICROPHONE_REINIT,
+#endif
    /* Deprecated */
    CMD_EVENT_SEND_DEBUG_INFO
 };
 
-typedef struct command_handle
-{
-   command_t *handle;
-   unsigned id;
-} command_handle_t;
-
 enum cmd_source_t
 {
    CMD_NONE = 0,
@@ -291,6 +279,40 @@ enum cmd_source_t
    CMD_NETWORK
 };
 
+struct cmd_map
+{
+   const char *str;
+   unsigned id;
+};
+
+struct command_handler;
+
+typedef void (*command_poller_t)(struct command_handler *cmd);
+typedef void (*command_replier_t)(struct command_handler *cmd, const char * data, size_t len);
+typedef void (*command_destructor_t)(struct command_handler *cmd);
+
+struct command_handler
+{
+   /* Interface to poll the driver */
+   command_poller_t poll;
+   /* Interface to reply */
+   command_replier_t replier;
+   /* Interface to delete the underlying command */
+   command_destructor_t destroy;
+   /* Underlying command storage */
+   void *userptr;
+   /* State received */
+   bool state[RARCH_BIND_LIST_END];
+};
+
+typedef struct command_handler command_t;
+
+typedef struct command_handle
+{
+   command_t *handle;
+   unsigned id;
+} command_handle_t;
+
 struct rarch_state;
 
 /**
@@ -310,45 +332,6 @@ command_t* command_uds_new(void);
 
 bool command_network_send(const char *cmd_);
 
-#ifdef HAVE_BSV_MOVIE
-enum bsv_flags
-{
-   BSV_FLAG_MOVIE_START_RECORDING    = (1 << 0),
-   BSV_FLAG_MOVIE_START_PLAYBACK     = (1 << 1),
-   BSV_FLAG_MOVIE_PLAYBACK           = (1 << 2),
-   BSV_FLAG_MOVIE_EOF_EXIT           = (1 << 3),
-   BSV_FLAG_MOVIE_END                = (1 << 4)
-};
-
-struct bsv_state
-{
-   uint8_t flags;
-   /* Movie playback/recording support. */
-   char movie_path[PATH_MAX_LENGTH];
-   /* Immediate playback/recording. */
-   char movie_start_path[PATH_MAX_LENGTH];
-};
-
-struct bsv_movie
-{
-   intfstream_t *file;
-   uint8_t *state;
-   /* A ring buffer keeping track of positions
-    * in the file for each frame. */
-   size_t *frame_pos;
-   size_t frame_mask;
-   size_t frame_ptr;
-   size_t min_file_pos;
-   size_t state_size;
-
-   bool playback;
-   bool first_rewind;
-   bool did_rewind;
-};
-
-typedef struct bsv_movie bsv_movie_t;
-#endif
-
 #ifdef HAVE_CONFIGFILE
 bool command_event_save_config(
       const char *config_path,
@@ -366,9 +349,7 @@ void command_event_set_mixer_volume(
 bool command_event_resize_windowed_scale(settings_t *settings,
       unsigned window_scale);
 
-bool command_event_save_auto_state(
-      bool savestate_auto_save,
-      const enum rarch_core_type current_core_type);
+bool command_event_save_auto_state(void);
 
 /**
  * event_set_volume:
@@ -391,7 +372,7 @@ void command_event_set_volume(
 void command_event_init_controllers(rarch_system_info_t *info,
       settings_t *settings, unsigned num_active_users);
 
-bool command_event_load_entry_state(void);
+bool command_event_load_entry_state(settings_t *settings);
 
 void command_event_load_auto_state(void);
 
@@ -403,6 +384,14 @@ void command_event_set_savestate_garbage_collect(
       bool show_hidden_files
       );
 
+void command_event_set_replay_auto_index(
+      settings_t *settings);
+
+void command_event_set_replay_garbage_collect(
+      unsigned max_to_keep,
+      bool show_hidden_files
+      );
+
 #if defined(HAVE_CG) || defined(HAVE_GLSL) || defined(HAVE_SLANG) || defined(HAVE_HLSL)
 bool command_set_shader(command_t *cmd, const char *arg);
 #endif
@@ -426,19 +415,14 @@ bool command_version(command_t *cmd, const char* arg);
 bool command_get_status(command_t *cmd, const char* arg);
 bool command_get_config_param(command_t *cmd, const char* arg);
 bool command_show_osd_msg(command_t *cmd, const char* arg);
+bool command_load_state_slot(command_t *cmd, const char* arg);
+bool command_play_replay_slot(command_t *cmd, const char* arg);
 #ifdef HAVE_CHEEVOS
 bool command_read_ram(command_t *cmd, const char *arg);
 bool command_write_ram(command_t *cmd, const char *arg);
 #endif
 bool command_read_memory(command_t *cmd, const char *arg);
 bool command_write_memory(command_t *cmd, const char *arg);
-uint8_t *command_memory_get_pointer(
-      const rarch_system_info_t* system,
-      unsigned address,
-      unsigned int* max_bytes,
-      int for_write,
-      char *reply_at,
-      size_t len);
 
 static const struct cmd_action_map action_map[] = {
 #if defined(HAVE_CG) || defined(HAVE_GLSL) || defined(HAVE_SLANG) || defined(HAVE_HLSL)
@@ -456,6 +440,9 @@ static const struct cmd_action_map action_map[] = {
 #endif
    { "READ_CORE_MEMORY", command_read_memory,      "<address> <number of bytes>" },
    { "WRITE_CORE_MEMORY",command_write_memory,     "<address> <byte1> <byte2> ..." },
+
+   { "LOAD_STATE_SLOT",command_load_state_slot, "<slot number>"},
+   { "PLAY_REPLAY_SLOT",command_play_replay_slot, "<slot number>"},
 };
 
 static const struct cmd_map map[] = {
@@ -481,6 +468,12 @@ static const struct cmd_map map[] = {
    { "STATE_SLOT_PLUS",        RARCH_STATE_SLOT_PLUS },
    { "STATE_SLOT_MINUS",       RARCH_STATE_SLOT_MINUS },
 
+   { "PLAY_REPLAY",            RARCH_PLAY_REPLAY_KEY },
+   { "RECORD_REPLAY",          RARCH_RECORD_REPLAY_KEY },
+   { "HALT_REPLAY",            RARCH_HALT_REPLAY_KEY },
+   { "REPLAY_SLOT_PLUS",       RARCH_REPLAY_SLOT_PLUS },
+   { "REPLAY_SLOT_MINUS",      RARCH_REPLAY_SLOT_MINUS },
+
    { "DISK_EJECT_TOGGLE",      RARCH_DISK_EJECT_TOGGLE },
    { "DISK_NEXT",              RARCH_DISK_NEXT },
    { "DISK_PREV",              RARCH_DISK_PREV },
@@ -496,7 +489,6 @@ static const struct cmd_map map[] = {
    { "SCREENSHOT",             RARCH_SCREENSHOT },
    { "RECORDING_TOGGLE",       RARCH_RECORDING_TOGGLE },
    { "STREAMING_TOGGLE",       RARCH_STREAMING_TOGGLE },
-   { "BSV_RECORD_TOGGLE",      RARCH_BSV_RECORD_TOGGLE },
 
    { "GRAB_MOUSE_TOGGLE",      RARCH_GRAB_MOUSE_TOGGLE },
    { "GAME_FOCUS_TOGGLE",      RARCH_GAME_FOCUS_TOGGLE },
@@ -505,6 +497,7 @@ static const struct cmd_map map[] = {
 
    { "VRR_RUNLOOP_TOGGLE",     RARCH_VRR_RUNLOOP_TOGGLE },
    { "RUNAHEAD_TOGGLE",        RARCH_RUNAHEAD_TOGGLE },
+   { "PREEMPT_TOGGLE",         RARCH_PREEMPT_TOGGLE },
    { "FPS_TOGGLE",             RARCH_FPS_TOGGLE },
    { "STATISTICS_TOGGLE",      RARCH_STATISTICS_TOGGLE },
    { "AI_SERVICE",             RARCH_AI_SERVICE },
@@ -552,6 +545,13 @@ bool command_event_save_core_config(
  * autosave state.
  **/
 void command_event_save_current_config(enum override_type type);
+
+/**
+ * command_event_remove_current_config:
+ *
+ * Removes current configuration file from disk.
+ **/
+void command_event_remove_current_config(enum override_type type);
 #endif
 
 /**
