@@ -68,6 +68,8 @@
    strlcpy(var, newvar, sizeof(var)); \
 }
 
+RETRO_BEGIN_DECLS
+
 enum crt_switch_type
 {
    CRT_SWITCH_NONE = 0,
@@ -80,13 +82,12 @@ enum crt_switch_type
 enum override_type
 {
    OVERRIDE_NONE = 0,
+   OVERRIDE_AS,
    OVERRIDE_CORE,
    OVERRIDE_CONTENT_DIR,
    OVERRIDE_GAME
 };
 
-RETRO_BEGIN_DECLS
-
 typedef struct settings
 {
    struct
@@ -104,7 +105,7 @@ typedef struct settings
       int location_update_interval_ms;
       int location_update_interval_distance;
       int state_slot;
-      int audio_wasapi_sh_buffer_length;
+      int replay_slot;
       int crt_switch_center_adjust;
       int crt_switch_porch_adjust;
 #ifdef HAVE_VULKAN
@@ -133,6 +134,9 @@ typedef struct settings
 #ifdef HAVE_XMB
       int menu_xmb_title_margin;
       int menu_xmb_title_margin_horizontal_offset;
+#endif
+#ifdef HAVE_OVERLAY
+      int input_overlay_lightgun_port;
 #endif
    } ints;
 
@@ -159,6 +163,20 @@ typedef struct settings
       unsigned audio_block_frames;
       unsigned audio_latency;
 
+#ifdef HAVE_WASAPI
+      unsigned audio_wasapi_sh_buffer_length;
+#endif
+
+#ifdef HAVE_MICROPHONE
+      unsigned microphone_sample_rate;
+      unsigned microphone_block_frames;
+      unsigned microphone_latency;
+      unsigned microphone_resampler_quality;
+#ifdef HAVE_WASAPI
+      unsigned microphone_wasapi_sh_buffer_length;
+#endif
+#endif
+
       unsigned fps_update_interval;
       unsigned memory_update_interval;
 
@@ -203,6 +221,8 @@ typedef struct settings
       unsigned rewind_granularity;
       unsigned rewind_buffer_size_step;
       unsigned autosave_interval;
+      unsigned replay_checkpoint_interval;
+      unsigned replay_max_keep;
       unsigned savestate_max_keep;
       unsigned network_cmd_port;
       unsigned network_remote_base_port;
@@ -236,9 +256,6 @@ typedef struct settings
       unsigned video_dingux_ipu_filter_type;
       unsigned video_dingux_refresh_rate;
       unsigned video_dingux_rs90_softfilter_type;
-#ifdef HAVE_VIDEO_LAYOUT
-      unsigned video_layout_selected_view;
-#endif
 #ifdef GEKKO
       unsigned video_overscan_correction_top;
       unsigned video_overscan_correction_bottom;
@@ -305,6 +322,12 @@ typedef struct settings
       unsigned input_overlay_show_inputs_port;
       unsigned input_overlay_dpad_diagonal_sensitivity;
       unsigned input_overlay_abxy_diagonal_sensitivity;
+      unsigned input_overlay_lightgun_trigger_delay;
+      unsigned input_overlay_lightgun_two_touch_input;
+      unsigned input_overlay_lightgun_three_touch_input;
+      unsigned input_overlay_lightgun_four_touch_input;
+      unsigned input_overlay_mouse_hold_msec;
+      unsigned input_overlay_mouse_dtap_msec;
 #endif
 
       unsigned run_ahead_frames;
@@ -325,9 +348,14 @@ typedef struct settings
       unsigned ai_service_mode;
       unsigned ai_service_target_lang;
       unsigned ai_service_source_lang;
+      unsigned ai_service_poll_delay;
+      unsigned ai_service_text_position;
+      unsigned ai_service_text_padding;
 
       unsigned core_updater_auto_backup_history_size;
       unsigned video_black_frame_insertion;
+      unsigned video_bfi_dark_frames;
+      unsigned video_shader_subframes;
       unsigned video_autoswitch_refresh_rate;
       unsigned quit_on_close_content;
 
@@ -340,6 +368,9 @@ typedef struct settings
 #ifdef HAVE_MIST
       unsigned steam_rich_presence_format;
 #endif
+
+      unsigned cheevos_appearance_anchor;
+      unsigned cheevos_visibility_summary;
    } uints;
 
    struct
@@ -347,6 +378,7 @@ typedef struct settings
       float placeholder;
       float video_aspect_ratio;
       float video_refresh_rate;
+      float video_autoswitch_pal_threshold;
       float crt_video_refresh_rate;
       float video_font_size;
       float video_msg_pos_x;
@@ -371,11 +403,15 @@ typedef struct settings
       float menu_screensaver_animation_speed;
       float ozone_thumbnail_scale_factor;
 
+      float cheevos_appearance_padding_h;
+      float cheevos_appearance_padding_v;
+
       float audio_max_timing_skew;
       float audio_volume; /* dB scale. */
       float audio_mixer_volume; /* dB scale. */
 
       float input_overlay_opacity;
+      float input_osk_overlay_opacity;
 
       float input_overlay_scale_landscape;
       float input_overlay_aspect_adjust_landscape;
@@ -391,6 +427,9 @@ typedef struct settings
       float input_overlay_x_offset_portrait;
       float input_overlay_y_offset_portrait;
 
+      float input_overlay_mouse_speed;
+      float input_overlay_mouse_swipe_threshold;
+
       float slowmotion_ratio;
       float fastforward_ratio;
       float input_analog_deadzone;
@@ -412,6 +451,7 @@ typedef struct settings
       char wifi_driver[32];
       char led_driver[32];
       char location_driver[32];
+      char cloud_sync_driver[32];
       char menu_driver[32];
       char cheevos_username[32];
       char cheevos_password[256];
@@ -429,12 +469,26 @@ typedef struct settings
 
       char input_keyboard_layout[64];
 
+#ifdef HAVE_MICROPHONE
+      char microphone_driver[32];
+      char microphone_resampler[32];
+      char microphone_device[255];
+#endif
+
+#ifdef ANDROID
+      char input_android_physical_keyboard[255];
+#endif
+
       char audio_device[255];
       char camera_device[255];
       char netplay_mitm_server[255];
 
       char translation_service_url[2048];
 
+      char webdav_url[255];
+      char webdav_username[255];
+      char webdav_password[255];
+
       char youtube_stream_key[PATH_MAX_LENGTH];
       char twitch_stream_key[PATH_MAX_LENGTH];
       char facebook_stream_key[PATH_MAX_LENGTH];
@@ -476,9 +530,7 @@ typedef struct settings
       char path_cheat_database[PATH_MAX_LENGTH];
       char path_content_database[PATH_MAX_LENGTH];
       char path_overlay[PATH_MAX_LENGTH];
-#ifdef HAVE_VIDEO_LAYOUT
-      char path_video_layout[PATH_MAX_LENGTH];
-#endif
+      char path_osk_overlay[PATH_MAX_LENGTH];
       char path_record_config[PATH_MAX_LENGTH];
       char path_stream_config[PATH_MAX_LENGTH];
       char path_menu_wallpaper[PATH_MAX_LENGTH];
@@ -502,10 +554,7 @@ typedef struct settings
       char directory_libretro[PATH_MAX_LENGTH];
       char directory_input_remapping[PATH_MAX_LENGTH];
       char directory_overlay[PATH_MAX_LENGTH];
-#ifdef HAVE_VIDEO_LAYOUT
-      char directory_video_layout[PATH_MAX_LENGTH];
-#endif
-      char directory_resampler[PATH_MAX_LENGTH];
+      char directory_osk_overlay[PATH_MAX_LENGTH];
       char directory_screenshot[PATH_MAX_LENGTH];
       char directory_system[PATH_MAX_LENGTH];
       char directory_cache[PATH_MAX_LENGTH];
@@ -525,11 +574,16 @@ typedef struct settings
       char streaming_title[PATH_MAX_LENGTH];
 #ifdef _3DS
       char directory_bottom_assets[PATH_MAX_LENGTH];
+#endif
+#ifdef HAVE_TEST_DRIVERS
+      char test_input_file_joypad[PATH_MAX_LENGTH];
 #endif
       char log_dir[PATH_MAX_LENGTH];
+      char app_icon[PATH_MAX_LENGTH];
    } paths;
 
    bool modified;
+   bool skip_window_positions;
 
    struct
    {
@@ -547,6 +601,7 @@ typedef struct settings
       bool video_ctx_scaling;
       bool video_force_aspect;
       bool video_frame_delay_auto;
+      bool video_frame_rest;
       bool video_crop_overscan;
       bool video_aspect_ratio_auto;
       bool video_dingux_ipu_keep_aspect;
@@ -556,6 +611,7 @@ typedef struct settings
       bool video_shader_watch_files;
       bool video_shader_remember_last_dir;
       bool video_shader_preset_save_reference_enable;
+      bool video_scan_subframes;
       bool video_threaded;
       bool video_font_enable;
       bool video_disable_composition;
@@ -575,9 +631,6 @@ typedef struct settings
 #endif
       bool video_wiiu_prefer_drc;
       bool video_notch_write_over_enable;
-#ifdef HAVE_VIDEO_LAYOUT
-      bool video_layout_enable;
-#endif
       bool video_hdr_enable;
       bool video_hdr_expand_gamut;
 
@@ -591,11 +644,28 @@ typedef struct settings
       bool audio_enable_menu_cancel;
       bool audio_enable_menu_notice;
       bool audio_enable_menu_bgm;
+      bool audio_enable_menu_scroll;
       bool audio_sync;
       bool audio_rate_control;
+      bool audio_fastforward_mute;
+      bool audio_fastforward_speedup;
+#ifdef TARGET_OS_IOS
+      bool audio_respect_silent_mode;
+#endif
+
+#ifdef HAVE_WASAPI
       bool audio_wasapi_exclusive_mode;
       bool audio_wasapi_float_format;
-      bool audio_fastforward_mute;
+#endif
+
+#ifdef HAVE_MICROPHONE
+      /* Microphone */
+      bool microphone_enable;
+#ifdef HAVE_WASAPI
+      bool microphone_wasapi_exclusive_mode;
+      bool microphone_wasapi_float_format;
+#endif
+#endif
 
       /* Input */
       bool input_remap_binds_enable;
@@ -609,6 +679,12 @@ typedef struct settings
       bool input_overlay_show_mouse_cursor;
       bool input_overlay_auto_rotate;
       bool input_overlay_auto_scale;
+      bool input_osk_overlay_auto_scale;
+      bool input_overlay_pointer_enable;
+      bool input_overlay_lightgun_trigger_on_touch;
+      bool input_overlay_lightgun_allow_offscreen;
+      bool input_overlay_mouse_hold_to_drag;
+      bool input_overlay_mouse_dtap_to_drag;
       bool input_descriptor_label_show;
       bool input_descriptor_hide_unbound;
       bool input_all_users_control_menu;
@@ -619,9 +695,18 @@ typedef struct settings
       bool input_small_keyboard_enable;
       bool input_keyboard_gamepad_enable;
       bool input_auto_mouse_grab;
+      bool input_allow_turbo_dpad;
+      bool input_hotkey_device_merge;
 #if defined(HAVE_DINPUT) || defined(HAVE_WINRAWINPUT)
       bool input_nowinkey_enable;
 #endif
+#ifdef UDEV_TOUCH_SUPPORT
+      bool input_touch_vmouse_pointer;
+      bool input_touch_vmouse_mouse;
+      bool input_touch_vmouse_touchpad;
+      bool input_touch_vmouse_trackball;
+      bool input_touch_vmouse_gesture;
+#endif
 
       /* Frame time counter */
       bool frame_time_counter_reset_after_fastforwarding;
@@ -638,6 +723,8 @@ typedef struct settings
       bool notification_show_remap_load;
       bool notification_show_config_override_load;
       bool notification_show_set_initial_disk;
+      bool notification_show_disk_control;
+      bool notification_show_save_state;
       bool notification_show_fast_forward;
 #ifdef HAVE_SCREENSHOTS
       bool notification_show_screenshot;
@@ -657,13 +744,11 @@ typedef struct settings
       bool menu_core_enable;
       bool menu_show_sublabels;
       bool menu_dynamic_wallpaper_enable;
-      bool menu_throttle;
       bool menu_mouse_enable;
       bool menu_pointer_enable;
       bool menu_navigation_wraparound_enable;
       bool menu_navigation_browser_filter_supported_extensions_enable;
       bool menu_show_advanced_settings;
-      bool menu_throttle_framerate;
       bool menu_linear_filter;
       bool menu_horizontal_animation;
       bool menu_scroll_fast;
@@ -689,12 +774,13 @@ typedef struct settings
       bool menu_show_latency;
       bool menu_show_rewind;
       bool menu_show_overlays;
+#if 0
+/* Thumbnailpack removal */
       bool menu_show_legacy_thumbnail_updater;
-#ifdef HAVE_VIDEO_LAYOUT
-      bool menu_show_video_layout;
 #endif
       bool menu_materialui_icons_enable;
       bool menu_materialui_playlist_icons_enable;
+      bool menu_materialui_switch_icons;
       bool menu_materialui_show_nav_bar;
       bool menu_materialui_auto_rotate_nav_bar;
       bool menu_materialui_dual_thumbnail_list_view_enable;
@@ -711,8 +797,9 @@ typedef struct settings
       bool menu_rgui_switch_icons;
       bool menu_rgui_particle_effect_screensaver;
       bool menu_xmb_shadows_enable;
-      bool menu_xmb_vertical_thumbnails;
       bool menu_xmb_show_title_header;
+      bool menu_xmb_switch_icons;
+      bool menu_xmb_vertical_thumbnails;
       bool menu_content_show_settings;
       bool menu_content_show_favorites;
       bool menu_content_show_images;
@@ -760,6 +847,7 @@ typedef struct settings
       bool quick_menu_show_take_screenshot;
       bool quick_menu_show_savestate_submenu;
       bool quick_menu_show_save_load_state;
+      bool quick_menu_show_replay;
       bool quick_menu_show_undo_save_load_state;
       bool quick_menu_show_add_to_favorites;
       bool quick_menu_show_start_recording;
@@ -823,6 +911,15 @@ typedef struct settings
       bool cheevos_start_active;
       bool cheevos_unlock_sound_enable;
       bool cheevos_challenge_indicators;
+      bool cheevos_appearance_padding_auto;
+      bool cheevos_visibility_unlock;
+      bool cheevos_visibility_mastery;
+      bool cheevos_visibility_account;
+      bool cheevos_visibility_lboard_start;
+      bool cheevos_visibility_lboard_submit;
+      bool cheevos_visibility_lboard_cancel;
+      bool cheevos_visibility_lboard_trackers;
+      bool cheevos_visibility_progress_tracker;
 
       /* Camera */
       bool camera_allow;
@@ -853,6 +950,10 @@ typedef struct settings
       bool steam_rich_presence_enable;
 #endif
 
+      /* Cloud Sync */
+      bool cloud_sync_enable;
+      bool cloud_sync_destructive;
+
       /* Misc. */
       bool discord_enable;
       bool threaded_data_runloop_enable;
@@ -863,14 +964,18 @@ typedef struct settings
       bool rewind_enable;
       bool fastforward_frameskip;
       bool vrr_runloop_enable;
+      bool menu_throttle_framerate;
       bool apply_cheats_after_toggle;
       bool apply_cheats_after_load;
       bool run_ahead_enabled;
       bool run_ahead_secondary_instance;
       bool run_ahead_hide_warnings;
+      bool preemptive_frames_enable;
+      bool preemptive_frames_hide_warnings;
       bool pause_nonactive;
       bool pause_on_disconnect;
       bool block_sram_overwrite;
+      bool replay_auto_index;
       bool savestate_auto_index;
       bool savestate_auto_save;
       bool savestate_auto_load;
@@ -886,6 +991,7 @@ typedef struct settings
       bool check_firmware_before_loading;
       bool core_option_category_enable;
       bool core_info_cache_enable;
+      bool core_info_savestate_bypass;
 #ifndef HAVE_DYNAMIC
       bool always_reload_core_on_run_content;
 #endif
@@ -893,6 +999,7 @@ typedef struct settings
       bool game_specific_options;
       bool auto_overrides_enable;
       bool auto_remaps_enable;
+      bool initial_disk_change_enable;
       bool global_core_options;
       bool auto_shaders_enable;
 
@@ -911,6 +1018,11 @@ typedef struct settings
       bool screenshots_in_content_dir;
       bool systemfiles_in_content_dir;
       bool ssh_enable;
+#ifdef HAVE_LAKKA_SWITCH
+      bool switch_oc;
+      bool switch_cec;
+      bool bluetooth_ertm_disable;
+#endif
       bool samba_enable;
       bool bluetooth_enable;
       bool localap_enable;
@@ -930,6 +1042,7 @@ typedef struct settings
       bool playlist_show_entry_idx;
       bool playlist_fuzzy_archive_match;
       bool playlist_portable_paths;
+      bool playlist_use_filename;
 
       bool quit_press_twice;
       bool vibrate_on_keypress;
@@ -943,6 +1056,7 @@ typedef struct settings
       bool log_to_file_timestamp;
 
       bool scan_without_core_match;
+      bool scan_serial_and_crc;
 
       bool ai_service_enable;
       bool ai_service_pause;
@@ -956,6 +1070,10 @@ typedef struct settings
 #ifdef ANDROID
       bool android_input_disconnect_workaround;
 #endif
+
+#if defined(HAVE_COCOATOUCH) && defined(TARGET_OS_TV)
+      bool gcdwebserver_alert;
+#endif
    } bools;
 
 } settings_t;
@@ -1014,6 +1132,17 @@ const char *config_get_default_video(void);
  **/
 const char *config_get_default_audio(void);
 
+#if defined(HAVE_MICROPHONE)
+/**
+ * config_get_default_microphone:
+ *
+ * Gets default microphone driver.
+ *
+ * Returns: Default microphone driver.
+ **/
+const char *config_get_default_microphone(void);
+#endif
+
 /**
  * config_get_default_audio_resampler:
  *
@@ -1068,6 +1197,18 @@ const char *config_get_default_record(void);
  */
 bool config_load_override(void *data);
 
+/**
+ * config_load_override_file:
+ *
+ * Tries to load specified configuration file.
+ * These settings will always have precedence, thus this feature
+ * can be used to enforce overrides.
+ *
+ * Returns: false if there was an error or no action was performed.
+ *
+ */
+bool config_load_override_file(const char *path);
+
 /**
  * config_unload_override:
  *
@@ -1089,6 +1230,16 @@ bool config_unload_override(void);
 bool config_load_remap(const char *directory_input_remapping,
       void *data);
 
+/**
+ * config_get_autoconf_profile_filename:
+ * @device_name       : Input device name
+ * @user              : Controller number to save
+ * Fills buf with the autoconf profile file name (including driver dir if needed).
+ **/
+
+void config_get_autoconf_profile_filename(
+      const char *device_name, unsigned user,
+      char *buf, size_t len_buf);
 /**
  * config_save_autoconf_profile:
  * @device_name       : Input device name
@@ -1113,9 +1264,10 @@ bool config_save_file(const char *path);
  *
  * Writes a config file override to disk.
  *
- * Returns: true (1) on success, otherwise returns false (0).
+ * Returns: true (1) on success, (-1) if nothing to write, otherwise returns false (0).
  **/
-bool config_save_overrides(enum override_type type, void *data);
+int8_t config_save_overrides(enum override_type type,
+      void *data, bool remove, const char *path);
 
 /* Replaces currently loaded configuration file with
  * another one. Will load a dummy core to flush state
@@ -1139,9 +1291,9 @@ void config_load_file_salamander(void);
 void config_save_file_salamander(void);
 #endif
 
-void rarch_config_init(void);
+void retroarch_config_init(void);
 
-void rarch_config_deinit(void);
+void retroarch_config_deinit(void);
 
 settings_t *config_get_ptr(void);
 
