@@ -854,12 +854,12 @@ core_option_manager_t *core_option_manager_new_vars(
    {
       if (core_option_manager_parse_variable(opt, size, var, config_src))
       {
+         size_t _len = 0;
          /* If variable is read correctly, add it to
           * the map */
          char address[256];
-
-         address[0] = '#';
-         address[1] = '\0';
+         address[  _len]  = '#';
+         address[++_len]  = '\0';
 
          /* Address string is normally:
           *    <category_key><delim><tag><option_key>
@@ -869,7 +869,7 @@ core_option_manager_t *core_option_manager_new_vars(
           * so we could just set the address to
           * <option_key> - but for consistency with
           * 'modern' options, we apply the tag regardless */
-         strlcat(address, var->key, sizeof(address));
+         strlcpy(address + _len, var->key, sizeof(address) - _len);
 
          if (!nested_list_add_item(opt->option_map,
                address, NULL, (const void*)&opt->opts[size]))
@@ -1190,7 +1190,6 @@ core_option_manager_t *core_option_manager_new(
          const char *category_key = opt->opts[size].category_key;
          char address[256];
 
-
          /* Address string is nominally:
           *    <category_key><delim><tag><option_key>
           * ...where <tag> is prepended to the option
@@ -1198,17 +1197,18 @@ core_option_manager_t *core_option_manager_new(
           * collisions */
          if (string_is_empty(category_key))
          {
-            address[0] = '#';
-            address[1] = '\0';
-            strlcat(address, option_def->key, sizeof(address));
+            size_t _len     = 0;
+            address[  _len] = '#';
+            address[++_len] = '\0';
+            strlcpy(address + _len, option_def->key, sizeof(address) - _len);
          }
          else
          {
             size_t _len      = strlcpy(address, category_key, sizeof(address));
-            address[_len  ]  = ':';
-            address[_len+1]  = '#';
-            address[_len+2]  = '\0';
-            strlcat(address, option_def->key, sizeof(address));
+            address[  _len]  = ':';
+            address[++_len]  = '#';
+            address[++_len]  = '\0';
+            strlcpy(address + _len, option_def->key, sizeof(address) - _len);
          }
 
          if (!nested_list_add_item(opt->option_map,
@@ -1334,12 +1334,10 @@ const char *core_option_manager_get_category_desc(core_option_manager_t *opt,
    {
       struct core_category *category = &opt->cats[i];
 
-      if ((key_hash == category->key_hash) &&
-          !string_is_empty(category->key) &&
-          string_is_equal(key, category->key))
-      {
+      if (   (key_hash == category->key_hash)
+          && !string_is_empty(category->key)
+          &&  string_is_equal(key, category->key))
          return category->desc;
-      }
    }
 
    return NULL;
@@ -1375,12 +1373,10 @@ const char *core_option_manager_get_category_info(core_option_manager_t *opt,
    {
       struct core_category *category = &opt->cats[i];
 
-      if ((key_hash == category->key_hash) &&
-          !string_is_empty(category->key) &&
-          string_is_equal(key, category->key))
-      {
+      if (   (key_hash == category->key_hash)
+          && !string_is_empty(category->key)
+          &&  string_is_equal(key, category->key))
          return category->info;
-      }
    }
 
    return NULL;
@@ -1404,14 +1400,12 @@ const char *core_option_manager_get_category_info(core_option_manager_t *opt,
 bool core_option_manager_get_category_visible(core_option_manager_t *opt,
       const char *key)
 {
+   size_t i;
    nested_list_item_t *category_item = NULL;
    nested_list_t *option_list        = NULL;
-   nested_list_item_t *option_item   = NULL;
-   const struct core_option *option  = NULL;
-   size_t i;
 
-   if (!opt ||
-       string_is_empty(key))
+   if (  !opt 
+       || string_is_empty(key))
       return false;
 
    /* Fetch category item from map */
@@ -1426,10 +1420,9 @@ bool core_option_manager_get_category_visible(core_option_manager_t *opt,
    /* Loop over child options */
    for (i = 0; i < nested_list_get_size(option_list); i++)
    {
-      option_item = nested_list_get_item_idx(option_list, i);
-      option      = (const struct core_option *)
+      nested_list_item_t *option_item  = nested_list_get_item_idx(option_list, i);
+      const struct core_option *option = (const struct core_option *)
             nested_list_item_get_value(option_item);
-
       /* Check if current option is visible */
       if (option && option->visible)
          return true;
@@ -1463,9 +1456,9 @@ bool core_option_manager_get_idx(core_option_manager_t *opt,
    uint32_t key_hash;
    size_t i;
 
-   if (!opt ||
-       string_is_empty(key) ||
-       !idx)
+   if (  !opt
+       || string_is_empty(key)
+       || !idx)
       return false;
 
    key_hash = core_option_manager_hash_string(key);
@@ -1474,9 +1467,9 @@ bool core_option_manager_get_idx(core_option_manager_t *opt,
    {
       struct core_option *option = &opt->opts[i];
 
-      if ((key_hash == option->key_hash) &&
-          !string_is_empty(option->key) &&
-          string_is_equal(key, option->key))
+      if (   (key_hash == option->key_hash)
+          && !string_is_empty(option->key)
+          &&  string_is_equal(key, option->key))
       {
          *idx = i;
          return true;
@@ -1511,10 +1504,10 @@ bool core_option_manager_get_val_idx(core_option_manager_t *opt,
    uint32_t val_hash;
    size_t i;
 
-   if (!opt ||
-       (idx >= opt->size) ||
-       string_is_empty(val) ||
-       !val_idx)
+   if (   !opt
+       || (idx >= opt->size)
+       || string_is_empty(val)
+       || !val_idx)
       return false;
 
    val_hash = core_option_manager_hash_string(val);
@@ -1560,20 +1553,17 @@ const char *core_option_manager_get_desc(core_option_manager_t *opt,
 {
    const char *desc = NULL;
 
-   if (!opt ||
-       (idx >= opt->size))
+   if (   !opt
+       || (idx >= opt->size))
       return NULL;
-
    /* Try categorised description first,
     * if requested */
    if (categorized)
       desc = opt->opts[idx].desc_categorized;
-
    /* Fall back to legacy description, if
     * required */
    if (string_is_empty(desc))
-      desc = opt->opts[idx].desc;
-
+      return opt->opts[idx].desc;
    return desc;
 }
 
@@ -1600,20 +1590,18 @@ const char *core_option_manager_get_info(core_option_manager_t *opt,
 {
    const char *info = NULL;
 
-   if (!opt ||
-       (idx >= opt->size))
+   if (   !opt
+       || (idx >= opt->size))
       return NULL;
 
    /* Try categorised information first,
     * if requested */
    if (categorized)
       info = opt->opts[idx].info_categorized;
-
    /* Fall back to legacy information, if
     * required */
    if (string_is_empty(info))
-      info = opt->opts[idx].info;
-
+      return opt->opts[idx].info;
    return info;
 }
 
@@ -1634,8 +1622,8 @@ const char *core_option_manager_get_val(core_option_manager_t *opt,
 {
    struct core_option *option = NULL;
 
-   if (!opt ||
-       (idx >= opt->size))
+   if (   !opt
+       || (idx >= opt->size))
       return NULL;
 
    option = (struct core_option*)&opt->opts[idx];
@@ -1661,8 +1649,8 @@ const char *core_option_manager_get_val_label(core_option_manager_t *opt,
 {
    struct core_option *option = NULL;
 
-   if (!opt ||
-       (idx >= opt->size))
+   if (   !opt 
+       || (idx >= opt->size))
       return NULL;
 
    option = (struct core_option*)&opt->opts[idx];
@@ -1739,9 +1727,9 @@ void core_option_manager_set_val(core_option_manager_t *opt,
    if (retroarch_ctl(RARCH_CTL_CORE_OPTION_UPDATE_DISPLAY, NULL) &&
        refresh_menu)
    {
-      bool refresh = false;
-      menu_entries_ctl(MENU_ENTRIES_CTL_SET_REFRESH, &refresh);
-      menu_driver_ctl(RARCH_MENU_CTL_SET_PREVENT_POPULATE, NULL);
+      struct menu_state *menu_st = menu_state_get_ptr();
+      menu_st->flags            |=  MENU_ST_FLAG_ENTRIES_NEED_REFRESH
+                                 |  MENU_ST_FLAG_PREVENT_POPULATE;
    }
 #endif
 }
@@ -1772,8 +1760,8 @@ void core_option_manager_adjust_val(core_option_manager_t* opt,
 {
    struct core_option* option = NULL;
 
-   if (!opt ||
-       (idx >= opt->size))
+   if (   !opt
+       || (idx >= opt->size))
       return;
 
    option        = (struct core_option*)&opt->opts[idx];
@@ -1791,9 +1779,9 @@ void core_option_manager_adjust_val(core_option_manager_t* opt,
    if (retroarch_ctl(RARCH_CTL_CORE_OPTION_UPDATE_DISPLAY, NULL) &&
        refresh_menu)
    {
-      bool refresh = false;
-      menu_entries_ctl(MENU_ENTRIES_CTL_SET_REFRESH, &refresh);
-      menu_driver_ctl(RARCH_MENU_CTL_SET_PREVENT_POPULATE, NULL);
+      struct menu_state *menu_st = menu_state_get_ptr();
+      menu_st->flags            |=  MENU_ST_FLAG_ENTRIES_NEED_REFRESH
+                                 |  MENU_ST_FLAG_PREVENT_POPULATE;
    }
 #endif
 }
@@ -1819,8 +1807,8 @@ void core_option_manager_adjust_val(core_option_manager_t* opt,
 void core_option_manager_set_default(core_option_manager_t *opt,
       size_t idx, bool refresh_menu)
 {
-   if (!opt ||
-       (idx >= opt->size))
+   if (   !opt
+       || (idx >= opt->size))
       return;
 
    opt->opts[idx].index = opt->opts[idx].default_index;
@@ -1837,9 +1825,9 @@ void core_option_manager_set_default(core_option_manager_t *opt,
    if (retroarch_ctl(RARCH_CTL_CORE_OPTION_UPDATE_DISPLAY, NULL) &&
        refresh_menu)
    {
-      bool refresh = false;
-      menu_entries_ctl(MENU_ENTRIES_CTL_SET_REFRESH, &refresh);
-      menu_driver_ctl(RARCH_MENU_CTL_SET_PREVENT_POPULATE, NULL);
+      struct menu_state *menu_st = menu_state_get_ptr();
+      menu_st->flags            |=  MENU_ST_FLAG_ENTRIES_NEED_REFRESH
+                                 |  MENU_ST_FLAG_PREVENT_POPULATE;
    }
 #endif
 }
