@@ -41,11 +41,11 @@
 #endif
 
 #include "../frontend_driver.h"
-#include "../../verbosity.h"
+#include "../../file_path_special.h"
 #include "../../defaults.h"
 #include "../../paths.h"
-#include "retroarch.h"
-#include "file_path_special.h"
+#include "../../retroarch.h"
+#include "../../verbosity.h"
 
 #include "ctr/ctr_debug.h"
 #include "ctr/exec-3dsx/exec_3dsx.h"
@@ -82,8 +82,8 @@ static void get_first_valid_core(char* path_return, size_t len)
          if (strlen(ent->d_name) > strlen(extension) 
                && !strcmp(ent->d_name + strlen(ent->d_name) - strlen(extension), extension))
          {
-            strlcpy(path_return, "sdmc:/retroarch/cores/", len);
-            strlcat(path_return, ent->d_name, len);
+            size_t _len = strlcpy(path_return, "sdmc:/retroarch/cores/", len);
+            strlcpy(path_return + _len, ent->d_name, len - _len);
             break;
          }
       }
@@ -377,7 +377,7 @@ u8* gfxBottomFramebuffers[2];
 
 void gfxSetFramebufferInfo(gfxScreen_t screen, u8 id)
 {
-   if(screen==GFX_TOP)
+   if (screen==GFX_TOP)
    {
       u8 enable3d = 0;
       u8 bit5=(enable3d != 0);
@@ -387,7 +387,9 @@ void gfxSetFramebufferInfo(gfxScreen_t screen, u8 id)
                        enable3d ? (u32*)gfxTopRightFramebuffers[id] : (u32*)gfxTopLeftFramebuffers[id],
                        240 * 3,
                        ((1)<<8)|((1^bit5)<<6)|((bit5)<<5)|GSP_BGR8_OES);
-   } else {
+   }
+   else
+   {
       gspPresentBuffer(GFX_BOTTOM,
                        id,
                        (u32*)gfxBottomFramebuffers[id],
