@@ -22,7 +22,7 @@
 
 #include <sys/utsname.h>
 
-#include <mach/mach_host.h>
+#include <mach/mach.h>
 
 #include <CoreFoundation/CoreFoundation.h>
 #include <CoreFoundation/CFArray.h>
@@ -49,7 +49,6 @@
 
 #include <boolean.h>
 #include <compat/apple_compat.h>
-#include <retro_assert.h>
 #include <retro_miscellaneous.h>
 #include <file/file_path.h>
 #include <streams/file_stream.h>
@@ -70,10 +69,6 @@
 #include "../../ui/ui_companion_driver.h"
 #include "../../paths.h"
 
-#if 1
-#define RELEASE_BUILD
-#endif
-
 typedef enum
 {
    CFApplicationDirectory           = 1,   /* Supported applications (Applications) */
@@ -128,25 +123,23 @@ static void CFSearchPathForDirectoriesInDomains(
 #else
    NSSearchPathDirectory dir = NSDocumentDirectory;
 #endif
-   CFStringRef array_val;
 #if __has_feature(objc_arc)
-   array_val = (__bridge CFStringRef)[
+   CFStringRef array_val     = (__bridge CFStringRef)[
          NSSearchPathForDirectoriesInDomains(dir,
             NSUserDomainMask, YES) firstObject];
 #else
-   NSArray *arr = NSSearchPathForDirectoriesInDomains(dir,
-						     NSUserDomainMask, YES);
-   if ([arr count] == 0) {
-     array_val = nil;
-   } else{
-     array_val = (CFStringRef)[arr objectAtIndex:0];
-   }
+   CFStringRef array_val     = nil;
+   NSArray *arr              = 
+      NSSearchPathForDirectoriesInDomains(dir,
+            NSUserDomainMask, YES);
+   if ([arr count] != 0)
+      array_val              = (CFStringRef)[arr objectAtIndex:0];
 #endif
    if (array_val)
       CFStringGetCString(array_val, s, len, kCFStringEncodingUTF8);
 }
 
-static void CFTemporaryDirectory(char *s, size_t len)
+void CFTemporaryDirectory(char *s, size_t len)
 {
 #if __has_feature(objc_arc)
    CFStringRef path = (__bridge CFStringRef)NSTemporaryDirectory();
@@ -165,9 +158,15 @@ void get_ios_version(int *major, int *minor);
 #define PMGMT_STRMATCH(a,b) (CFStringCompare(a, b, 0) == kCFCompareEqualTo)
 #define PMGMT_GETVAL(k,v)   CFDictionaryGetValueIfPresent(dict, CFSTR(k), (const void **) v)
 
-/* Note that AC power sources also include a laptop battery it is charging. */
-static void checkps(CFDictionaryRef dict, bool * have_ac, bool * have_battery,
-      bool * charging, int *seconds, int *percent)
+/* Note that AC power sources also include a 
+ * laptop battery it is charging. */
+static void darwin_check_power_source(
+      CFDictionaryRef dict,
+      bool *have_ac,
+      bool *have_battery,
+      bool *charging,
+      int  *seconds,
+      int  *percent)
 {
    CFStringRef strval; /* don't CFRelease() this. */
    CFBooleanRef bval;
@@ -188,7 +187,7 @@ static void checkps(CFDictionaryRef dict, bool * have_ac, bool * have_battery,
    if (PMGMT_STRMATCH(strval, CFSTR(kIOPSACPowerValue)))
       is_ac = *have_ac = true;
    else if (!PMGMT_STRMATCH(strval, CFSTR(kIOPSBatteryPowerValue)))
-      return;                 /* not a battery? */
+      return; /* Not a battery? */
 
    if ((PMGMT_GETVAL(kIOPSIsChargingKey, &bval)) && (bval == kCFBooleanTrue))
       charge = true;
@@ -200,7 +199,7 @@ static void checkps(CFDictionaryRef dict, bool * have_ac, bool * have_battery,
       if (val > 0)
       {
          *have_battery = true;
-         maxpct        = (int) val;
+         maxpct        = (int)val;
       }
    }
 
@@ -211,7 +210,7 @@ static void checkps(CFDictionaryRef dict, bool * have_ac, bool * have_battery,
       if (val > 0)
       {
          *have_battery = true;
-         maxpct        = (int) val;
+         maxpct        = (int)val;
       }
    }
 
@@ -221,12 +220,12 @@ static void checkps(CFDictionaryRef dict, bool * have_ac, bool * have_battery,
       CFNumberGetValue(numval, kCFNumberSInt32Type, &val);
 
       /* Mac OS X reports 0 minutes until empty if you're plugged in. :( */
-      if ((val == 0) && (is_ac))
-         val = -1;           /* !!! FIXME: calc from timeToFull and capacity? */
+      if ((val == 0) && is_ac)
+         val = -1; /* !!! FIXME: calc from timeToFull and capacity? */
 
-      secs = (int) val;
+      secs = (int)val;
       if (secs > 0)
-         secs *= 60;         /* value is in minutes, so convert to seconds. */
+         secs *= 60; /* value is in minutes, so convert to seconds. */
    }
 
    if (PMGMT_GETVAL(kIOPSCurrentCapacityKey, &numval))
@@ -269,11 +268,8 @@ static void frontend_darwin_get_name(char *s, size_t len)
 {
 #if defined(IOS)
    struct utsname buffer;
-
-   if (uname(&buffer) != 0)
-      return;
-
-   strlcpy(s, buffer.machine, len);
+   if (uname(&buffer) == 0)
+      strlcpy(s, buffer.machine, len);
 #elif defined(OSX)
    size_t length = 0;
    sysctlbyname("hw.model", NULL, &length, NULL, 0);
@@ -286,13 +282,21 @@ static void frontend_darwin_get_os(char *s, size_t len, int *major, int *minor)
 {
 #if defined(IOS)
    get_ios_version(major, minor);
+#if TARGET_OS_TV
+   s[0] = 't';
+   s[1] = 'v';
+   s[2] = 'O';
+   s[3] = 'S';
+   s[4] = '\0';
+#else
    s[0] = 'i';
    s[1] = 'O';
    s[2] = 'S';
    s[3] = '\0';
+#endif
 #elif defined(OSX)
 
-#if MAC_OS_X_VERSION_MIN_REQUIRED >= 101300 // MAC_OS_X_VERSION_10_13
+#if MAC_OS_X_VERSION_MIN_REQUIRED >= 101300 /* MAC_OS_X_VERSION_10_13 */
    NSOperatingSystemVersion version = NSProcessInfo.processInfo.operatingSystemVersion;
    *major = (int)version.majorVersion;
    *minor = (int)version.minorVersion;
@@ -335,143 +339,115 @@ static void frontend_darwin_get_env(int *argc, char *argv[],
    char assets_zip_path[PATH_MAX_LENGTH];
    CFURLRef bundle_url;
    CFStringRef bundle_path;
-   CFURLRef resource_url;
-   CFStringRef resource_path;
-#if TARGET_OS_IPHONE
-   char resolved_home_dir_buf[
-      PATH_MAX_LENGTH]                   = {0};
-   char resolved_bundle_dir_buf[
-      PATH_MAX_LENGTH]                   = {0};
-#endif
-   char temp_dir[PATH_MAX_LENGTH]        = {0};
-   char bundle_path_buf[PATH_MAX_LENGTH] = {0};
-   char resource_path_buf[PATH_MAX_LENGTH] = {0};
-   char full_resource_path_buf[PATH_MAX_LENGTH];
-   char home_dir_buf[PATH_MAX_LENGTH]    = {0};
-   CFBundleRef bundle                    = CFBundleGetMainBundle();
+   char temp_dir[PATH_MAX_LENGTH]          = {0};
+   char bundle_path_buf[PATH_MAX_LENGTH]   = {0};
+   char documents_dir_buf[PATH_MAX_LENGTH] = {0};
+   char application_data[PATH_MAX_LENGTH]  = {0};
+   CFBundleRef bundle                      = CFBundleGetMainBundle();
 
    if (!bundle)
       return;
 
-   bundle_url  = CFBundleCopyBundleURL(bundle);
-   bundle_path = CFURLCopyPath(bundle_url);
-   
-   resource_url = CFBundleCopyResourcesDirectoryURL(bundle);
-   resource_path = CFURLCopyPath(resource_url);
+   bundle_url    = CFBundleCopyBundleURL(bundle);
+   bundle_path   = CFURLCopyFileSystemPath(bundle_url, kCFURLPOSIXPathStyle);
+   CFStringGetCString(bundle_path, bundle_path_buf, sizeof(bundle_path_buf), kCFStringEncodingUTF8);
+   CFRelease(bundle_path);
+   CFRelease(bundle_url);
+   path_resolve_realpath(bundle_path_buf, sizeof(bundle_path_buf), true);
 
-   CFRelease(resource_url);
-
-   CFStringGetCString(bundle_path,
-         bundle_path_buf, sizeof(bundle_path_buf), kCFStringEncodingUTF8);
-   CFStringGetCString(resource_path,
-         resource_path_buf, sizeof(resource_path_buf), kCFStringEncodingUTF8);
-   CFRelease(resource_path);
-   fill_pathname_join_special(full_resource_path_buf, bundle_path_buf, resource_path_buf, sizeof(full_resource_path_buf));
-   CFSearchPathForDirectoriesInDomains(
-         home_dir_buf, sizeof(home_dir_buf));
-
-#if TARGET_OS_IPHONE
-   if (realpath(home_dir_buf, resolved_home_dir_buf))
-   {
-      retro_assert(strlcpy(home_dir_buf,
-               resolved_home_dir_buf,
-               sizeof(home_dir_buf)) < sizeof(home_dir_buf));
-   }
-   if (realpath(bundle_path_buf, resolved_bundle_dir_buf))
-   {
-      retro_assert(strlcpy(bundle_path_buf,
-               resolved_bundle_dir_buf,
-               sizeof(bundle_path_buf)) < sizeof(bundle_path_buf));
-   }
-#endif
-
-   strlcat(home_dir_buf, "/RetroArch", sizeof(home_dir_buf));
-#ifdef HAVE_METAL
-   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_SHADER],
-                      home_dir_buf, "shaders_slang",
-                      sizeof(g_defaults.dirs[DEFAULT_DIR_SHADER]));
-#else
-   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_SHADER],
-         home_dir_buf, "shaders_glsl",
-         sizeof(g_defaults.dirs[DEFAULT_DIR_SHADER]));
-#endif
-#ifdef HAVE_UPDATE_CORES
-    fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_CORE],
-		    home_dir_buf, "cores", sizeof(g_defaults.dirs[DEFAULT_DIR_CORE]));
-#else
-    fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_CORE],
-		    bundle_path_buf, "modules", sizeof(g_defaults.dirs[DEFAULT_DIR_CORE]));
-#endif
-   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_CORE_INFO], home_dir_buf, "info", sizeof(g_defaults.dirs[DEFAULT_DIR_CORE_INFO]));
-   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_OVERLAY], home_dir_buf, "overlays", sizeof(g_defaults.dirs[DEFAULT_DIR_OVERLAY]));
-#ifdef HAVE_VIDEO_LAYOUT
-   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_VIDEO_LAYOUT], home_dir_buf, "layouts", sizeof(g_defaults.dirs[DEFAULT_DIR_VIDEO_LAYOUT]));
-#endif
-   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_AUTOCONFIG], home_dir_buf, "autoconfig", sizeof(g_defaults.dirs[DEFAULT_DIR_AUTOCONFIG]));
-   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_CORE_ASSETS], home_dir_buf, "downloads", sizeof(g_defaults.dirs[DEFAULT_DIR_CORE_ASSETS]));
-   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_ASSETS], home_dir_buf, "assets", sizeof(g_defaults.dirs[DEFAULT_DIR_ASSETS]));
-   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_SYSTEM], home_dir_buf, "system", sizeof(g_defaults.dirs[DEFAULT_DIR_SYSTEM]));
-   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_MENU_CONFIG], home_dir_buf, "config", sizeof(g_defaults.dirs[DEFAULT_DIR_MENU_CONFIG]));
-   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_REMAP], g_defaults.dirs[DEFAULT_DIR_MENU_CONFIG], "remaps", sizeof(g_defaults.dirs[DEFAULT_DIR_REMAP]));
-   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_DATABASE], home_dir_buf, "database/rdb", sizeof(g_defaults.dirs[DEFAULT_DIR_DATABASE]));
-   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_CHEATS], home_dir_buf, "cht", sizeof(g_defaults.dirs[DEFAULT_DIR_CHEATS]));
-   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_THUMBNAILS], home_dir_buf, "thumbnails", sizeof(g_defaults.dirs[DEFAULT_DIR_THUMBNAILS]));
-   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_SRAM], home_dir_buf, "saves", sizeof(g_defaults.dirs[DEFAULT_DIR_SRAM]));
-   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_SAVESTATE], home_dir_buf, "states", sizeof(g_defaults.dirs[DEFAULT_DIR_SAVESTATE]));
-   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_RECORD_CONFIG], home_dir_buf, "records_config", sizeof(g_defaults.dirs[DEFAULT_DIR_RECORD_CONFIG]));
-   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_RECORD_OUTPUT], home_dir_buf, "records", sizeof(g_defaults.dirs[DEFAULT_DIR_RECORD_OUTPUT]));
-   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_LOGS], home_dir_buf, "logs", sizeof(g_defaults.dirs[DEFAULT_DIR_LOGS]));
-#if defined(IOS)
-   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_PLAYLIST], home_dir_buf, "playlists", sizeof(g_defaults.dirs[DEFAULT_DIR_PLAYLIST]));
-#endif
 #if defined(OSX)
-   char application_data[PATH_MAX_LENGTH];
-
    fill_pathname_application_data(application_data, sizeof(application_data));
 
-#ifdef HAVE_CG
-   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_SHADER], home_dir_buf, "shaders_cg", sizeof(g_defaults.dirs[DEFAULT_DIR_SHADER]));
+   BOOL portable; /* steam || RAPortableInstall || portable.txt */
+#if HAVE_STEAM
+   /* For Steam, we're going to put everything next to the .app */
+   portable = YES;
+#else
+   portable = [[[NSBundle mainBundle] objectForInfoDictionaryKey:@"RAPortableInstall"] boolValue];
+   if (!portable)
+   {
+      char portable_buf[PATH_MAX_LENGTH] = {0};
+      fill_pathname_join(portable_buf, application_data, "portable.txt", sizeof(portable_buf));
+      portable = path_is_valid(portable_buf);
+   }
 #endif
-   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_PLAYLIST], application_data, "playlists", sizeof(g_defaults.dirs[DEFAULT_DIR_PLAYLIST]));
-   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_THUMBNAILS], application_data, "thumbnails", sizeof(g_defaults.dirs[DEFAULT_DIR_THUMBNAILS]));
+   if (portable)
+      strncpy(documents_dir_buf, application_data, sizeof(documents_dir_buf));
+   else
+   {
+      CFSearchPathForDirectoriesInDomains(documents_dir_buf, sizeof(documents_dir_buf));
+      path_resolve_realpath(documents_dir_buf, sizeof(documents_dir_buf), true);
+      strlcat(documents_dir_buf, "/RetroArch", sizeof(documents_dir_buf));
+   }
+#else
+   CFSearchPathForDirectoriesInDomains(documents_dir_buf, sizeof(documents_dir_buf));
+   path_resolve_realpath(documents_dir_buf, sizeof(documents_dir_buf), true);
+   strlcat(documents_dir_buf, "/RetroArch", sizeof(documents_dir_buf));
+   /* iOS and tvOS are going to put everything in the documents dir */
+   strncpy(application_data, documents_dir_buf, sizeof(application_data));
+#endif
+
+   /* By the time we are here:
+    * bundle_path_buf is the full path of the .app
+    * documents_dir_buf is where user documents go (macos: ~/Documents/RetroArch)
+    * application_data is where "hidden" app data goes (macos: ~/Library/Application Support/RetroArch, ios: documents dir)
+
+    * this stuff we expect the user to find easily, possibly sync across iCloud */
+   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_LOGS], documents_dir_buf, "logs", sizeof(g_defaults.dirs[DEFAULT_DIR_LOGS]));
+   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_PLAYLIST], documents_dir_buf, "playlists", sizeof(g_defaults.dirs[DEFAULT_DIR_PLAYLIST]));
+   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_RECORD_OUTPUT], documents_dir_buf, "records", sizeof(g_defaults.dirs[DEFAULT_DIR_RECORD_OUTPUT]));
+   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_RECORD_CONFIG], documents_dir_buf, "records_config", sizeof(g_defaults.dirs[DEFAULT_DIR_RECORD_CONFIG]));
+   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_SRAM], documents_dir_buf, "saves", sizeof(g_defaults.dirs[DEFAULT_DIR_SRAM]));
+   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_SCREENSHOT], documents_dir_buf, "screenshots", sizeof(g_defaults.dirs[DEFAULT_DIR_SCREENSHOT]));
+   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_SAVESTATE], documents_dir_buf, "states", sizeof(g_defaults.dirs[DEFAULT_DIR_SAVESTATE]));
+   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_SYSTEM], documents_dir_buf, "system", sizeof(g_defaults.dirs[DEFAULT_DIR_SYSTEM]));
+
+   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_ASSETS], application_data, "assets", sizeof(g_defaults.dirs[DEFAULT_DIR_ASSETS]));
+   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_AUTOCONFIG], application_data, "autoconfig", sizeof(g_defaults.dirs[DEFAULT_DIR_AUTOCONFIG]));
+   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_CHEATS], application_data, "cht", sizeof(g_defaults.dirs[DEFAULT_DIR_CHEATS]));
    fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_MENU_CONFIG], application_data, "config", sizeof(g_defaults.dirs[DEFAULT_DIR_MENU_CONFIG]));
    fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_REMAP], g_defaults.dirs[DEFAULT_DIR_MENU_CONFIG], "remaps", sizeof(g_defaults.dirs[DEFAULT_DIR_REMAP]));
-   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_CORE_ASSETS], application_data, "downloads", sizeof(g_defaults.dirs[DEFAULT_DIR_CORE_ASSETS]));
-   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_SCREENSHOT], application_data, "screenshots", sizeof(g_defaults.dirs[DEFAULT_DIR_SCREENSHOT]));
-#if defined(RELEASE_BUILD)
-   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_SHADER], application_data, "shaders", sizeof(g_defaults.dirs[DEFAULT_DIR_SHADER]));
+#if defined(HAVE_UPDATE_CORES) || defined(HAVE_STEAM)
    fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_CORE], application_data, "cores", sizeof(g_defaults.dirs[DEFAULT_DIR_CORE]));
+#else
+   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_CORE], bundle_path_buf, "modules", sizeof(g_defaults.dirs[DEFAULT_DIR_CORE]));
+#endif
+   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_DATABASE], application_data, "database/rdb", sizeof(g_defaults.dirs[DEFAULT_DIR_DATABASE]));
+   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_CORE_ASSETS], application_data, "downloads", sizeof(g_defaults.dirs[DEFAULT_DIR_CORE_ASSETS]));
+   NSURL *url = [[NSBundle mainBundle] URLForResource:nil withExtension:@"dsp" subdirectory:@"filters/audio"];
+   if (url) {
+       strlcpy(g_defaults.dirs[DEFAULT_DIR_AUDIO_FILTER], [[url baseURL] fileSystemRepresentation],  sizeof(g_defaults.dirs[DEFAULT_DIR_AUDIO_FILTER]));
+   } else {
+       fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_AUDIO_FILTER], application_data, "filters/audio", sizeof(g_defaults.dirs[DEFAULT_DIR_AUDIO_FILTER]));
+   }
+   url = [[NSBundle mainBundle] URLForResource:nil withExtension:@"filt" subdirectory:@"filters/video"];
+   if (url) {
+       strlcpy(g_defaults.dirs[DEFAULT_DIR_VIDEO_FILTER], [[url baseURL] fileSystemRepresentation],  sizeof(g_defaults.dirs[DEFAULT_DIR_VIDEO_FILTER]));
+   } else {
+       fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_VIDEO_FILTER], application_data, "filters/video", sizeof(g_defaults.dirs[DEFAULT_DIR_VIDEO_FILTER]));
+   }
    fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_CORE_INFO], application_data, "info", sizeof(g_defaults.dirs[DEFAULT_DIR_CORE_INFO]));
    fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_OVERLAY], application_data, "overlays", sizeof(g_defaults.dirs[DEFAULT_DIR_OVERLAY]));
-#ifdef HAVE_VIDEO_LAYOUT
-   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_VIDEO_LAYOUT], application_data, "layouts", sizeof(g_defaults.dirs[DEFAULT_DIR_VIDEO_LAYOUT]));
-#endif
-   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_AUTOCONFIG], application_data, "autoconfig", sizeof(g_defaults.dirs[DEFAULT_DIR_AUTOCONFIG]));
-   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_ASSETS], application_data, "assets", sizeof(g_defaults.dirs[DEFAULT_DIR_ASSETS]));
-   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_DATABASE], application_data, "database/rdb", sizeof(g_defaults.dirs[DEFAULT_DIR_DATABASE]));
-   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_CHEATS], application_data, "cht", sizeof(g_defaults.dirs[DEFAULT_DIR_CHEATS]));
-   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_AUDIO_FILTER], application_data, "audio_filters", sizeof(g_defaults.dirs[DEFAULT_DIR_AUDIO_FILTER]));
-   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_VIDEO_FILTER], application_data, "video_filters", sizeof(g_defaults.dirs[DEFAULT_DIR_VIDEO_FILTER]));
+   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_OSK_OVERLAY], application_data, "overlays/keyboards", sizeof(g_defaults.dirs[DEFAULT_DIR_OSK_OVERLAY]));
+   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_SHADER], application_data, "shaders", sizeof(g_defaults.dirs[DEFAULT_DIR_SHADER]));
+   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_THUMBNAILS], application_data, "thumbnails", sizeof(g_defaults.dirs[DEFAULT_DIR_THUMBNAILS]));
+
+#if TARGET_OS_IPHONE
+    fill_pathname_join_special(assets_zip_path,
+          bundle_path_buf, "assets.zip", sizeof(assets_zip_path));
 #else
-   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_AUDIO_FILTER], home_dir_buf, "audio_filters", sizeof(g_defaults.dirs[DEFAULT_DIR_AUDIO_FILTER]));
-   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_VIDEO_FILTER], home_dir_buf, "video_filters", sizeof(g_defaults.dirs[DEFAULT_DIR_VIDEO_FILTER]));
-#endif
+    char full_resource_path_buf[PATH_MAX_LENGTH];
+    char resource_path_buf[PATH_MAX_LENGTH] = {0};
+    CFURLRef resource_url     = CFBundleCopyResourcesDirectoryURL(bundle);
+    CFStringRef resource_path = CFURLCopyPath(resource_url);
+    CFStringGetCString(resource_path, resource_path_buf, sizeof(resource_path_buf), kCFStringEncodingUTF8);
+    CFRelease(resource_path);
+    CFRelease(resource_url);
 
-#endif
-
-#if TARGET_OS_IOS
-    {
-       int major, minor;
-       get_ios_version(&major, &minor);
-       if (major > 8)
-          strlcpy(g_defaults.path_buildbot_server_url, "http://buildbot.libretro.com/nightly/apple/ios9/latest/", sizeof(g_defaults.path_buildbot_server_url));
-    }
-#endif
-
-#if TARGET_OS_IOS
-    fill_pathname_join_special(assets_zip_path, bundle_path_buf, "assets.zip", sizeof(assets_zip_path));
-#else
-    fill_pathname_join_special(assets_zip_path, full_resource_path_buf, "assets.zip", sizeof(assets_zip_path));
+    fill_pathname_join_special(full_resource_path_buf,
+          bundle_path_buf, resource_path_buf, sizeof(full_resource_path_buf));
+    fill_pathname_join_special(assets_zip_path,
+          full_resource_path_buf, "assets.zip", sizeof(assets_zip_path));
 #endif
 
     if (path_is_valid(assets_zip_path))
@@ -482,11 +458,7 @@ static void frontend_darwin_get_env(int *argc, char *argv[],
              assets_zip_path);
        configuration_set_string(settings,
              settings->paths.bundle_assets_dst,
-#if TARGET_OS_IOS || TARGET_OS_TV
-             home_dir_buf
-#else
              application_data
-#endif
        );
        /* TODO/FIXME: Just hardcode this for now */
        configuration_set_uint(settings, settings->uints.bundle_assets_extract_version_current, 1);
@@ -497,27 +469,8 @@ static void frontend_darwin_get_env(int *argc, char *argv[],
          temp_dir,
          sizeof(g_defaults.dirs[DEFAULT_DIR_CACHE]));
 
-   path_mkdir(bundle_path_buf);
-
-   if (access(bundle_path_buf, 0755) != 0) { }
-   else
-   {
-      path_mkdir(g_defaults.dirs[DEFAULT_DIR_SYSTEM]);
-
-      if (access(g_defaults.dirs[DEFAULT_DIR_SYSTEM], 0755) != 0) { }
-   }
-
-   CFRelease(bundle_path);
-   CFRelease(bundle_url);
-
-#ifndef IS_SALAMANDER
-   dir_check_defaults("custom.ini");
-#endif
-}
-
-static void frontend_darwin_content_loaded(void)
-{
-   ui_companion_driver_notify_content_loaded();
+   if (!path_is_directory(g_defaults.dirs[DEFAULT_DIR_MENU_CONFIG]))
+      path_mkdir(g_defaults.dirs[DEFAULT_DIR_MENU_CONFIG]);
 }
 
 static int frontend_darwin_get_rating(void)
@@ -630,7 +583,8 @@ static int frontend_darwin_get_rating(void)
    return -1;
 }
 
-static enum frontend_powerstate frontend_darwin_get_powerstate(int *seconds, int *percent)
+static enum frontend_powerstate frontend_darwin_get_powerstate(
+      int *seconds, int *percent)
 {
    enum frontend_powerstate ret = FRONTEND_POWERSTATE_NONE;
 #if defined(OSX)
@@ -643,12 +597,13 @@ static enum frontend_powerstate frontend_darwin_get_powerstate(int *seconds, int
    *percent        = -1;
 
    if (!blob)
-      goto end;
+      return FRONTEND_POWERSTATE_NONE;
 
-   list = IOPSCopyPowerSourcesList(blob);
-
-   if (!list)
-      goto end;
+   if (!(list = IOPSCopyPowerSourcesList(blob)))
+   {
+      CFRelease(blob);
+      return FRONTEND_POWERSTATE_NONE;
+   }
 
    /* don't CFRelease() the list items, or dictionaries! */
    have_ac         = false;
@@ -661,7 +616,7 @@ static enum frontend_powerstate frontend_darwin_get_powerstate(int *seconds, int
       CFTypeRef ps = (CFTypeRef)CFArrayGetValueAtIndex(list, i);
       CFDictionaryRef dict = IOPSGetPowerSourceDescription(blob, ps);
       if (dict)
-         checkps(dict, &have_ac, &have_battery, &charging,
+         darwin_check_power_source(dict, &have_ac, &have_battery, &charging,
                seconds, percent);
    }
 
@@ -675,38 +630,35 @@ static enum frontend_powerstate frontend_darwin_get_powerstate(int *seconds, int
       ret = FRONTEND_POWERSTATE_ON_POWER_SOURCE;
 
    CFRelease(list);
-end:
-   if (blob)
-      CFRelease(blob);
+   CFRelease(blob);
 #elif TARGET_OS_IOS
    float level;
    UIDevice *uidev = [UIDevice currentDevice];
-
-   if (!uidev)
-	   return ret;
-
-   [uidev setBatteryMonitoringEnabled:true];
-
-   switch (uidev.batteryState)
+   if (uidev)
    {
-	   case UIDeviceBatteryStateCharging:
-		   ret = FRONTEND_POWERSTATE_CHARGING;
-		   break;
-	   case UIDeviceBatteryStateFull:
-		   ret = FRONTEND_POWERSTATE_CHARGED;
-		   break;
-	   case UIDeviceBatteryStateUnplugged:
-		   ret = FRONTEND_POWERSTATE_ON_POWER_SOURCE;
-		   break;
-	   case UIDeviceBatteryStateUnknown:
-		   break;
+      [uidev setBatteryMonitoringEnabled:true];
+
+      switch (uidev.batteryState)
+      {
+         case UIDeviceBatteryStateCharging:
+            ret = FRONTEND_POWERSTATE_CHARGING;
+            break;
+         case UIDeviceBatteryStateFull:
+            ret = FRONTEND_POWERSTATE_CHARGED;
+            break;
+         case UIDeviceBatteryStateUnplugged:
+            ret = FRONTEND_POWERSTATE_ON_POWER_SOURCE;
+            break;
+         case UIDeviceBatteryStateUnknown:
+            break;
+      }
+
+      level = uidev.batteryLevel;
+
+      *percent = ((level < 0.0f) ? -1 : ((int)((level * 100) + 0.5f)));
+
+      [uidev setBatteryMonitoringEnabled:false];
    }
-
-   level = uidev.batteryLevel;
-
-   *percent = ((level < 0.0f) ? -1 : ((int)((level * 100) + 0.5f)));
-
-   [uidev setBatteryMonitoringEnabled:false];
 #endif
    return ret;
 }
@@ -792,7 +744,6 @@ static int frontend_darwin_parse_drive_list(void *data, bool load_content)
    CFRelease(bundle_url);
 #endif
 #endif
-
    return ret;
 }
 
@@ -801,15 +752,17 @@ static uint64_t frontend_darwin_get_total_mem(void)
 #if defined(OSX)
     uint64_t size;
     int mib[2]     = { CTL_HW, HW_MEMSIZE };
-    u_int namelen  = sizeof(mib) / sizeof(mib[0]);
+    u_int namelen  = ARRAY_SIZE(mib);
     size_t len     = sizeof(size);
-
-    if (sysctl(mib, namelen, &size, &len, NULL, 0) < 0)
-        return 0;
-    return size;
-#else
-    return 0;
+    if (sysctl(mib, namelen, &size, &len, NULL, 0) >= 0)
+       return size;
+#elif defined(IOS)
+    task_vm_info_data_t vmInfo;
+    mach_msg_type_number_t count = TASK_VM_INFO_COUNT;
+    if (task_info(mach_task_self(), TASK_VM_INFO, (task_info_t) &vmInfo, &count) == KERN_SUCCESS)
+       return vmInfo.resident_size_peak;
 #endif
+    return 0;
 }
 
 static uint64_t frontend_darwin_get_free_mem(void)
@@ -822,21 +775,27 @@ static uint64_t frontend_darwin_get_free_mem(void)
 
     if (KERN_SUCCESS == host_page_size(mach_port, &page_size) &&
         KERN_SUCCESS == host_statistics64(mach_port, HOST_VM_INFO,
-                                          (host_info64_t)&vm_stats, &count))
+           (host_info64_t)&vm_stats, &count))
     {
-
-        long long used_memory = ((int64_t)vm_stats.active_count +
-                                 (int64_t)vm_stats.inactive_count +
-                                 (int64_t)vm_stats.wire_count) *  (int64_t)page_size;
+        long long used_memory = (
+              (int64_t)vm_stats.active_count   +
+              (int64_t)vm_stats.inactive_count +
+              (int64_t)vm_stats.wire_count)    * (int64_t)page_size;
         return used_memory;
     }
+#elif defined(IOS)
+    task_vm_info_data_t vmInfo;
+    mach_msg_type_number_t count = TASK_VM_INFO_COUNT;
+    if (task_info(mach_task_self(), TASK_VM_INFO, (task_info_t) &vmInfo, &count) == KERN_SUCCESS)
+        return vmInfo.resident_size_peak - vmInfo.resident_size;
 #endif
     return 0;
 }
 
 static const char* frontend_darwin_get_cpu_model_name(void)
 {
-   cpu_features_get_model_name(darwin_cpu_model_name, sizeof(darwin_cpu_model_name));
+   cpu_features_get_model_name(darwin_cpu_model_name,
+         sizeof(darwin_cpu_model_name));
    return darwin_cpu_model_name;
 }
 
@@ -865,7 +824,8 @@ static char* accessibility_mac_language_code(const char* language)
       return "Ioana";
    else if (string_is_equal(language,"pt_pt"))
       return "Joana";
-   else if (string_is_equal(language,"pt_bt") || string_is_equal(language,"pt"))
+   else if (string_is_equal(language,"pt_bt") 
+         || string_is_equal(language,"pt"))
       return "Luciana";
    else if (string_is_equal(language,"th"))
       return "Kanya";
@@ -879,7 +839,8 @@ static char* accessibility_mac_language_code(const char* language)
       return "Maged";
    else if (string_is_equal(language,"hu"))
       return "Mariska";
-   else if (string_is_equal(language,"zh_tw") || string_is_equal(language,"zh"))
+   else if (string_is_equal(language,"zh_tw") 
+         || string_is_equal(language,"zh"))
       return "Mei-Jia";
    else if (string_is_equal(language,"el"))
       return "Melina";
@@ -903,8 +864,7 @@ static char* accessibility_mac_language_code(const char* language)
       return "Zosia";
    else if (string_is_equal(language,"cs")) 
       return "Zuzana";
-   else
-      return "";
+   return "";
 }
 
 static bool is_narrator_running_macos(void)
@@ -913,17 +873,16 @@ static bool is_narrator_running_macos(void)
 }
 
 static bool accessibility_speak_macos(int speed,
-      const char* speak_text, int priority)
+      const char* speak_text, int priority, const char* voice)
 {
    int pid;
-   const char *voice      = get_user_language_iso639_1(false);
    char* language_speaker = accessibility_mac_language_code(voice);
-   char* speeds[10]       = {"80", "100", "125", "150", "170", "210", "260", "310", "380", "450"};
-
+   char* speeds[10]       = {"80",  "100", "125", "150", "170", "210",
+                             "260", "310", "380", "450"};
    if (speed < 1)
-      speed = 1;
+      speed               = 1;
    else if (speed > 10)
-      speed = 10;
+      speed               = 10;
 
    if (priority < 10 && speak_pid > 0)
    {
@@ -966,8 +925,8 @@ static bool accessibility_speak_macos(int speed,
       else
       {
          char* cmd[] = {"say", NULL, "-r", NULL,  NULL};
-         cmd[1] = (char*) speak_text;
-         cmd[3] = speeds[speed-1];
+         cmd[1]      = (char*) speak_text;
+         cmd[3]      = speeds[speed-1];
          execvp("say",cmd);
       }
    }
@@ -987,7 +946,7 @@ frontend_ctx_driver_t frontend_ctx_darwin = {
    frontend_darwin_get_name,        /* get_name */
    frontend_darwin_get_os,          /* get_os               */
    frontend_darwin_get_rating,      /* get_rating           */
-   frontend_darwin_content_loaded,  /* content_loaded       */
+   NULL,                            /* content_loaded       */
    frontend_darwin_get_arch,        /* get_architecture     */
    frontend_darwin_get_powerstate,  /* get_powerstate       */
    frontend_darwin_parse_drive_list,/* parse_drive_list     */
