@@ -43,8 +43,8 @@
 #include "../frontend_driver.h"
 #include "../../defaults.h"
 #include "../../file_path_special.h"
-#include "../../verbosity.h"
 #include "../../paths.h"
+#include "../../verbosity.h"
 
 #if defined(DEBUG)
 #define DEFAULT_PARTITION "hdd0:__common:pfs"
@@ -63,9 +63,8 @@ static enum HDD_INIT_STATUS hddStatus        = HDD_INIT_STATUS_UNKNOWN;
 static void create_path_names(void)
 {
    char user_path[FILENAME_MAX];
-
-   strlcpy(user_path, cwd, sizeof(user_path));
-   strlcat(user_path, "retroarch", sizeof(user_path));
+   size_t _len = strlcpy(user_path, cwd, sizeof(user_path));
+   strlcpy(user_path + _len, "/retroarch", sizeof(user_path) - _len);
    fill_pathname_basedir(g_defaults.dirs[DEFAULT_DIR_PORT], cwd, sizeof(g_defaults.dirs[DEFAULT_DIR_PORT]));
 
    /* Content in the same folder */
@@ -100,10 +99,6 @@ static void create_path_names(void)
          "temp", sizeof(g_defaults.dirs[DEFAULT_DIR_CACHE]));
    fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_OVERLAY], user_path,
          "overlays", sizeof(g_defaults.dirs[DEFAULT_DIR_OVERLAY]));
-#ifdef HAVE_VIDEO_LAYOUT
-   fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_VIDEO_LAYOUT], user_path,
-         "layouts", sizeof(g_defaults.dirs[DEFAULT_DIR_VIDEO_LAYOUT]));
-#endif
    fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_THUMBNAILS], user_path,
          "thumbnails", sizeof(g_defaults.dirs[DEFAULT_DIR_THUMBNAILS]));
    fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_LOGS], user_path,
@@ -146,10 +141,8 @@ static void reset_IOP()
 bool getMountInfo(char *path, char *mountString, char *mountPoint, char *newCWD)
 {
    struct string_list *str_list = string_split(path, ":");
-   if (str_list->size < 3 )
-   {
+   if (str_list->size < 3)
       return false;
-   }
 
    sprintf(mountPoint, "%s:%s", str_list->elems[0].data, str_list->elems[1].data);
    sprintf(mountString, "%s:", str_list->elems[2].data);
@@ -222,16 +215,17 @@ static void mount_partition(void)
       return;
    }
 
+   /* If we're booting from HDD, we must update the cwd variable 
+    * and add : to the mount point */
    if (bootDeviceID == BOOT_DEVICE_HDD || bootDeviceID == BOOT_DEVICE_HDD0)
    {
-      /* If we're booting from HDD, we must update the cwd variable and add : to the mount point */
-      strlcpy(cwd, new_cwd, sizeof(cwd));
-      strlcat(mountPoint, ":", sizeof(mountPoint));
+      size_t _len = strlcpy(cwd, new_cwd, sizeof(cwd));
+      strlcpy(mountPoint + _len, ":", sizeof(mountPoint) - _len);
    }
    else
    {
       /* We MUST put mountPoint as empty to avoid wrong results 
-	 with LoadELFFromFileWithPartition */
+         with LoadELFFromFileWithPartition */
       strlcpy(mountPoint, "", sizeof(mountPoint));
    }
 }
@@ -243,7 +237,8 @@ static void deinit_drivers(bool deinit_filesystem, bool deinit_powerOff)
    deinit_joystick_driver(false);
 #endif
 
-   if (deinit_filesystem) {
+   if (deinit_filesystem)
+   {
       umount_hdd_partition(mountString);
 
       deinit_hdd_driver(false);
@@ -305,7 +300,7 @@ static void frontend_ps2_get_env(int *argc, char *argv[],
 
 static void common_init_drivers(bool extra_drivers) 
 {
-   init_drivers(true);
+   init_drivers(extra_drivers);
 
    poweroffSetCallback(&poweroffHandler, NULL);
 
@@ -414,21 +409,61 @@ static void frontend_ps2_exitspawn(char *s, size_t len, char *args)
    frontend_ps2_exec(s, should_load_content);
 }
 
-static int frontend_ps2_get_rating(void) { return 10; }
+static int frontend_ps2_get_rating(void) { return 4; }
 
 enum frontend_architecture frontend_ps2_get_arch(void)
 {
    return FRONTEND_ARCH_MIPS;
 }
 
+static uint64_t frontend_ps2_get_total_mem(void) { return 32*1024*1024; }
+
+/* Crude try-and-fail approach, in lack of a better solution. */
+static uint64_t frontend_ps2_get_free_mem(void)
+{
+  uint64_t free_mem;
+  size_t s0 = 32*1024*1024;
+  void* p1;
+  void* p2;
+  void* p3;
+
+  while (s0 && (p1 = malloc(s0)) == NULL)
+    s0 >>= 1;
+
+  free_mem = s0;
+
+  s0 = 32*1024*1024;
+
+  while (s0 && (p2 = malloc(s0)) == NULL)
+    s0 >>= 1;
+
+  free_mem += s0;
+
+  s0 = 32*1024*1024;
+
+  while (s0 && (p3 = malloc(s0)) == NULL)
+    s0 >>= 1;
+
+  free_mem += s0;
+
+  if (p1)
+    free(p1);
+  if (p2)
+    free(p2);
+  if (p3)
+    free(p3);
+
+  return free_mem;
+}
+
 static int frontend_ps2_parse_drive_list(void *data, bool load_content)
 {
 #ifndef IS_SALAMANDER
    char hdd[10];
    file_list_t *list = (file_list_t*)data;
-   enum msg_hash_enums enum_idx = load_content ?
-      MENU_ENUM_LABEL_FILE_DETECT_CORE_LIST_PUSH_DIR :
-      MENU_ENUM_LABEL_FILE_BROWSER_DIRECTORY;
+   enum msg_hash_enums enum_idx = load_content
+      ? MENU_ENUM_LABEL_FILE_DETECT_CORE_LIST_PUSH_DIR
+      : MENU_ENUM_LABEL_FILE_BROWSER_DIRECTORY;
 
    menu_entries_append(list,
          rootDevicePath(BOOT_DEVICE_MC0),
@@ -450,11 +485,12 @@ static int frontend_ps2_parse_drive_list(void *data, bool load_content)
          msg_hash_to_str(MENU_ENUM_LABEL_FILE_DETECT_CORE_LIST_PUSH_DIR),
          enum_idx,
          FILE_TYPE_DIRECTORY, 0, 0, NULL);
+
    if (hddMountStatus == HDD_MOUNT_STATUS_OK)
    {
-      size_t _len = strlcpy(hdd, mountString, sizeof(hdd));
-      hdd[_len  ] = '/';
-      hdd[_len+1] = '\0';
+      size_t _len  = strlcpy(hdd, mountString, sizeof(hdd));
+      hdd[   _len] = '/';
+      hdd[ ++_len] = '\0';
       menu_entries_append(list,
             hdd,
             msg_hash_to_str(MENU_ENUM_LABEL_FILE_DETECT_CORE_LIST_PUSH_DIR),
@@ -478,12 +514,23 @@ static int frontend_ps2_parse_drive_list(void *data, bool load_content)
    return 0;
 }
 
+static void frontend_ps2_process_args(int *argc, char *argv[])
+{
+#ifndef IS_SALAMANDER
+   /* Make sure active core path is set here. */
+   char path[PATH_MAX_LENGTH] = {0};
+   strlcpy(path, argv[0], sizeof(path));
+   if (path_is_valid(path))
+      path_set(RARCH_PATH_CORE, path);
+#endif
+}
+
 frontend_ctx_driver_t frontend_ctx_ps2 = {
    frontend_ps2_get_env,         /* get_env */
    frontend_ps2_init,            /* init */
    frontend_ps2_deinit,          /* deinit */
    frontend_ps2_exitspawn,       /* exitspawn */
-   NULL,                         /* process_args */
+   frontend_ps2_process_args,    /* process_args */
    frontend_ps2_exec,            /* exec */
 #ifdef IS_SALAMANDER
    NULL,                         /* set_fork */
@@ -498,8 +545,8 @@ frontend_ctx_driver_t frontend_ctx_ps2 = {
    frontend_ps2_get_arch,        /* get_architecture */
    NULL,                         /* get_powerstate */
    frontend_ps2_parse_drive_list,/* parse_drive_list */
-   NULL,                         /* get_total_mem */
-   NULL,                         /* get_free_mem */
+   frontend_ps2_get_total_mem,   /* get_total_mem */
+   frontend_ps2_get_free_mem,    /* get_free_mem */
    NULL,                         /* install_signal_handler */
    NULL,                         /* get_sighandler_state */
    NULL,                         /* set_sighandler_state */
