@@ -26,6 +26,11 @@
 
 #include "wayland_common.h"
 #include "../../frontend/frontend_driver.h"
+#include "../../verbosity.h"
+
+#ifdef HAVE_DBUS
+#include "dbus_common.h"
+#endif
 
 #define SPLASH_SHM_NAME "retroarch-wayland-vk-splash"
 
@@ -39,6 +44,10 @@
 #define DEFAULT_WINDOWED_WIDTH 640
 #define DEFAULT_WINDOWED_HEIGHT 480
 
+/* Icon is 16x15 scaled by 16 */
+#define SPLASH_WINDOW_WIDTH 240
+#define SPLASH_WINDOW_HEIGHT 256
+
 #ifndef MFD_CLOEXEC
 #define MFD_CLOEXEC		0x0001U
 #endif
@@ -55,11 +64,31 @@
 #define F_SEAL_SHRINK		0x0002
 #endif
 
+static const unsigned long retroarch_icon_data[] = {
+0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
+0x00000000,0x00000000,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0x00000000,0x00000000,0x00000000,
+0x00000000,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0x00000000,0x00000000,
+0x00000000,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0x00000000,0x00000000,
+0x00000000,0xff333333,0xff333333,0xff333333,0xfff2f2f2,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xfff2f2f2,0xff333333,0xff333333,0xff333333,0x00000000,0x00000000,
+0x00000000,0xff333333,0xfff2f2f2,0xff333333,0xff333333,0xfff2f2f2,0xff333333,0xff333333,0xff333333,0xfff2f2f2,0xff333333,0xff333333,0xfff2f2f2,0xff333333,0x00000000,0x00000000,
+0x00000000,0xff333333,0xfff2f2f2,0xff333333,0xfff2f2f2,0xfff2f2f2,0xfff2f2f2,0xfff2f2f2,0xfff2f2f2,0xfff2f2f2,0xfff2f2f2,0xff333333,0xfff2f2f2,0xff333333,0x00000000,0x00000000,
+0x00000000,0xff333333,0xfff2f2f2,0xfff2f2f2,0xfff2f2f2,0xff333333,0xfff2f2f2,0xfff2f2f2,0xfff2f2f2,0xff333333,0xfff2f2f2,0xfff2f2f2,0xfff2f2f2,0xff333333,0x00000000,0x00000000,
+0x00000000,0xff333333,0xfff2f2f2,0xfff2f2f2,0xfff2f2f2,0xfff2f2f2,0xfff2f2f2,0xfff2f2f2,0xfff2f2f2,0xfff2f2f2,0xfff2f2f2,0xfff2f2f2,0xfff2f2f2,0xff333333,0x00000000,0x00000000,
+0x00000000,0xff333333,0xff333333,0xfff2f2f2,0xfff2f2f2,0xfff2f2f2,0xfff2f2f2,0xfff2f2f2,0xfff2f2f2,0xfff2f2f2,0xfff2f2f2,0xfff2f2f2,0xff333333,0xff333333,0x00000000,0x00000000,
+0x00000000,0xff333333,0xff333333,0xff333333,0xfff2f2f2,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xfff2f2f2,0xff333333,0xff333333,0xff333333,0x00000000,0x00000000,
+0x00000000,0xff333333,0xff333333,0xfff2f2f2,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xfff2f2f2,0xff333333,0xff333333,0x00000000,0x00000000,
+0x00000000,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0x00000000,0x00000000,
+0x00000000,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0x00000000,0x00000000,
+0x00000000,0x00000000,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0xff333333,0x00000000,0x00000000,0x00000000,
+0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000
+};
+
 void xdg_toplevel_handle_configure_common(gfx_ctx_wayland_data_t *wl,
       void *toplevel,
       int32_t width, int32_t height, struct wl_array *states)
 {
    const uint32_t *state;
+   bool floating              = true;
 
    wl->fullscreen             = false;
    wl->maximized              = false;
@@ -70,36 +99,62 @@ void xdg_toplevel_handle_configure_common(gfx_ctx_wayland_data_t *wl,
       {
          case XDG_TOPLEVEL_STATE_FULLSCREEN:
             wl->fullscreen = true;
+            floating       = false;
             break;
          case XDG_TOPLEVEL_STATE_MAXIMIZED:
-            wl->maximized = true;
+            wl->maximized  = true;
+            /* fall-through */
+         case XDG_TOPLEVEL_STATE_TILED_LEFT:
+         case XDG_TOPLEVEL_STATE_TILED_RIGHT:
+         case XDG_TOPLEVEL_STATE_TILED_TOP:
+         case XDG_TOPLEVEL_STATE_TILED_BOTTOM:
+            floating       = false;
             break;
          case XDG_TOPLEVEL_STATE_RESIZING:
-            wl->resize = true;
+            wl->resize     = true;
             break;
          case XDG_TOPLEVEL_STATE_ACTIVATED:
-            wl->activated = true;
+            wl->activated  = true;
             break;
       }
    }
 
-   wl->width  = width  > 0 ? width  : DEFAULT_WINDOWED_WIDTH;
-   wl->height = height > 0 ? height : DEFAULT_WINDOWED_HEIGHT;
+   if (width == 0 || height == 0)
+   {
+      width  = wl->floating_width;
+      height = wl->floating_height;
+   }
+
+   if (     (width  > 0)
+         && (height > 0))
+   {
+      wl->width         = width;
+      wl->height        = height;
+      wl->buffer_width  = wl->fractional_scale ?
+         FRACTIONAL_SCALE_MULT(wl->width,  wl->fractional_scale_num) : wl->width  * wl->buffer_scale;
+      wl->buffer_height = wl->fractional_scale ?
+         FRACTIONAL_SCALE_MULT(wl->height, wl->fractional_scale_num) : wl->height * wl->buffer_scale;
+      wl->resize        = true;
+      if (wl->viewport) /* Update viewport */
+         wp_viewport_set_destination(wl->viewport, wl->width, wl->height);
+   }
+
+   if (floating)
+   {
+      wl->floating_width  = width;
+      wl->floating_height = height;
+   }
 }
 
 void xdg_toplevel_handle_close(void *data,
-      struct xdg_toplevel *xdg_toplevel)
-{
-   gfx_ctx_wayland_data_t *wl = (gfx_ctx_wayland_data_t*)data;
-   command_event(CMD_EVENT_QUIT, NULL);
-}
+      struct xdg_toplevel *xdg_toplevel) { command_event(CMD_EVENT_QUIT, NULL); }
 
 #ifdef HAVE_LIBDECOR_H
 void libdecor_frame_handle_configure_common(struct libdecor_frame *frame,
       struct libdecor_configuration *configuration,
       gfx_ctx_wayland_data_t *wl)
 {
-   int width, height;
+   int width = 0, height = 0;
    struct libdecor_state *state = NULL;
    enum libdecor_window_state window_state;
 #if 0
@@ -138,57 +193,64 @@ void libdecor_frame_handle_configure_common(struct libdecor_frame *frame,
    if (     width  > 0
          && height > 0)
    {
-      wl->width  = width;
-      wl->height = height;
+      wl->width         = width;
+      wl->height        = height;
+      wl->buffer_width  = wl->fractional_scale ?
+         FRACTIONAL_SCALE_MULT(width,  wl->fractional_scale_num) : width  * wl->buffer_scale;
+      wl->buffer_height = wl->fractional_scale ?
+         FRACTIONAL_SCALE_MULT(height, wl->fractional_scale_num) : height * wl->buffer_scale;
+      wl->resize        = true;
+      if (wl->viewport) /* Update viewport */
+         wp_viewport_set_destination(wl->viewport, wl->width, wl->height);
    }
 
    state = wl->libdecor_state_new(wl->width, wl->height);
    wl->libdecor_frame_commit(frame, state, configuration);
    wl->libdecor_state_free(state);
 
-   if (wl->libdecor_frame_is_floating(frame)) {
+   if (wl->libdecor_frame_is_floating(frame))
+   {
       wl->floating_width  = width;
       wl->floating_height = height;
    }
 }
 
 void libdecor_frame_handle_close(struct libdecor_frame *frame,
-      void *data)
-{
-   gfx_ctx_wayland_data_t *wl = (gfx_ctx_wayland_data_t*)data;
-   command_event(CMD_EVENT_QUIT, NULL);
-}
-
+      void *data) { command_event(CMD_EVENT_QUIT, NULL); }
 void libdecor_frame_handle_commit(struct libdecor_frame *frame,
-      void *data)
-{
-   gfx_ctx_wayland_data_t *wl = (gfx_ctx_wayland_data_t*)data;
-}
+      void *data) { }
 #endif
 
-
-void gfx_ctx_wl_get_video_size_common(gfx_ctx_wayland_data_t *wl,
+void gfx_ctx_wl_get_video_size_common(void *data,
       unsigned *width, unsigned *height)
 {
+   gfx_ctx_wayland_data_t *wl   = (gfx_ctx_wayland_data_t*)data;
+   if (!wl)
+      return;
    if (!wl->reported_display_size)
    {
-      output_info_t *tmp        = NULL;
+      display_output_t *od;
       output_info_t *oi         = wl->current_output;
 
       wl->reported_display_size = true;
 
       /* If window is not ready get any monitor */
       if (!oi)
-          wl_list_for_each_safe(oi, tmp, &wl->all_outputs, link)
-              break;
+         wl_list_for_each(od, &wl->all_outputs, link)
+         {
+            oi = od->output;
+            break;
+         };
 
       *width  = oi->width;
       *height = oi->height;
    }
    else
    {
-      *width  = wl->width  * wl->buffer_scale;
-      *height = wl->height * wl->buffer_scale;
+     *width  = wl->fractional_scale ?
+        FRACTIONAL_SCALE_MULT(wl->width,  wl->pending_fractional_scale_num) : wl->width  * wl->pending_buffer_scale;
+     *height = wl->fractional_scale ?
+        FRACTIONAL_SCALE_MULT(wl->height, wl->pending_fractional_scale_num) : wl->height * wl->pending_buffer_scale;
    }
 }
 
@@ -213,28 +275,67 @@ void gfx_ctx_wl_destroy_resources_common(gfx_ctx_wayland_data_t *wl)
    if (wl->cursor.surface)
       wl_surface_destroy(wl->cursor.surface);
 
-   if (wl->seat)
-      wl_seat_destroy(wl->seat);
-   if (wl->xdg_shell)
-      xdg_wm_base_destroy(wl->xdg_shell);
-   if (wl->compositor)
-      wl_compositor_destroy(wl->compositor);
-   if (wl->registry)
-      wl_registry_destroy(wl->registry);
+   if (wl->viewport)
+      wp_viewport_destroy(wl->viewport);
+   if (wl->fractional_scale)
+      wp_fractional_scale_v1_destroy(wl->fractional_scale);
+   if (wl->idle_inhibitor)
+      zwp_idle_inhibitor_v1_destroy(wl->idle_inhibitor);
+   if (wl->deco)
+      zxdg_toplevel_decoration_v1_destroy(wl->deco);
+   if (wl->xdg_toplevel)
+      xdg_toplevel_destroy(wl->xdg_toplevel);
    if (wl->xdg_surface)
       xdg_surface_destroy(wl->xdg_surface);
    if (wl->surface)
       wl_surface_destroy(wl->surface);
-   if (wl->xdg_toplevel)
-      xdg_toplevel_destroy(wl->xdg_toplevel);
-   if (wl->idle_inhibit_manager)
-      zwp_idle_inhibit_manager_v1_destroy(wl->idle_inhibit_manager);
-   if (wl->deco)
-      zxdg_toplevel_decoration_v1_destroy(wl->deco);
+
    if (wl->deco_manager)
       zxdg_decoration_manager_v1_destroy(wl->deco_manager);
-   if (wl->idle_inhibitor)
-      zwp_idle_inhibitor_v1_destroy(wl->idle_inhibitor);
+   if (wl->idle_inhibit_manager)
+      zwp_idle_inhibit_manager_v1_destroy(wl->idle_inhibit_manager);
+   else
+   {
+#ifdef HAVE_DBUS
+      dbus_screensaver_uninhibit();
+      dbus_close_connection();
+#endif
+   }
+   if (wl->pointer_constraints)
+      zwp_pointer_constraints_v1_destroy(wl->pointer_constraints);
+   if (wl->relative_pointer_manager)
+      zwp_relative_pointer_manager_v1_destroy (wl->relative_pointer_manager);
+   if (wl->seat)
+      wl_seat_destroy(wl->seat);
+   if (wl->xdg_shell)
+      xdg_wm_base_destroy(wl->xdg_shell);
+   if (wl->data_device_manager)
+      wl_data_device_manager_destroy (wl->data_device_manager);
+   while (!wl_list_empty(&wl->current_outputs))
+   {
+      surface_output_t *os = wl_container_of(wl->current_outputs.next, os, link);
+      wl_list_remove(&os->link);
+      free(os);
+   }
+   while (!wl_list_empty(&wl->all_outputs))
+   {
+      display_output_t *od = wl_container_of(wl->all_outputs.next, od, link);
+      output_info_t    *oi = od->output;
+      wl_output_destroy(oi->output);
+      wl_list_remove(&od->link);
+      free(oi);
+      free(od);
+   }
+   if (wl->shm)
+      wl_shm_destroy (wl->shm);
+   if (wl->viewporter)
+      wp_viewporter_destroy(wl->viewporter);
+   if (wl->fractional_scale_manager)
+      wp_fractional_scale_manager_v1_destroy(wl->fractional_scale_manager);
+   if (wl->compositor)
+      wl_compositor_destroy(wl->compositor);
+   if (wl->registry)
+      wl_registry_destroy(wl->registry);
 
    if (wl->input.dpy)
    {
@@ -242,54 +343,76 @@ void gfx_ctx_wl_destroy_resources_common(gfx_ctx_wayland_data_t *wl)
       wl_display_disconnect(wl->input.dpy);
    }
 
-   wl->xdg_shell    = NULL;
-   wl->compositor   = NULL;
-   wl->registry     = NULL;
-   wl->input.dpy    = NULL;
-   wl->xdg_surface  = NULL;
-   wl->surface      = NULL;
-   wl->xdg_toplevel = NULL;
+   wl->input.dpy                = NULL;
+   wl->registry                 = NULL;
+   wl->compositor               = NULL;
+   wl->shm                      = NULL;
+   wl->data_device_manager      = NULL;
+   wl->xdg_shell                = NULL;
+   wl->seat                     = NULL;
+   wl->relative_pointer_manager = NULL;
+   wl->pointer_constraints      = NULL;
+   wl->idle_inhibit_manager     = NULL;
+   wl->deco_manager             = NULL;
+   wl->surface                  = NULL;
+   wl->xdg_surface              = NULL;
+   wl->xdg_toplevel             = NULL;
+   wl->deco                     = NULL;
+   wl->idle_inhibitor           = NULL;
+   wl->wl_touch                 = NULL;
+   wl->wl_pointer               = NULL;
+   wl->wl_keyboard              = NULL;
 
-   wl->width        = 0;
-   wl->height       = 0;
+   wl->width                    = 0;
+   wl->height                   = 0;
+   wl->buffer_width             = 0;
+   wl->buffer_height            = 0;
 }
 
-void gfx_ctx_wl_update_title_common(gfx_ctx_wayland_data_t *wl)
+void gfx_ctx_wl_update_title_common(void *data)
 {
    char title[128];
+   gfx_ctx_wayland_data_t *wl = (gfx_ctx_wayland_data_t*)data;
 
    title[0] = '\0';
 
    video_driver_get_window_title(title, sizeof(title));
 
+   if (wl)
+   {
 #ifdef HAVE_LIBDECOR_H
-   if (wl->libdecor)
-   {
-      if (wl && title[0])
-         wl->libdecor_frame_set_title(wl->libdecor_frame, title);
-   }
-   else
-#endif
-   {
-      if (wl && title[0])
+      if (wl->libdecor)
       {
-         if (wl->deco)
-            zxdg_toplevel_decoration_v1_set_mode(wl->deco,
-               ZXDG_TOPLEVEL_DECORATION_V1_MODE_SERVER_SIDE);
-         xdg_toplevel_set_title(wl->xdg_toplevel, title);
+         if (title[0])
+            wl->libdecor_frame_set_title(wl->libdecor_frame, title);
+      }
+      else
+#endif
+      {
+         if (title[0])
+         {
+            if (wl->deco)
+               zxdg_toplevel_decoration_v1_set_mode(wl->deco,
+                     ZXDG_TOPLEVEL_DECORATION_V1_MODE_SERVER_SIDE);
+            xdg_toplevel_set_title(wl->xdg_toplevel, title);
+         }
       }
    }
 }
 
-bool gfx_ctx_wl_get_metrics_common(gfx_ctx_wayland_data_t *wl,
+bool gfx_ctx_wl_get_metrics_common(void *data,
       enum display_metric_types type, float *value)
 {
-   output_info_t *tmp;
-   output_info_t *oi = wl->current_output;
+   display_output_t *od;
+   gfx_ctx_wayland_data_t *wl = (gfx_ctx_wayland_data_t*)data;
+   output_info_t *oi          = wl ? wl->current_output : NULL;
 
    if (!oi)
-      wl_list_for_each_safe(oi, tmp, &wl->all_outputs, link)
+      wl_list_for_each(od, &wl->all_outputs, link)
+      {
+         oi = od->output;
          break;
+      };
 
    switch (type)
    {
@@ -302,8 +425,8 @@ bool gfx_ctx_wl_get_metrics_common(gfx_ctx_wayland_data_t *wl,
          break;
 
       case DISPLAY_METRIC_DPI:
-         *value = (float)oi->width * 25.4f /
-                  (float)oi->physical_width;
+         *value =   (float)oi->width * 25.4f
+                  / (float)oi->physical_width;
          break;
 
       default:
@@ -316,8 +439,7 @@ bool gfx_ctx_wl_get_metrics_common(gfx_ctx_wayland_data_t *wl,
 
 static int create_shm_file(off_t size)
 {
-   int fd;
-   int ret;
+   int fd, ret;
    if ((fd = syscall(SYS_memfd_create, SPLASH_SHM_NAME,
                MFD_CLOEXEC | MFD_ALLOW_SEALING)) >= 0)
    {
@@ -339,10 +461,9 @@ static int create_shm_file(off_t size)
       for (retry_count = 0; retry_count < 100; retry_count++)
       {
          char *name;
-         if (asprintf (&name, "%s-%02d", SPLASH_SHM_NAME, retry_count) < 0)
+         if (asprintf(&name, "%s-%02d", SPLASH_SHM_NAME, retry_count) < 0)
             continue;
-         fd = shm_open(name, O_RDWR | O_CREAT, 0600);
-         if (fd >= 0)
+         if ((fd = shm_open(name, O_RDWR | O_CREAT, 0600)) >= 0)
          {
             shm_unlink(name);
             free(name);
@@ -403,6 +524,38 @@ static shm_buffer_t *create_shm_buffer(gfx_ctx_wayland_data_t *wl, int width,
    return buffer;
 }
 
+static void shm_buffer_paint_icon(
+      shm_buffer_t *buffer,
+      int width, int height, int scale,
+      size_t icon_scale)
+{
+   int y, x;
+   uint32_t *pixels = buffer->data;
+   int stride       = width * scale;
+
+   for (y = 0; y < height; y++)
+   {
+      for (x = 0; x < width; x++)
+      {
+         uint32_t color = retroarch_icon_data[16 * ((y / icon_scale) % 16) + ((x / icon_scale) % 16)];
+         int sx;
+         if (color >> 4)
+         {
+            for (sx = 0; sx < scale; sx++)
+            {
+               int sy;
+               for (sy = 0; sy < scale; sy++)
+               {
+                  size_t  off = x * scale + sx
+                        + (y * scale + sy) * stride;
+                     pixels[off] = color;
+               }
+            }
+         }
+      }
+   }
+}
+
 static void shm_buffer_paint_checkerboard(
       shm_buffer_t *buffer,
       int width, int height, int scale,
@@ -416,8 +569,8 @@ static void shm_buffer_paint_checkerboard(
    {
       for (x = 0; x < width; x++)
       {
-         int sx;
          uint32_t color = (x & chk) ^ (y & chk) ? fg : bg;
+         int sx;
          for (sx = 0; sx < scale; sx++)
          {
             int sy;
@@ -432,27 +585,28 @@ static void shm_buffer_paint_checkerboard(
    }
 }
 
-
-static bool draw_splash_screen(gfx_ctx_wayland_data_t *wl)
+static bool wl_draw_splash_screen(gfx_ctx_wayland_data_t *wl)
 {
    shm_buffer_t *buffer = create_shm_buffer(wl,
-      wl->width * wl->buffer_scale,
-      wl->height * wl->buffer_scale,
+      wl->buffer_width,
+      wl->buffer_height,
       WL_SHM_FORMAT_XRGB8888);
 
    if (!buffer)
      return false;
 
-   shm_buffer_paint_checkerboard(buffer, wl->width,
-      wl->height, wl->buffer_scale,
-      16, 0xffbcbcbc, 0xff8e8e8e);
+   shm_buffer_paint_checkerboard(buffer, wl->buffer_width,
+      wl->buffer_height, 1,
+      8, 0xffbcbcbc, 0xff8e8e8e);
+   shm_buffer_paint_icon(buffer, wl->buffer_width,
+      wl->buffer_height, 1,
+      16);
 
    wl_surface_attach(wl->surface, buffer->wl_buffer, 0, 0);
-   wl_surface_set_buffer_scale(wl->surface, wl->buffer_scale);
    if (wl_surface_get_version(wl->surface) >= WL_SURFACE_DAMAGE_BUFFER_SINCE_VERSION)
       wl_surface_damage_buffer(wl->surface, 0, 0,
-         wl->width * wl->buffer_scale,
-         wl->height * wl->buffer_scale);
+         wl->buffer_width,
+         wl->buffer_height);
    wl_surface_commit(wl->surface);
    return true;
 }
@@ -461,8 +615,12 @@ bool gfx_ctx_wl_init_common(
       const toplevel_listener_t *toplevel_listener, gfx_ctx_wayland_data_t **wwl)
 {
    int i;
-   *wwl = calloc(1, sizeof(gfx_ctx_wayland_data_t));
-   gfx_ctx_wayland_data_t *wl = *wwl;
+   gfx_ctx_wayland_data_t *wl;
+   settings_t *settings         = config_get_ptr();
+   unsigned video_monitor_index = settings->uints.video_monitor_index;
+
+   *wwl                         = calloc(1, sizeof(gfx_ctx_wayland_data_t));
+   wl                           = *wwl;
 
    if (!wl)
       return false;
@@ -478,14 +636,19 @@ bool gfx_ctx_wl_init_common(
 #endif
 
    wl_list_init(&wl->all_outputs);
+   wl_list_init(&wl->current_outputs);
 
    frontend_driver_destroy_signal_handler_state();
 
-   wl->input.dpy         = wl_display_connect(NULL);
-   wl->last_buffer_scale = 1;
-   wl->buffer_scale      = 1;
-   wl->floating_width    = DEFAULT_WINDOWED_WIDTH;
-   wl->floating_height   = DEFAULT_WINDOWED_HEIGHT;
+   wl->input.dpy                    = wl_display_connect(NULL);
+   wl->last_buffer_scale            = 1;
+   wl->buffer_scale                 = 1;
+   wl->pending_buffer_scale         = 1;
+   wl->last_fractional_scale_num    = FRACTIONAL_SCALE_V1_DEN;
+   wl->fractional_scale_num         = FRACTIONAL_SCALE_V1_DEN;
+   wl->pending_fractional_scale_num = FRACTIONAL_SCALE_V1_DEN;
+   wl->floating_width               = SPLASH_WINDOW_WIDTH;
+   wl->floating_height              = SPLASH_WINDOW_HEIGHT;
 
    if (!wl->input.dpy)
    {
@@ -520,6 +683,9 @@ bool gfx_ctx_wl_init_common(
    if (!wl->idle_inhibit_manager)
    {
       RARCH_LOG("[Wayland]: Compositor doesn't support zwp_idle_inhibit_manager_v1 protocol\n");
+#ifdef HAVE_DBUS
+      dbus_ensure_connection();
+#endif
    }
 
    if (!wl->deco_manager)
@@ -528,28 +694,34 @@ bool gfx_ctx_wl_init_common(
    }
 
    wl->surface = wl_compositor_create_surface(wl->compositor);
+   if (wl->viewporter)
+      wl->viewport = wp_viewporter_get_viewport(wl->viewporter, wl->surface);
+   if (wl->fractional_scale_manager)
+   {
+      wl->fractional_scale = wp_fractional_scale_manager_v1_get_fractional_scale(
+           wl->fractional_scale_manager, wl->surface);
+      wp_fractional_scale_v1_add_listener(wl->fractional_scale, &wp_fractional_scale_v1_listener, wl);
+      RARCH_LOG("[Wayland]: fractional_scale_v1 enabled\n");
+   }
 
-   wl_surface_set_buffer_scale(wl->surface, wl->buffer_scale);
    wl_surface_add_listener(wl->surface, &wl_surface_listener, wl);
 
 #ifdef HAVE_LIBDECOR_H
    if (wl->libdecor)
    {
       wl->libdecor_context = wl->libdecor_new(wl->input.dpy, &libdecor_interface);
-      if (wl->libdecor_context)
-      {
-         wl->libdecor_frame = wl->libdecor_decorate(wl->libdecor_context, wl->surface, &toplevel_listener->libdecor_frame_interface, wl);
-         if (!wl->libdecor_frame)
-         {
-            RARCH_ERR("[Wayland]: Failed to crate libdecor frame\n");
-            goto error;
-         }
 
-         wl->libdecor_frame_set_app_id(wl->libdecor_frame, APP_ID);
-         wl->libdecor_frame_set_title(wl->libdecor_frame, WINDOW_TITLE);
-         wl->libdecor_frame_map(wl->libdecor_frame);
+      wl->libdecor_frame = wl->libdecor_decorate(wl->libdecor_context, wl->surface, &toplevel_listener->libdecor_frame_interface, wl);
+      if (!wl->libdecor_frame)
+      {
+         RARCH_ERR("[Wayland]: Failed to create libdecor frame\n");
+         goto error;
       }
 
+      wl->libdecor_frame_set_app_id(wl->libdecor_frame, APP_ID);
+      wl->libdecor_frame_set_title(wl->libdecor_frame, WINDOW_TITLE);
+      wl->libdecor_frame_map(wl->libdecor_frame);
+
       /* Waiting for libdecor to be configured before starting to draw */
       wl_surface_commit(wl->surface);
       wl->configured = true;
@@ -593,10 +765,29 @@ bool gfx_ctx_wl_init_common(
    /* Bind SHM based wl_buffer to wl_surface until the vulkan surface is ready.
     * This shows the window which assigns us a display (wl_output)
     *  which is usefull for HiDPI and auto selecting a display for fullscreen. */
-   if (!draw_splash_screen(wl))
-      RARCH_ERR("[Wayland`]: Failed to draw splash screen\n");
+   if (video_monitor_index == 0 && wl_list_length (&wl->all_outputs) > 1)
+   {
+      if (!wl_draw_splash_screen(wl))
+         RARCH_ERR("[Wayland]: Failed to draw splash screen\n");
 
-   wl_display_roundtrip(wl->input.dpy);
+      /* Make sure splash screen is on screen and sized */
+#ifdef HAVE_LIBDECOR_H
+      if (wl->libdecor)
+      {
+         wl->configured = true;
+         while (wl->configured)
+            if (wl->libdecor_dispatch(wl->libdecor_context, 0) < 0)
+               RARCH_ERR("[Wayland]: libdecor failed to dispatch\n");
+      }
+      else
+#endif
+      {
+         wl->configured = true;
+
+         while (wl->configured)
+            wl_display_dispatch(wl->input.dpy);
+      }
+   }
 
    wl->input.fd = wl_display_get_fd(wl->input.dpy);
 
@@ -609,7 +800,7 @@ bool gfx_ctx_wl_init_common(
 
    wl->num_active_touches    = 0;
 
-   for (i = 0;i < MAX_TOUCHES;i++)
+   for (i = 0; i < MAX_TOUCHES; i++)
    {
        wl->active_touch_positions[i].active = false;
        wl->active_touch_positions[i].id     = -1;
@@ -625,22 +816,33 @@ error:
    return false;
 }
 
-
 bool gfx_ctx_wl_set_video_mode_common_size(gfx_ctx_wayland_data_t *wl,
-      unsigned width, unsigned height)
+      unsigned width, unsigned height, bool fullscreen)
 {
    settings_t *settings         = config_get_ptr();
    unsigned video_monitor_index = settings->uints.video_monitor_index;
 
-   wl->width                    = width  ? width  : DEFAULT_WINDOWED_WIDTH;
-   wl->height                   = height ? height : DEFAULT_WINDOWED_HEIGHT;
+   wl->width         = width  ? width  : DEFAULT_WINDOWED_WIDTH;
+   wl->height        = height ? height : DEFAULT_WINDOWED_HEIGHT;
+   wl->buffer_width  = wl->width;
+   wl->buffer_height = wl->height;
 
-   wl_surface_set_buffer_scale(wl->surface, wl->buffer_scale);
+   if (!fullscreen)
+   {
+      wl->buffer_scale         = wl->pending_buffer_scale;
+      wl->fractional_scale_num = wl->pending_fractional_scale_num;
+      wl->buffer_width         = wl->fractional_scale ?
+         FRACTIONAL_SCALE_MULT(wl->buffer_width,  wl->fractional_scale_num) : wl->buffer_width  * wl->buffer_scale;
+      wl->buffer_height        = wl->fractional_scale ?
+         FRACTIONAL_SCALE_MULT(wl->buffer_height, wl->fractional_scale_num) : wl->buffer_height * wl->buffer_scale;
+   }
+   if (wl->viewport) /* Update viewport */
+      wp_viewport_set_destination(wl->viewport, wl->width, wl->height);
 
 #ifdef HAVE_LIBDECOR_H
    if (wl->libdecor)
    {
-     struct libdecor_state *state = wl->libdecor_state_new(width, height);
+     struct libdecor_state *state = wl->libdecor_state_new(wl->width, wl->height);
      wl->libdecor_frame_commit(wl->libdecor_frame, state, NULL);
      wl->libdecor_state_free(state);
    }
@@ -657,24 +859,29 @@ bool gfx_ctx_wl_set_video_mode_common_fullscreen(gfx_ctx_wayland_data_t *wl,
 
    if (fullscreen)
    {
-      output_info_t *oi, *tmp;
+      display_output_t *od;
+      output_info_t *oi;
       struct wl_output *output = NULL;
       int output_i             = 0;
 
       if (video_monitor_index <= 0 && wl->current_output != NULL)
       {
-         oi = wl->current_output;
+         oi     = wl->current_output;
          output = oi->output;
          RARCH_LOG("[Wayland]: Auto fullscreen on display \"%s\" \"%s\"\n", oi->make, oi->model);
       }
-      else wl_list_for_each_safe(oi, tmp, &wl->all_outputs, link)
+      else
       {
-         if (++output_i == video_monitor_index)
+         wl_list_for_each(od, &wl->all_outputs, link)
          {
-            output = oi->output;
-            RARCH_LOG("[Wayland]: Fullscreen on display %i \"%s\" \"%s\"\n", output_i, oi->make, oi->model);
-            break;
-         }
+            if (++output_i == (int)video_monitor_index)
+            {
+               oi     = od->output;
+               output = oi->output;
+               RARCH_LOG("[Wayland]: Fullscreen on display %i \"%s\" \"%s\"\n", output_i, oi->make, oi->model);
+               break;
+            }
+         };
       }
 
       if (!output)
@@ -708,33 +915,38 @@ bool gfx_ctx_wl_suppress_screensaver(void *data, bool state)
    gfx_ctx_wayland_data_t *wl = (gfx_ctx_wayland_data_t*)data;
 
    if (!wl->idle_inhibit_manager)
+#ifdef HAVE_DBUS
+      /* Some Wayland compositors (e.g. Phoc) don't implement Wayland's Idle protocol.
+       * They instead rely on things like Gnome Screensaver. */
+      return dbus_suspend_screensaver(state);
+#else
       return false;
-   if (state == (!!wl->idle_inhibitor))
-      return true;
+#endif
+   if (state != (!!wl->idle_inhibitor))
+   {
+      if (state)
+      {
+         RARCH_LOG("[Wayland]: Enabling idle inhibitor\n");
+         struct zwp_idle_inhibit_manager_v1 *mgr = wl->idle_inhibit_manager;
+         wl->idle_inhibitor = zwp_idle_inhibit_manager_v1_create_inhibitor(mgr, wl->surface);
+      }
+      else
+      {
+         RARCH_LOG("[Wayland]: Disabling the idle inhibitor\n");
+         zwp_idle_inhibitor_v1_destroy(wl->idle_inhibitor);
+         wl->idle_inhibitor = NULL;
+      }
+   }
 
-   if (state)
-   {
-      RARCH_LOG("[Wayland]: Enabling idle inhibitor\n");
-      struct zwp_idle_inhibit_manager_v1 *mgr = wl->idle_inhibit_manager;
-      wl->idle_inhibitor = zwp_idle_inhibit_manager_v1_create_inhibitor(mgr, wl->surface);
-   }
-   else
-   {
-      RARCH_LOG("[Wayland]: Disabling the idle inhibitor\n");
-      zwp_idle_inhibitor_v1_destroy(wl->idle_inhibitor);
-      wl->idle_inhibitor = NULL;
-   }
    return true;
 }
 
 float gfx_ctx_wl_get_refresh_rate(void *data)
 {
    gfx_ctx_wayland_data_t *wl = (gfx_ctx_wayland_data_t*)data;
-
    if (!wl || !wl->current_output)
       return false;
-
-   return (float) wl->current_output->refresh_rate / 1000.0f;
+   return (float)wl->current_output->refresh_rate / 1000.0f;
 }
 
 bool gfx_ctx_wl_has_focus(void *data)
@@ -752,19 +964,18 @@ void gfx_ctx_wl_check_window_common(gfx_ctx_wayland_data_t *wl,
 
    flush_wayland_fd(&wl->input);
 
-   new_width  = *width  * wl->last_buffer_scale;
-   new_height = *height * wl->last_buffer_scale;
-
    get_video_size(wl, &new_width, &new_height);
 
-   if (     new_width  != *width  * wl->last_buffer_scale
-         || new_height != *height * wl->last_buffer_scale)
+   if (     wl->pending_buffer_scale != wl->buffer_scale
+         || wl->pending_fractional_scale_num != wl->fractional_scale_num
+         || new_width  != *width
+         || new_height != *height)
    {
-      *width  = new_width;
-      *height = new_height;
-      *resize = true;
-
-      wl->last_buffer_scale = wl->buffer_scale;
+      wl->buffer_scale         = wl->pending_buffer_scale;
+      wl->fractional_scale_num = wl->pending_fractional_scale_num;
+      *width                   = new_width;
+      *height                  = new_height;
+      *resize                  = true;
    }
 
    *quit = (bool)frontend_driver_get_signal_handler_state();
@@ -781,8 +992,8 @@ static void shm_buffer_handle_release(void *data,
 }
 
 #if 0
-static void xdg_surface_handle_configure(void *data, struct xdg_surface *surface,
-                                  uint32_t serial)
+static void xdg_surface_handle_configure(void *data,
+      struct xdg_surface *surface, uint32_t serial)
 {
    xdg_surface_ack_configure(surface, serial);
 }
