@@ -54,7 +54,7 @@
 #include <defines/d3d_defines.h>
 #include "../common/d3d_common.h"
 #include "../common/d3d9_common.h"
-#include "../video_coord_array.h"
+#include "../video_driver.h"
 #include "../../configuration.h"
 #include "../../dynamic.h"
 #include "../../frontend/frontend_driver.h"
@@ -91,6 +91,306 @@ typedef struct hlsl_renderchain
    struct shader_pass stock_shader;
 } hlsl_renderchain_t;
 
+/*
+ * DISPLAY DRIVER
+ */
+
+static const float d3d9_hlsl_vertexes[8] = {
+   0, 0,
+   1, 0,
+   0, 1,
+   1, 1
+};
+
+static const float d3d9_hlsl_tex_coords[8] = {
+   0, 1,
+   1, 1,
+   0, 0,
+   1, 0
+};
+
+static const float *gfx_display_d3d9_hlsl_get_default_vertices(void)
+{
+   return &d3d9_hlsl_vertexes[0];
+}
+
+static const float *gfx_display_d3d9_hlsl_get_default_tex_coords(void)
+{
+   return &d3d9_hlsl_tex_coords[0];
+}
+
+static void *gfx_display_d3d9_hlsl_get_default_mvp(void *data)
+{
+   static float id[16] =       { 1.0f, 0.0f, 0.0f, 0.0f,
+                                 0.0f, 1.0f, 0.0f, 0.0f,
+                                 0.0f, 0.0f, 1.0f, 0.0f,
+                                 0.0f, 0.0f, 0.0f, 1.0f
+                               };
+   return &id;
+}
+
+static INT32 gfx_display_prim_to_d3d9_hlsl_enum(
+      enum gfx_display_prim_type prim_type)
+{
+   switch (prim_type)
+   {
+      case GFX_DISPLAY_PRIM_TRIANGLES:
+      case GFX_DISPLAY_PRIM_TRIANGLESTRIP:
+         return D3DPT_COMM_TRIANGLESTRIP;
+      case GFX_DISPLAY_PRIM_NONE:
+      default:
+         break;
+   }
+
+   /* TODO/FIXME - hack */
+   return 0;
+}
+
+static void gfx_display_d3d9_hlsl_blend_begin(void *data)
+{
+   d3d9_video_t *d3d = (d3d9_video_t*)data;
+
+   if (!d3d)
+      return;
+
+   IDirect3DDevice9_SetRenderState(d3d->dev, D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
+   IDirect3DDevice9_SetRenderState(d3d->dev, D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
+   IDirect3DDevice9_SetRenderState(d3d->dev, D3DRS_ALPHABLENDENABLE, true);
+}
+
+static void gfx_display_d3d9_hlsl_blend_end(void *data)
+{
+   d3d9_video_t *d3d = (d3d9_video_t*)data;
+
+   if (d3d)
+      IDirect3DDevice9_SetRenderState(d3d->dev, D3DRS_ALPHABLENDENABLE, false);
+}
+
+static void gfx_display_d3d9_bind_texture(gfx_display_ctx_draw_t *draw,
+      d3d9_video_t *d3d)
+{
+   LPDIRECT3DDEVICE9 dev = d3d->dev;
+
+   IDirect3DDevice9_SetTexture(dev, 0,
+         (IDirect3DBaseTexture9*)draw->texture);
+   IDirect3DDevice9_SetSamplerState(dev,
+         0, D3DSAMP_ADDRESSU, D3DTADDRESS_COMM_CLAMP);
+   IDirect3DDevice9_SetSamplerState(dev,
+         0, D3DSAMP_ADDRESSV, D3DTADDRESS_COMM_CLAMP);
+   IDirect3DDevice9_SetSamplerState(dev,
+         0, D3DSAMP_MINFILTER, D3DTEXF_COMM_LINEAR);
+   IDirect3DDevice9_SetSamplerState(dev,
+         0, D3DSAMP_MAGFILTER, D3DTEXF_COMM_LINEAR);
+   IDirect3DDevice9_SetSamplerState(dev, 0,
+         D3DSAMP_MIPFILTER, D3DTEXF_COMM_LINEAR);
+}
+
+static void gfx_display_d3d9_hlsl_draw(gfx_display_ctx_draw_t *draw,
+      void *data, unsigned video_width, unsigned video_height)
+{
+   unsigned i;
+   math_matrix_4x4 mop, m1, m2;
+   LPDIRECT3DDEVICE9 dev;
+   D3DPRIMITIVETYPE type;
+   unsigned start                = 0;
+   unsigned count                = 0;
+   d3d9_video_t *d3d             = (d3d9_video_t*)data;
+   Vertex * pv                   = NULL;
+   const float *vertex           = NULL;
+   const float *tex_coord        = NULL;
+   const float *color            = NULL;
+
+   if (!d3d || !draw || draw->pipeline_id)
+      return;
+
+   dev                           = d3d->dev;
+
+   if ((d3d->menu_display.offset + draw->coords->vertices )
+         > (unsigned)d3d->menu_display.size)
+      return;
+
+   IDirect3DVertexBuffer9_Lock((LPDIRECT3DVERTEXBUFFER9)
+            d3d->menu_display.buffer, 0, 0, (void**)&pv, 0);
+   if (!pv)
+      return;
+
+   pv          += d3d->menu_display.offset;
+   vertex       = draw->coords->vertex;
+   tex_coord    = draw->coords->tex_coord;
+   color        = draw->coords->color;
+
+   if (!vertex)
+      vertex    = &d3d9_hlsl_vertexes[0];
+   if (!tex_coord)
+      tex_coord = &d3d9_hlsl_tex_coords[0];
+
+   for (i = 0; i < draw->coords->vertices; i++)
+   {
+      int colors[4];
+
+      colors[0]   = *color++ * 0xFF;
+      colors[1]   = *color++ * 0xFF;
+      colors[2]   = *color++ * 0xFF;
+      colors[3]   = *color++ * 0xFF;
+
+      pv[i].x     = *vertex++;
+      pv[i].y     = *vertex++;
+      pv[i].z     = 0.5f;
+      pv[i].u     = *tex_coord++;
+      pv[i].v     = *tex_coord++;
+
+      pv[i].color =
+         D3DCOLOR_ARGB(
+               colors[3], /* A */
+               colors[0], /* R */
+               colors[1], /* G */
+               colors[2]  /* B */
+               );
+   }
+   IDirect3DVertexBuffer9_Unlock((LPDIRECT3DVERTEXBUFFER9)
+         d3d->menu_display.buffer);
+
+   if (!draw->matrix_data)
+      draw->matrix_data = gfx_display_d3d9_hlsl_get_default_mvp(d3d);
+
+   /* ugh */
+   matrix_4x4_scale(m1,       2.0,  2.0, 0);
+   matrix_4x4_translate(mop, -1.0, -1.0, 0);
+   matrix_4x4_multiply(m2, mop, m1);
+   matrix_4x4_multiply(m1,
+         *((math_matrix_4x4*)draw->matrix_data), m2);
+   matrix_4x4_scale(mop,
+         (draw->width  / 2.0) / video_width,
+         (draw->height / 2.0) / video_height, 0);
+   matrix_4x4_multiply(m2, mop, m1);
+   matrix_4x4_translate(mop,
+         (draw->x + (draw->width  / 2.0)) / video_width,
+         (draw->y + (draw->height / 2.0)) / video_height,
+         0);
+   matrix_4x4_multiply(m1, mop, m2);
+   matrix_4x4_multiply(m2, d3d->mvp_transposed, m1);
+
+   IDirect3DDevice9_SetVertexShaderConstantF(d3d->dev,
+         0, (const float*)&m2, 4);
+
+   if (draw && draw->texture)
+      gfx_display_d3d9_bind_texture(draw, d3d);
+
+   type  = (D3DPRIMITIVETYPE)gfx_display_prim_to_d3d9_hlsl_enum(draw->prim_type);
+   start = d3d->menu_display.offset;
+   count = draw->coords->vertices -
+         ((draw->prim_type == GFX_DISPLAY_PRIM_TRIANGLESTRIP)
+          ? 2 : 0);
+
+   IDirect3DDevice9_BeginScene(dev);
+   IDirect3DDevice9_DrawPrimitive(dev, type, start, count);
+   IDirect3DDevice9_EndScene(dev);
+
+   d3d->menu_display.offset += draw->coords->vertices;
+}
+
+static void gfx_display_d3d9_hlsl_draw_pipeline(
+      gfx_display_ctx_draw_t *draw,
+      gfx_display_t *p_disp,
+      void *data, unsigned video_width, unsigned video_height)
+{
+   static float t                    = 0;
+   video_coord_array_t *ca           = NULL;
+
+   if (!draw)
+      return;
+
+   ca                                = &p_disp->dispca;
+
+   draw->x                           = 0;
+   draw->y                           = 0;
+   draw->coords                      = NULL;
+   draw->matrix_data                 = NULL;
+
+   if (ca)
+      draw->coords                   = (struct video_coords*)&ca->coords;
+
+   switch (draw->pipeline_id)
+   {
+      case VIDEO_SHADER_MENU:
+      case VIDEO_SHADER_MENU_2:
+      case VIDEO_SHADER_MENU_3:
+         /* TODO/FIXME - implement */
+#if 0
+         {
+            struct uniform_info uniform_param  = {0};
+            t                                 += 0.01;
+
+            uniform_param.enabled              = true;
+            uniform_param.lookup.enable        = true;
+            uniform_param.lookup.add_prefix    = true;
+            uniform_param.lookup.idx           = draw->pipeline_id;
+            uniform_param.lookup.type          = SHADER_PROGRAM_VERTEX;
+            uniform_param.type                 = UNIFORM_1F;
+            uniform_param.lookup.ident         = "time";
+            uniform_param.result.f.v0          = t;
+         }
+#endif
+         break;
+   }
+}
+
+void gfx_display_d3d9_hlsl_scissor_begin(
+      void *data,
+      unsigned video_width, unsigned video_height,
+      int x, int y, unsigned width, unsigned height)
+{
+   RECT rect;
+   d3d9_video_t *d3d9 = (d3d9_video_t*)data;
+
+   if (!d3d9)
+      return;
+
+   rect.left          = x;
+   rect.top           = y;
+   rect.right         = width + x;
+   rect.bottom        = height + y;
+
+   IDirect3DDevice9_SetScissorRect(d3d9->dev, &rect);
+}
+
+void gfx_display_d3d9_hlsl_scissor_end(void *data,
+      unsigned video_width, unsigned video_height)
+{
+   RECT rect;
+   d3d9_video_t   *d3d9 = (d3d9_video_t*)data;
+
+   if (!d3d9)
+      return;
+
+   rect.left            = 0;
+   rect.top             = 0;
+   rect.right           = video_width;
+   rect.bottom          = video_height;
+
+   IDirect3DDevice9_SetScissorRect(d3d9->dev, &rect);
+}
+
+gfx_display_ctx_driver_t gfx_display_ctx_d3d9_hlsl = {
+   gfx_display_d3d9_hlsl_draw,
+   gfx_display_d3d9_hlsl_draw_pipeline,
+   gfx_display_d3d9_hlsl_blend_begin,
+   gfx_display_d3d9_hlsl_blend_end,
+   gfx_display_d3d9_hlsl_get_default_mvp,
+   gfx_display_d3d9_hlsl_get_default_vertices,
+   gfx_display_d3d9_hlsl_get_default_tex_coords,
+   FONT_DRIVER_RENDER_D3D9_API,
+   GFX_VIDEO_DRIVER_DIRECT3D9_HLSL,
+   "d3d9_hlsl",
+   false,
+   gfx_display_d3d9_hlsl_scissor_begin,
+   gfx_display_d3d9_hlsl_scissor_end
+};
+
+/*
+ * VIDEO DRIVER
+ */
+
 static void *d3d9_hlsl_get_constant_by_name(LPD3DXCONSTANTTABLE prog, const char *name)
 {
    char lbl[64];
@@ -183,19 +483,20 @@ static bool d3d9_hlsl_load_program(
       struct shader_pass *pass,
       const char *prog)
 {
-   ID3DXBuffer *listing_f                    = NULL;
-   ID3DXBuffer *listing_v                    = NULL;
-   ID3DXBuffer *code_f                       = NULL;
-   ID3DXBuffer *code_v                       = NULL;
+   ID3DXBuffer *listing_f = NULL;
+   ID3DXBuffer *listing_v = NULL;
+   ID3DXBuffer *code_f    = NULL;
+   ID3DXBuffer *code_v    = NULL;
+   size_t prog_len        = strlen(prog);
 
-   if (!d3d9x_compile_shader(prog, strlen(prog), NULL, NULL,
+   if (!d3d9x_compile_shader(prog, prog_len, NULL, NULL,
             "main_fragment", "ps_3_0", 0, &code_f, &listing_f,
             &pass->ftable ))
    {
       RARCH_ERR("Could not compile stock fragment shader..\n");
       goto error;
    }
-   if (!d3d9x_compile_shader(prog, strlen(prog), NULL, NULL,
+   if (!d3d9x_compile_shader(prog, prog_len, NULL, NULL,
             "main_vertex", "vs_3_0", 0, &code_v, &listing_v,
             &pass->vtable ))
    {
@@ -293,10 +594,10 @@ static bool hlsl_d3d9_renderchain_create_first_pass(
       const struct LinkInfo *info,
       unsigned _fmt)
 {
-   unsigned i;
+   int i;
    struct shader_pass pass       = { 0 };
    unsigned fmt                  =
-        (_fmt == RETRO_PIXEL_FORMAT_RGB565) 
+        (_fmt == RETRO_PIXEL_FORMAT_RGB565)
       ? D3D9_RGB565_FORMAT
       : D3D9_XRGB8888_FORMAT;
 
@@ -465,7 +766,7 @@ static void d3d9_hlsl_deinit_progs(hlsl_renderchain_t *chain)
 {
    if (chain->chain.passes->count >= 1)
    {
-      unsigned i;
+      size_t i;
 
       d3d9_vertex_buffer_free(NULL,
             chain->chain.passes->data[0].vertex_decl);
@@ -484,7 +785,7 @@ static void d3d9_hlsl_deinit_progs(hlsl_renderchain_t *chain)
 
 static void d3d9_hlsl_destroy_resources(hlsl_renderchain_t *chain)
 {
-   unsigned i;
+   size_t i;
 
    for (i = 0; i < TEXTURES; i++)
    {
@@ -544,8 +845,8 @@ static void hlsl_d3d9_renderchain_render_pass(
       struct shader_pass *pass,
       unsigned pass_index)
 {
-   /* Currently we override the passes shader program 
-      with the stock shader as at least the last pass 
+   /* Currently we override the passes shader program
+      with the stock shader as at least the last pass
       is not setup correctly */
 #if 0
    d3d9_hlsl_bind_program(chain->chain.dev, pass);
@@ -556,8 +857,8 @@ static void hlsl_d3d9_renderchain_render_pass(
    IDirect3DDevice9_SetTexture(chain->chain.dev, 0,
          (IDirect3DBaseTexture9*)pass->tex);
 
-   /* D3D8 sets the sampler address modes - 
-      I've left them out for the time being 
+   /* D3D8 sets the sampler address modes -
+      I've left them out for the time being
       but maybe this is a bug in d3d9 */
 #if 0
    IDirect3DDevice9_SetSamplerState(chain->chain.dev,
@@ -719,7 +1020,7 @@ static void hlsl_d3d9_renderchain_render(
             chain->chain.frame_count, 0);
 
       hlsl_d3d9_renderchain_render_pass(chain,
-            from_pass, 
+            from_pass,
             i + 1);
 
       current_width  = out_width;
@@ -886,7 +1187,7 @@ static bool d3d9_hlsl_init_chain(d3d9_video_t *d3d,
             d3d, (hlsl_renderchain_t*)d3d->renderchain_data,
             d3d->dev, &d3d->final_viewport, &link_info,
             rgb32
-            ? RETRO_PIXEL_FORMAT_XRGB8888 
+            ? RETRO_PIXEL_FORMAT_XRGB8888
             : RETRO_PIXEL_FORMAT_RGB565
             )
       )
@@ -934,8 +1235,8 @@ static bool d3d9_hlsl_init_chain(d3d9_video_t *d3d,
                   chain,
                   d3d->shader.lut[i].id,
                   d3d->shader.lut[i].path,
-                  d3d->shader.lut[i].filter == RARCH_FILTER_UNSPEC 
-                  ? video_smooth 
+                  d3d->shader.lut[i].filter == RARCH_FILTER_UNSPEC
+                  ? video_smooth
                   : (d3d->shader.lut[i].filter == RARCH_FILTER_LINEAR)))
          {
             RARCH_ERR("[D3D9]: Failed to init LUTs.\n");
@@ -1151,10 +1452,12 @@ static bool d3d9_hlsl_init_internal(d3d9_video_t *d3d,
 
    windowed_full         = settings->bools.video_windowed_fullscreen;
 
-   full_x                = (windowed_full || info->width  == 0) ?
-      (mon_rect.right  - mon_rect.left) : info->width;
-   full_y                = (windowed_full || info->height == 0) ?
-      (mon_rect.bottom - mon_rect.top)  : info->height;
+   full_x                = (windowed_full || info->width  == 0)
+      ? (unsigned)(mon_rect.right  - mon_rect.left)
+      : info->width;
+   full_y                = (windowed_full || info->height == 0)
+      ? (unsigned)(mon_rect.bottom - mon_rect.top)
+      : info->height;
 #else
    d3d9_get_video_size(d3d, &full_x, &full_y);
 #endif
@@ -1186,9 +1489,9 @@ static bool d3d9_hlsl_init_internal(d3d9_video_t *d3d,
 #ifndef _XBOX_
    d3d9_hlsl_fake_context.get_metrics = win32_get_metrics;
 #endif
-   video_context_driver_set(&d3d9_hlsl_fake_context); 
+   video_context_driver_set(&d3d9_hlsl_fake_context);
    {
-      const char *shader_preset   = retroarch_get_shader_preset();
+      const char *shader_preset   = video_shader_get_current_shader_preset();
       enum rarch_shader_type type = video_shader_parse_type(shader_preset);
 
       d3d9_hlsl_set_shader(d3d, type, shader_preset);
@@ -1213,8 +1516,6 @@ static bool d3d9_hlsl_init_internal(d3d9_video_t *d3d,
 
       RARCH_LOG("[D3D9]: Using GPU: \"%s\".\n", ident.Description);
       RARCH_LOG("[D3D9]: GPU API Version: %s\n", version_str);
-
-      video_driver_set_gpu_device_string(ident.Description);
       video_driver_set_gpu_api_version_string(version_str);
    }
 
@@ -1314,7 +1615,7 @@ static bool d3d9_hlsl_frame(void *data, const void *frame,
    struct font_params *osd_params      = (struct font_params*)
       &video_info->osd_stat_params;
    const char *stat_text               = video_info->stat_text;
-   bool menu_is_alive                  = video_info->menu_is_alive;
+   bool menu_is_alive                  = (video_info->menu_st_flags & MENU_ST_FLAG_ALIVE) ? true : false;
    bool overlay_behind_menu            = video_info->overlay_behind_menu;
 #ifdef HAVE_GFX_WIDGETS
    bool widgets_active                 = video_info->widgets_active;
@@ -1371,12 +1672,12 @@ static bool d3d9_hlsl_frame(void *data, const void *frame,
    hlsl_d3d9_renderchain_render(
          d3d, frame, frame_width, frame_height,
          pitch, d3d->dev_rotation);
-   
+
    if (black_frame_insertion && !d3d->menu->enabled)
    {
       unsigned n;
-      for (n = 0; n < video_info->black_frame_insertion; ++n) 
-      {   
+      for (n = 0; n < video_info->black_frame_insertion; ++n)
+      {
 #ifdef _XBOX
         bool ret = true;
         IDirect3DDevice9_Present(d3d->dev, NULL, NULL, NULL, NULL);
@@ -1389,7 +1690,7 @@ static bool d3d9_hlsl_frame(void *data, const void *frame,
         IDirect3DDevice9_Clear(d3d->dev, 0, 0, D3DCLEAR_TARGET,
               0, 1, 0);
       }
-   }   
+   }
 
 #ifdef HAVE_OVERLAY
    if (d3d->overlays_enabled && overlay_behind_menu)
@@ -1452,7 +1753,7 @@ static bool d3d9_hlsl_frame(void *data, const void *frame,
       IDirect3DDevice9_EndScene(d3d->dev);
    }
 
-   win32_update_title();
+   video_driver_update_title(NULL);
    IDirect3DDevice9_Present(d3d->dev, NULL, NULL, NULL, NULL);
 
    return true;
@@ -1464,12 +1765,12 @@ static const video_poke_interface_t d3d9_hlsl_poke_interface = {
    d3d9_unload_texture,
    d3d9_set_video_mode,
 #if defined(_XBOX) || defined(__WINRT__)
-   NULL,
+   NULL, /* get_refresh_rate */
 #else
    /* UWP does not expose this information easily */
    win32_get_refresh_rate,
 #endif
-   NULL,
+   NULL, /* set_filtering */
    NULL, /* get_video_output_size */
    NULL, /* get_video_output_prev */
    NULL, /* get_video_output_next */
@@ -1480,16 +1781,15 @@ static const video_poke_interface_t d3d9_hlsl_poke_interface = {
    d3d9_set_menu_texture_frame,
    d3d9_set_menu_texture_enable,
    d3d9_set_osd_msg,
-
    win32_show_cursor,
-   NULL,                         /* grab_mouse_toggle */
-   NULL,                         /* get_current_shader */
-   NULL,                         /* get_current_software_framebuffer */
-   NULL,                         /* get_hw_render_interface */
-   NULL,                         /* set_hdr_max_nits */
-   NULL,                         /* set_hdr_paper_white_nits */
-   NULL,                         /* set_hdr_contrast */
-   NULL                          /* set_hdr_expand_gamut */
+   NULL, /* grab_mouse_toggle */
+   NULL, /* get_current_shader */
+   NULL, /* get_current_software_framebuffer */
+   NULL, /* get_hw_render_interface */
+   NULL, /* set_hdr_max_nits */
+   NULL, /* set_hdr_paper_white_nits */
+   NULL, /* set_hdr_contrast */
+   NULL  /* set_hdr_expand_gamut */
 };
 
 static void d3d9_hlsl_get_poke_interface(void *data,
@@ -1582,10 +1882,7 @@ static void d3d9_hlsl_set_nonblock_state(void *data, bool state,
 }
 
 #ifdef _XBOX
-static bool d3d9_hlsl_suppress_screensaver(void *data, bool enable)
-{
-   return true;
-}
+static bool d3d9_hlsl_suspend_screensaver(void *data, bool enable) { return true; }
 #endif
 
 video_driver_t video_d3d9_hlsl = {
@@ -1593,11 +1890,11 @@ video_driver_t video_d3d9_hlsl = {
    d3d9_hlsl_frame,
    d3d9_hlsl_set_nonblock_state,
    d3d9_hlsl_alive,
-   NULL,                      /* focus */
+   NULL, /* focus */
 #ifdef _XBOX
-   d3d9_hlsl_suppress_screensaver,
+   d3d9_hlsl_suspend_screensaver,
 #else
-   win32_suppress_screensaver,
+   win32_suspend_screensaver,
 #endif
    d3d9_has_windowed,
    d3d9_hlsl_set_shader,
@@ -1607,12 +1904,9 @@ video_driver_t video_d3d9_hlsl = {
    d3d9_set_rotation,
    d3d9_viewport_info,
    d3d9_read_viewport,
-   NULL,                      /* read_frame_raw */
+   NULL, /* read_frame_raw */
 #ifdef HAVE_OVERLAY
    d3d9_get_overlay_interface,
-#endif
-#ifdef HAVE_VIDEO_LAYOUT
-   NULL,
 #endif
    d3d9_hlsl_get_poke_interface,
    NULL, /* wrap_type_to_enum */
