@@ -15,9 +15,13 @@
  *  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <retro_miscellaneous.h>
 #include <stdlib.h>
 
+#include <retro_miscellaneous.h>
+#include <string/stdstring.h>
+
+#include <sixel.h>
+
 #ifdef HAVE_CONFIG_H
 #include "../../config.h"
 #endif
@@ -33,7 +37,7 @@
 #include "../../retroarch.h"
 #include "../../verbosity.h"
 #include "../../frontend/frontend_driver.h"
-#include "../common/sixel_common.h"
+#include "../common/sixel_defines.h"
 
 #ifndef _WIN32
 #define HAVE_SYS_IOCTL_H
@@ -57,6 +61,75 @@
 #error "Old version of libsixel detected, please upgrade to at least 1.6.0."
 #endif
 
+/*
+ * FONT DRIVER
+ */
+
+typedef struct
+{
+   const font_renderer_driver_t *font_driver;
+   void *font_data;
+   sixel_t *sixel;
+} sixel_raster_t;
+
+static void *sixel_font_init(void *data,
+      const char *font_path, float font_size,
+      bool is_threaded)
+{
+   sixel_raster_t *font  = (sixel_raster_t*)calloc(1, sizeof(*font));
+
+   if (!font)
+      return NULL;
+
+   font->sixel = (sixel_t*)data;
+
+   if (!font_renderer_create_default(
+            &font->font_driver,
+            &font->font_data, font_path, font_size))
+      return NULL;
+
+   return font;
+}
+
+static void sixel_font_free(void *data, bool is_threaded)
+{
+  sixel_raster_t *font  = (sixel_raster_t*)data;
+  if (!font)
+     return;
+
+  if (font->font_driver && font->font_data && font->font_driver->free)
+     font->font_driver->free(font->font_data);
+
+  free(font);
+}
+
+static int sixel_font_get_message_width(void *data, const char *msg,
+      size_t msg_len, float scale) { return 0; }
+static const struct font_glyph *sixel_font_get_glyph(
+      void *data, uint32_t code) { return NULL; }
+/* TODO/FIXME: add text drawing support */
+static void sixel_font_render_msg(
+      void *userdata,
+      void *data,
+      const char *msg,
+      const struct font_params *_params) { }
+
+font_renderer_t sixel_font = {
+   sixel_font_init,
+   sixel_font_free,
+   sixel_font_render_msg,
+   "sixel",
+   sixel_font_get_glyph,
+   NULL,                       /* bind_block */
+   NULL,                       /* flush */
+   sixel_font_get_message_width,
+   NULL                        /* get_line_metrics */
+};
+
+/*
+ * VIDEO DRIVER
+ */
+
 static unsigned char *sixel_menu_frame = NULL;
 static unsigned sixel_menu_width       = 0;
 static unsigned sixel_menu_height      = 0;
@@ -67,8 +140,6 @@ static unsigned sixel_video_pitch      = 0;
 static unsigned sixel_video_bits       = 0;
 static unsigned sixel_menu_bits        = 0;
 static double sixel_video_scale        = 1;
-static bool sixel_rgb32                = false;
-static bool sixel_menu_rgb32           = false;
 static unsigned *sixel_temp_buf        = NULL;
 
 static int sixel_write(char *data, int size, void *priv)
@@ -201,7 +272,6 @@ static void *sixel_gfx_init(const video_info_t *video,
    *input                               = NULL;
    *input_data                          = NULL;
 
-   sixel_rgb32                          = video->rgb32;
    sixel_video_bits                     = video->rgb32 ? 32 : 16;
 
    if (video->rgb32)
@@ -256,7 +326,7 @@ static bool sixel_gfx_frame(void *data, const void *frame,
    bool draw                 = true;
    sixel_t *sixel            = (sixel_t*)data;
 #ifdef HAVE_MENU
-   bool menu_is_alive        = video_info->menu_is_alive;
+   bool menu_is_alive        = (video_info->menu_st_flags & MENU_ST_FLAG_ALIVE) ? true : false;
 #endif
 
    if (!frame || !frame_width || !frame_height)
@@ -523,34 +593,34 @@ static void sixel_set_video_mode(void *data, unsigned width, unsigned height,
       bool fullscreen) { }
 
 static const video_poke_interface_t sixel_poke_interface = {
-   NULL,
-   NULL,
-   NULL,
+   NULL, /* get_flags */
+   NULL, /* load_texture */
+   NULL, /* unload_texture */
    sixel_set_video_mode,
-   NULL,
-   NULL,
+   NULL, /* get_refresh_rate */
+   NULL, /* set_filtering */
    sixel_get_video_output_size,
    sixel_get_video_output_prev,
    sixel_get_video_output_next,
-   NULL,
-   NULL,
-   NULL,
-   NULL,
-#if defined(HAVE_MENU)
+   NULL, /* get_current_framebuffer */
+   NULL, /* get_proc_address */
+   NULL, /* set_aspect_ratio */
+   NULL, /* apply_state_changes */
+#ifdef HAVE_MENU
    sixel_set_texture_frame,
-   NULL,
+   NULL, /* set_texture_enable */
    font_driver_render_msg,
-   NULL,
+   NULL, /* show_mouse */
 #else
-   NULL,
-   NULL,
-   NULL,
-   NULL,
+   NULL, /* set_texture_frame */
+   NULL, /* set_texture_enable */
+   NULL, /* set_osd_msg */
+   NULL, /* show_mouse */
 #endif
-   NULL,
-   NULL,
-   NULL,
-   NULL,
+   NULL, /* grab_mouse_toggle */
+   NULL, /* get_current_shader */
+   NULL, /* get_current_software_framebuffer */
+   NULL, /* get_hw_render_interface */
    NULL, /* set_hdr_max_nits */
    NULL, /* set_hdr_paper_white_nits */
    NULL, /* set_hdr_contrast */
@@ -591,11 +661,11 @@ video_driver_t video_sixel = {
    NULL, /* read_viewport */
    NULL, /* read_frame_raw */
 #ifdef HAVE_OVERLAY
-   NULL, /* overlay_interface */
-#endif
-#ifdef HAVE_VIDEO_LAYOUT
-  NULL,
+   NULL, /* get_overlay_interface */
 #endif
    sixel_gfx_get_poke_interface,
-   NULL /* wrap_type_to_enum */
+   NULL, /* wrap_type_to_enum */
+#ifdef HAVE_GFX_WIDGETS
+   NULL  /* gfx_widgets_enabled */
+#endif
 };
