@@ -18,9 +18,9 @@
 
 /* Win32/WGL context. */
 
-/* necessary for mingw32 multimon defines: */
+/* Necessary for mingw32 multimon defines: */
 #ifndef _WIN32_WINNT
-#define _WIN32_WINNT 0x0500 //_WIN32_WINNT_WIN2K
+#define _WIN32_WINNT 0x0500 /* _WIN32_WINNT_WIN2K */
 #endif
 
 #include <tchar.h>
@@ -54,6 +54,10 @@
 #endif
 #endif
 
+#ifdef __WINRT__
+#include "../common/uwpgdi.h"
+#endif
+
 #if (defined(HAVE_OPENGL) || defined(HAVE_OPENGL1) || defined(HAVE_OPENGL_CORE)) && !defined(HAVE_OPENGLES)
 #include "../common/gl_common.h"
 #endif
@@ -84,6 +88,14 @@
 #endif
 #endif
 
+/* Forward declarations */
+LRESULT CALLBACK wnd_proc_wgl_common(HWND hwnd, UINT message,
+      WPARAM wparam, LPARAM lparam);
+LRESULT CALLBACK wnd_proc_wgl_dinput(HWND hwnd, UINT message,
+      WPARAM wparam, LPARAM lparam);
+LRESULT CALLBACK wnd_proc_wgl_winraw(HWND hwnd, UINT message,
+      WPARAM wparam, LPARAM lparam);
+
 static BOOL (APIENTRY *p_swap_interval)(int);
 
 enum wgl_flags
@@ -105,7 +117,7 @@ static void             *dinput_wgl       = NULL;
 static unsigned         win32_major       = 0;
 static unsigned         win32_minor       = 0;
 static int              win32_interval    = 0;
-static enum gfx_ctx_api win32_api         = GFX_CTX_NONE;
+enum gfx_ctx_api win32_api                = GFX_CTX_NONE;
 #ifdef HAVE_DYLIB
 static dylib_t          dll_handle        = NULL; /* Handle to OpenGL32.dll/libGLESv2.dll */
 #endif
@@ -115,23 +127,19 @@ typedef struct gfx_ctx_cgl_data
    void *empty;
 } gfx_ctx_wgl_data_t;
 
+/* FORWARD DECLARATIONS */
+void win32_get_video_size(void *data, unsigned *width, unsigned *height);
+
 static gfx_ctx_proc_t gfx_ctx_wgl_get_proc_address(const char *symbol)
 {
-   switch (win32_api)
-   {
-      case GFX_CTX_OPENGL_API:
 #if (defined(HAVE_OPENGL) || defined(HAVE_OPENGL1) || defined(HAVE_OPENGL_CORE)) && !defined(HAVE_OPENGLES)
-         {
-            gfx_ctx_proc_t func = (gfx_ctx_proc_t)wglGetProcAddress(symbol);
-            if (func)
-               return func;
-         }
-#endif
-         break;
-      default:
-         break;
+   if (win32_api == GFX_CTX_OPENGL_API)
+   {
+      gfx_ctx_proc_t func = (gfx_ctx_proc_t)wglGetProcAddress(symbol);
+      if (func)
+         return func;
    }
-
+#endif
 #ifdef HAVE_DYLIB
    return (gfx_ctx_proc_t)GetProcAddress((HINSTANCE)dll_handle, symbol);
 #else
@@ -140,37 +148,35 @@ static gfx_ctx_proc_t gfx_ctx_wgl_get_proc_address(const char *symbol)
 }
 
 #if (defined(HAVE_OPENGL) || defined(HAVE_OPENGL1) || defined(HAVE_OPENGL_CORE)) && !defined(HAVE_OPENGLES)
-static bool wgl_has_extension(const char *extension, const char *extensions)
+static bool wgl_has_extension(const char *ext, const char *exts)
 {
-   const char *start      = NULL;
-   const char *terminator = NULL;
-   const char      *where = strchr(extension, ' ');
+   const char *where = strchr(ext, ' ');
 
-   if (where || *extension == '\0')
+   if (where || *ext == '\0')
       return false;
 
-   if (!extensions)
-      return false;
-
-   start = extensions;
-
-   for (;;)
+   if (exts)
    {
-      where = strstr(start, extension);
-      if (!where)
-         break;
+      const char *terminator = NULL;
+      const char *start      = exts;
 
-      terminator = where + strlen(extension);
-      if (where == start || *(where - 1) == ' ')
-         if (*terminator == ' ' || *terminator == '\0')
-            return true;
+      for (;;)
+      {
+         if (!(where = strstr(start, ext)))
+            break;
 
-      start = terminator;
+         terminator = where + strlen(ext);
+         if (where == start || *(where - 1) == ' ')
+            if (*terminator == ' ' || *terminator == '\0')
+               return true;
+
+         start = terminator;
+      }
    }
    return false;
 }
 
-static void create_gl_context(HWND hwnd, bool *quit)
+void create_gl_context(HWND hwnd, bool *quit)
 {
    struct retro_hw_render_callback *hwr = video_driver_get_hw_context();
    bool core_context                    = (win32_major * 1000 + win32_minor) >= 3001;
@@ -180,7 +186,11 @@ static void create_gl_context(HWND hwnd, bool *quit)
    bool debug                           = hwr->debug_context;
 #endif
 
+#ifdef __WINRT__
+   win32_hdc                            = (HDC)(hwnd);
+#else
    win32_hdc                            = GetDC(hwnd);
+#endif
 
    win32_setup_pixel_format(win32_hdc, true);
 
@@ -191,22 +201,21 @@ static void create_gl_context(HWND hwnd, bool *quit)
    }
    else
    {
-      win32_hrc = wglCreateContext(win32_hdc);
-
+      win32_hrc         = wglCreateContext(win32_hdc);
       /* We'll create shared context later if not. */
       if (win32_hrc && !core_context && !debug)
       {
-         win32_hw_hrc = wglCreateContext(win32_hdc);
+         win32_hw_hrc   = wglCreateContext(win32_hdc);
          if (win32_hw_hrc)
          {
             if (!wglShareLists(win32_hrc, win32_hw_hrc))
             {
                RARCH_LOG("[WGL]: Failed to share contexts.\n");
-               *quit = true;
+               *quit    = true;
             }
          }
          else
-            *quit = true;
+            *quit       = true;
       }
    }
 
@@ -219,7 +228,7 @@ static void create_gl_context(HWND hwnd, bool *quit)
    }
    else
    {
-      *quit        = true;
+      *quit             = true;
       return;
    }
 
@@ -265,11 +274,11 @@ static void create_gl_context(HWND hwnd, bool *quit)
          pcreate_context = (wglCreateContextAttribsProc)
             gfx_ctx_wgl_get_proc_address("wglCreateContextAttribsARB");
 
-      /* In order to support the core info "required_hw_api" 
+      /* In order to support the core info "required_hw_api"
        * field correctly, we should try to init the highest available
-       * version GL context possible. This means trying successively 
+       * version GL context possible. This means trying successively
        * lower versions until it works, because GL has
-       * no facility for determining the highest possible 
+       * no facility for determining the highest possible
        * supported version.
        */
       if (pcreate_context)
@@ -281,11 +290,11 @@ static void create_gl_context(HWND hwnd, bool *quit)
          int version_rows     = gl_version_rows;
          int (*versions)[2]   = gl_versions;
 
-         /* only try higher versions when core_context is true */
+         /* Only try higher versions when core_context is true */
          if (!core_context)
             version_rows = 1;
 
-         /* try versions from highest down to requested version */
+         /* Try versions from highest down to requested version */
          for (i = 0; i < version_rows; i++)
          {
             if (core_context)
@@ -320,15 +329,15 @@ static void create_gl_context(HWND hwnd, bool *quit)
                   }
                }
 
-               /* found a suitable version that is high enough, we can stop now */
+               /* Found a suitable version that is high enough, we can stop now */
                break;
             }
             else if (
-                  versions[i][0] == win32_major && 
-                  versions[i][1] == win32_minor)
+                     (versions[i][0] == (int)win32_major)
+                  && (versions[i][1] == (int)win32_minor))
             {
-               /* The requested version was tried and 
-                * is not supported, go ahead and fail 
+               /* The requested version was tried and
+                * is not supported, go ahead and fail
                 * since everything else will be lower than that. */
                break;
             }
@@ -345,17 +354,17 @@ static void create_gl_context(HWND hwnd, bool *quit)
    }
 
    {
-
       const char *(WINAPI * wglGetExtensionsStringARB) (HDC) = 0;
-      const char *extensions                                 = NULL;
+      const char *exts                                       = NULL;
+      wglGetExtensionsStringARB                              =
+	      (const char *(WINAPI *) (HDC))
+	      gfx_ctx_wgl_get_proc_address("wglGetExtensionsStringARB");
 
-      wglGetExtensionsStringARB = (const char *(WINAPI *) (HDC))
-         gfx_ctx_wgl_get_proc_address("wglGetExtensionsStringARB");
       if (wglGetExtensionsStringARB)
       {
-         extensions = wglGetExtensionsStringARB(win32_hdc);
-         RARCH_LOG("[WGL]: Extensions: %s\n", extensions);
-         if (wgl_has_extension("WGL_EXT_swap_control_tear", extensions))
+         exts = wglGetExtensionsStringARB(win32_hdc);
+         RARCH_LOG("[WGL]: Extensions: %s\n", exts);
+         if (wgl_has_extension("WGL_EXT_swap_control_tear", exts))
          {
             RARCH_LOG("[WGL]: Adaptive VSync supported.\n");
             wgl_flags |= WGL_FLAG_ADAPTIVE_VSYNC;
@@ -366,7 +375,7 @@ static void create_gl_context(HWND hwnd, bool *quit)
 #endif
 
 #if defined(HAVE_OPENGLES) && defined(HAVE_EGL)
-static void create_gles_context(HWND hwnd, bool *quit)
+void create_gles_context(HWND hwnd, bool *quit)
 {
    EGLint n, major, minor;
    EGLint format;
@@ -418,32 +427,8 @@ error:
 }
 #endif
 
-void create_wgl_context(HWND hwnd, bool *quit)
-{
-   switch (win32_api)
-   {
-      case GFX_CTX_OPENGL_API:
-#if (defined(HAVE_OPENGL) || defined(HAVE_OPENGL1) || defined(HAVE_OPENGL_CORE)) && !defined(HAVE_OPENGLES)
-         create_gl_context(hwnd, quit);
-#endif
-         break;
-
-      case GFX_CTX_OPENGL_ES_API:
-#if defined (HAVE_OPENGLES)
-         create_gles_context(hwnd, quit);
-#endif
-         break;
-
-      case GFX_CTX_NONE:
-      default:
-         break;
-   }
-}
-
 static void gfx_ctx_wgl_swap_interval(void *data, int interval)
 {
-   (void)data;
-
    switch (win32_api)
    {
       case GFX_CTX_OPENGL_API:
@@ -476,12 +461,14 @@ static void gfx_ctx_wgl_swap_interval(void *data, int interval)
 
 static void gfx_ctx_wgl_swap_buffers(void *data)
 {
-   (void)data;
-
    switch (win32_api)
    {
       case GFX_CTX_OPENGL_API:
+#ifdef __WINRT__
+         wglSwapBuffers(win32_hdc);
+#else
          SwapBuffers(win32_hdc);
+#endif
          break;
       case GFX_CTX_OPENGL_ES_API:
 #if defined(HAVE_EGL)
@@ -497,47 +484,6 @@ static void gfx_ctx_wgl_swap_buffers(void *data)
 static bool gfx_ctx_wgl_set_resize(void *data,
       unsigned width, unsigned height) { return false; }
 
-static void gfx_ctx_wgl_update_title(void *data)
-{
-   char title[128];
-
-   title[0] = '\0';
-
-   video_driver_get_window_title(title, sizeof(title));
-
-   if (title[0])
-   {
-      const ui_window_t *window = ui_companion_driver_get_window_ptr();
-
-      if (window)
-         window->set_title(&main_window, title);
-   }
-}
-
-static void gfx_ctx_wgl_get_video_size(void *data,
-      unsigned *width, unsigned *height)
-{
-   HWND         window          = win32_get_window();
-
-   if (window)
-   {
-      *width                    = g_win32_resize_width;
-      *height                   = g_win32_resize_height;
-   }
-   else
-   {
-      RECT mon_rect;
-      MONITORINFOEX current_mon;
-      unsigned mon_id           = 0;
-      HMONITOR hm_to_use        = NULL;
-
-      win32_monitor_info(&current_mon, &hm_to_use, &mon_id);
-      mon_rect                  = current_mon.rcMonitor;
-      *width                    = mon_rect.right - mon_rect.left;
-      *height                   = mon_rect.bottom - mon_rect.top;
-   }
-}
-
 static void gfx_ctx_wgl_destroy(void *data)
 {
    HWND            window  = win32_get_window();
@@ -550,10 +496,11 @@ static void gfx_ctx_wgl_destroy(void *data)
          if (win32_hrc)
          {
             uint32_t video_st_flags;
+            video_driver_state_t *video_st = video_state_get_ptr();
             gl_finish();
             wglMakeCurrent(NULL, NULL);
 
-            video_st_flags = video_driver_get_st_flags();
+            video_st_flags = video_st->flags;
             if (!(video_st_flags & VIDEO_FLAG_CACHE_CONTEXT))
             {
                if (win32_hw_hrc)
@@ -579,19 +526,25 @@ static void gfx_ctx_wgl_destroy(void *data)
 
    if (window && win32_hdc)
    {
+#ifndef __WINRT__
       ReleaseDC(window, win32_hdc);
+#endif
       win32_hdc = NULL;
    }
 
+#ifndef __WINRT__
    if (window)
    {
       win32_monitor_from_window();
       win32_destroy_window();
    }
 
+#endif
    if (g_win32_flags & WIN32_CMN_FLAG_RESTORE_DESKTOP)
    {
+#ifndef __WINRT__
       win32_monitor_get_info();
+#endif
       g_win32_flags &= ~WIN32_CMN_FLAG_RESTORE_DESKTOP;
    }
 
@@ -614,9 +567,12 @@ static void gfx_ctx_wgl_destroy(void *data)
 
 static void *gfx_ctx_wgl_init(void *video_driver)
 {
+#ifndef __WINRT__
    WNDCLASSEX wndclass     = {0};
+#endif
    gfx_ctx_wgl_data_t *wgl = (gfx_ctx_wgl_data_t*)calloc(1, sizeof(*wgl));
    uint8_t win32_flags     = win32_get_flags();
+   settings_t *settings    = config_get_ptr();
 
    if (!wgl)
       return NULL;
@@ -632,31 +588,37 @@ static void *gfx_ctx_wgl_init(void *video_driver)
 #endif
 #endif
 
+#ifndef __WINRT__
    win32_window_reset();
    win32_monitor_init();
 
-   {
-      settings_t *settings     = config_get_ptr();
-      wndclass.lpfnWndProc   = wnd_proc_wgl_common;
+
+   wndclass.lpfnWndProc    = wnd_proc_wgl_common;
 #ifdef HAVE_DINPUT
-      if (string_is_equal(settings->arrays.input_driver, "dinput"))
-         wndclass.lpfnWndProc   = wnd_proc_wgl_dinput;
+   if (string_is_equal(settings->arrays.input_driver, "dinput"))
+	   wndclass.lpfnWndProc = wnd_proc_wgl_dinput;
 #endif
 #ifdef HAVE_WINRAWINPUT
-      if (string_is_equal(settings->arrays.input_driver, "raw"))
-         wndclass.lpfnWndProc   = wnd_proc_wgl_winraw;
+   if (string_is_equal(settings->arrays.input_driver, "raw"))
+	   wndclass.lpfnWndProc = wnd_proc_wgl_winraw;
 #endif
-   }
 
    if (!win32_window_init(&wndclass, true, NULL))
-      goto error;
-
-   return wgl;
-
-error:
-   if (wgl)
+   {
       free(wgl);
-   return NULL;
+      return NULL;
+   }
+#else
+   bool quit = false;
+   create_gl_context(uwp_get_corewindow(), &quit);
+   if (quit)
+   {
+      RARCH_ERR("[UWP WGL]: create_gl_context failed.\n");
+      free(wgl);
+      return NULL;
+   }
+#endif
+   return wgl;
 }
 
 static bool gfx_ctx_wgl_set_video_mode(void *data,
@@ -666,25 +628,16 @@ static bool gfx_ctx_wgl_set_video_mode(void *data,
    if (!win32_set_video_mode(NULL, width, height, fullscreen))
    {
       RARCH_ERR("[WGL]: win32_set_video_mode failed.\n");
-      goto error;
+      gfx_ctx_wgl_destroy(data);
+      return false;
    }
 
-   switch (win32_api)
-   {
-      case GFX_CTX_OPENGL_API:
-         p_swap_interval = (BOOL (APIENTRY *)(int))gfx_ctx_wgl_get_proc_address("wglSwapIntervalEXT");
-         break;
-      case GFX_CTX_NONE:
-      default:
-         break;
-   }
+
+   if (win32_api == GFX_CTX_OPENGL_API)
+      p_swap_interval = (BOOL (APIENTRY *)(int))gfx_ctx_wgl_get_proc_address("wglSwapIntervalEXT");
 
    gfx_ctx_wgl_swap_interval(data, win32_interval);
    return true;
-
-error:
-   gfx_ctx_wgl_destroy(data);
-   return false;
 }
 
 static void gfx_ctx_wgl_input_driver(void *data,
@@ -715,19 +668,33 @@ static void gfx_ctx_wgl_input_driver(void *data,
    dinput_wgl  = input_driver_init_wrap(&input_dinput, joypad_name);
    *input      = dinput_wgl ? &input_dinput : NULL;
    *input_data = dinput_wgl;
+#elif defined(__WINRT__)
+   /* Plain xinput is supported on UWP, but it
+    * supports joypad only (uwp driver was added later) */
+   if (string_is_equal(settings->arrays.input_driver, "xinput"))
+   {
+      void* xinput = input_driver_init_wrap(&input_xinput, joypad_name);
+      *input = xinput ? (input_driver_t*)&input_xinput : NULL;
+      *input_data = xinput;
+   }
+   else
+   {
+      void* uwp = input_driver_init_wrap(&input_uwp, joypad_name);
+      *input = uwp ? (input_driver_t*)&input_uwp : NULL;
+      *input_data = uwp;
+   }
+#elif defined(_XBOX)
+   void* xinput = input_driver_init_wrap(&input_xinput, joypad_name);
+   *input = xinput ? (input_driver_t*)&input_xinput : NULL;
+   *input_data = xinput;
 #endif
 }
 
-static enum gfx_ctx_api gfx_ctx_wgl_get_api(void *data)
-{
-   return win32_api;
-}
+static enum gfx_ctx_api gfx_ctx_wgl_get_api(void *data) { return win32_api; }
 
 static bool gfx_ctx_wgl_bind_api(void *data,
       enum gfx_ctx_api api, unsigned major, unsigned minor)
 {
-   (void)data;
-
    win32_major = major;
    win32_minor = minor;
    win32_api   = api;
@@ -843,15 +810,59 @@ static void gfx_ctx_wgl_set_flags(void *data, uint32_t flags)
 
 }
 
-static void gfx_ctx_wgl_get_video_output_size(void *data,
-      unsigned *width, unsigned *height, char *desc, size_t desc_len)
-{
-   win32_get_video_output_size(width, height, desc, desc_len);
-}
-
 static void gfx_ctx_wgl_get_video_output_prev(void *data) { }
 static void gfx_ctx_wgl_get_video_output_next(void *data) { }
 
+//TODO: maybe create an uwp_mesa_common.c?
+#ifdef __WINRT__
+
+static void win32_get_video_size(void* data,
+   unsigned* width, unsigned* height)
+{
+   bool quit = false;
+   bool resize = false;
+   win32_check_window(NULL, &quit, &resize, width, height);
+   width = uwp_get_width();
+   height = uwp_get_height();
+}
+
+void win32_get_video_output_size(void* data, unsigned* width, unsigned* height, char* desc, size_t desc_len)
+{
+   win32_get_video_size(data, width, height);
+}
+
+bool win32_suspend_screensaver(void* data, bool enable)
+{
+   return true;
+}
+
+float win32_get_refresh_rate(void* data)
+{
+   return 60.0;
+}
+
+#define win32_get_refresh_rate NULL
+
+HWND win32_get_window(void)
+{
+   return (HWND)uwp_get_corewindow();
+}
+
+/* TODO/FIXME - static globals */
+uint8_t g_win32_flags = 0;
+
+uint8_t win32_get_flags(void)
+{
+   return g_win32_flags;
+}
+
+void win32_setup_pixel_format(HDC hdc, bool supports_gl)
+{
+   //NTD, already done by mesa
+}
+
+#endif
+
 const gfx_ctx_driver_t gfx_ctx_wgl = {
    gfx_ctx_wgl_init,
    gfx_ctx_wgl_destroy,
@@ -859,18 +870,18 @@ const gfx_ctx_driver_t gfx_ctx_wgl = {
    gfx_ctx_wgl_bind_api,
    gfx_ctx_wgl_swap_interval,
    gfx_ctx_wgl_set_video_mode,
-   gfx_ctx_wgl_get_video_size,
+   win32_get_video_size,
    win32_get_refresh_rate,
-   gfx_ctx_wgl_get_video_output_size,
+   win32_get_video_output_size,
    gfx_ctx_wgl_get_video_output_prev,
    gfx_ctx_wgl_get_video_output_next,
    win32_get_metrics,
    NULL,
-   gfx_ctx_wgl_update_title,
+   video_driver_update_title,
    win32_check_window,
    gfx_ctx_wgl_set_resize,
    win32_has_focus,
-   win32_suppress_screensaver,
+   win32_suspend_screensaver,
    true, /* has_windowed */
    gfx_ctx_wgl_swap_buffers,
    gfx_ctx_wgl_input_driver,
