@@ -112,9 +112,9 @@ static void gfx_thumbnail_init_fade(
    /* A 'fade in' animation is triggered if:
     * - Thumbnail is available
     * - Thumbnail is missing and 'fade_missing' is enabled */
-   if ((thumbnail->status == GFX_THUMBNAIL_STATUS_AVAILABLE) ||
-       (p_gfx_thumb->fade_missing &&
-            (thumbnail->status == GFX_THUMBNAIL_STATUS_MISSING)))
+   if (   (thumbnail->status == GFX_THUMBNAIL_STATUS_AVAILABLE)
+       || (p_gfx_thumb->fade_missing
+       && (thumbnail->status == GFX_THUMBNAIL_STATUS_MISSING)))
    {
       if (p_gfx_thumb->fade_duration > 0.0f)
       {
@@ -239,17 +239,15 @@ void gfx_thumbnail_cancel_pending_requests(void)
  *         and gfx_thumbnail_set_content*()
  * NOTE 2: 'playlist' and 'idx' are only required here for
  *         on-demand thumbnail download support
- *         (an annoyance...) */ 
+ *         (an annoyance...) */
 void gfx_thumbnail_request(
       gfx_thumbnail_path_data_t *path_data, enum gfx_thumbnail_id thumbnail_id,
       playlist_t *playlist, size_t idx, gfx_thumbnail_t *thumbnail,
       unsigned gfx_thumbnail_upscale_threshold,
       bool network_on_demand_thumbnails)
 {
-   const char *thumbnail_path         = NULL;
-   bool has_thumbnail                 = false;
    gfx_thumbnail_state_t *p_gfx_thumb = &gfx_thumb_st;
-   
+
    if (!path_data || !thumbnail)
       return;
 
@@ -260,74 +258,91 @@ void gfx_thumbnail_request(
 
    /* Update/extract thumbnail path */
    if (gfx_thumbnail_is_enabled(path_data, thumbnail_id))
-      if (gfx_thumbnail_update_path(path_data, thumbnail_id))
-         has_thumbnail = gfx_thumbnail_get_path(path_data, thumbnail_id, &thumbnail_path);
-
-   /* Load thumbnail, if required */
-   if (has_thumbnail)
    {
-      if (path_is_valid(thumbnail_path))
+      if (gfx_thumbnail_update_path(path_data, thumbnail_id))
       {
-         gfx_thumbnail_tag_t *thumbnail_tag =
-               (gfx_thumbnail_tag_t*)malloc(sizeof(gfx_thumbnail_tag_t));
+         const char *thumbnail_path = NULL;
+         if (gfx_thumbnail_get_path(path_data, thumbnail_id, &thumbnail_path))
+         {
+            /* Load thumbnail, if required */
+            if (path_is_valid(thumbnail_path))
+            {
+               gfx_thumbnail_tag_t *thumbnail_tag =
+                  (gfx_thumbnail_tag_t*)malloc(sizeof(gfx_thumbnail_tag_t));
 
-         if (!thumbnail_tag)
-            goto end;
+               if (!thumbnail_tag)
+                  goto end;
 
-         /* Configure user data */
-         thumbnail_tag->thumbnail = thumbnail;
-         thumbnail_tag->list_id   = p_gfx_thumb->list_id;
+               /* Configure user data */
+               thumbnail_tag->thumbnail = thumbnail;
+               thumbnail_tag->list_id   = p_gfx_thumb->list_id;
 
-         /* Would like to cancel any existing image load tasks
-          * here, but can't see how to do it... */
-         if (task_push_image_load(
-               thumbnail_path, video_driver_supports_rgba(),
-               gfx_thumbnail_upscale_threshold,
-               gfx_thumbnail_handle_upload, thumbnail_tag))
-            thumbnail->status = GFX_THUMBNAIL_STATUS_PENDING;
-      }
+               /* Would like to cancel any existing image load tasks
+                * here, but can't see how to do it... */
+               if (task_push_image_load(
+                        thumbnail_path, video_driver_supports_rgba(),
+                        gfx_thumbnail_upscale_threshold,
+                        gfx_thumbnail_handle_upload, thumbnail_tag))
+                  thumbnail->status = GFX_THUMBNAIL_STATUS_PENDING;
+            }
 #ifdef HAVE_NETWORKING
-      /* Handle on demand thumbnail downloads */
-      else if (network_on_demand_thumbnails)
-      {
-         const char *system                         = NULL;
-         const char *img_name                       = NULL;
-         static char last_img_name[PATH_MAX_LENGTH] = {0};
+            /* Handle on demand thumbnail downloads */
+            else if (network_on_demand_thumbnails)
+            {
+               const char *system                         = NULL;
+               const char *img_name                       = NULL;
+               static char last_img_name[PATH_MAX_LENGTH] = {0};
+               settings_t *settings                       = config_get_ptr();
+               enum playlist_thumbnail_name_flags curr_flag;
+               if (!playlist)
+                  goto end;
 
-         if (!playlist)
-            goto end;
+               /* Validate entry */
+               if (!gfx_thumbnail_get_img_name(path_data, &img_name, PLAYLIST_THUMBNAIL_FLAG_STD_NAME))
+                  goto end;
 
-         /* Get current image name */
-         if (!gfx_thumbnail_get_img_name(path_data, &img_name))
-            goto end;
+               /* Only trigger a thumbnail download if image
+                * name has changed since the last call of
+                * gfx_thumbnail_request()
+                * > Allows gfx_thumbnail_request() to be used
+                *   for successive right/left thumbnail requests
+                *   with minimal duplication of effort
+                *   (i.e. task_push_pl_entry_thumbnail_download()
+                *   will automatically cancel if a download for the
+                *   existing playlist entry is pending, but the
+                *   checks required for this involve significant
+                *   overheads. We can avoid this entirely with
+                *   a simple string comparison) */
+               if (string_is_equal(img_name, last_img_name))
+                  goto end;
 
-         /* Only trigger a thumbnail download if image
-          * name has changed since the last call of
-          * gfx_thumbnail_request()
-          * > Allows gfx_thumbnail_request() to be used
-          *   for successive right/left thumbnail requests
-          *   with minimal duplication of effort
-          *   (i.e. task_push_pl_entry_thumbnail_download()
-          *   will automatically cancel if a download for the
-          *   existing playlist entry is pending, but the
-          *   checks required for this involve significant
-          *   overheads. We can avoid this entirely with
-          *   a simple string comparison) */
-         if (string_is_equal(img_name, last_img_name))
-            goto end;
+               strlcpy(last_img_name, img_name, sizeof(last_img_name));
 
-         strlcpy(last_img_name, img_name, sizeof(last_img_name));
+               /* Get system name */
+               if (!gfx_thumbnail_get_system(path_data, &system))
+                  goto end;
 
-         /* Get system name */
-         if (!gfx_thumbnail_get_system(path_data, &system))
-            goto end;
+               /* Since task_push_pl_entry_download will shift the flag, do not attempt if it is already
+                * at second to last option. */
+               curr_flag = playlist_get_curr_thumbnail_name_flag(playlist,idx);
+               if (curr_flag & PLAYLIST_THUMBNAIL_FLAG_NONE || curr_flag & PLAYLIST_THUMBNAIL_FLAG_SHORT_NAME)
+                  goto end;
+               /* Do not try to fetch full names here, if it is not explicitly wanted */
+               if (!settings->bools.playlist_use_filename && 
+                   !playlist_thumbnail_match_with_filename(playlist) &&
+                   curr_flag == PLAYLIST_THUMBNAIL_FLAG_INVALID)
+                    playlist_update_thumbnail_name_flag(playlist, idx, PLAYLIST_THUMBNAIL_FLAG_FULL_NAME);
 
-         /* Trigger thumbnail download */
-         task_push_pl_entry_thumbnail_download(
-               system, playlist, (unsigned)idx,
-               false, true);
-      }
+               /* Trigger thumbnail download *
+                * Note: download will grab all 3 possible thumbnails, no matter
+                * what left/right thumbnails are set at the moment */
+               task_push_pl_entry_thumbnail_download(
+                     system, playlist, (unsigned)idx,
+                     false, true);
+            }
 #endif
+         }
+      }
    }
 
 end:
@@ -447,8 +462,8 @@ void gfx_thumbnail_request_stream(
 
    /* Only process request if current status
     * is GFX_THUMBNAIL_STATUS_UNKNOWN */
-   if (!thumbnail ||
-       (thumbnail->status != GFX_THUMBNAIL_STATUS_UNKNOWN))
+   if (   !thumbnail
+       || (thumbnail->status != GFX_THUMBNAIL_STATUS_UNKNOWN))
       return;
 
    /* Check if stream delay timer has elapsed */
@@ -621,9 +636,9 @@ void gfx_thumbnail_process_stream(
          if (thumbnail->delay_timer > p_gfx_thumb->stream_delay)
          {
             /* Update thumbnail content */
-            if (!path_data ||
-                !playlist ||
-                !gfx_thumbnail_set_content_playlist(path_data, playlist, idx))
+            if (   !path_data
+                || !playlist
+                || !gfx_thumbnail_set_content_playlist(path_data, playlist, idx))
             {
                /* Content is invalid
                 * > Reset thumbnail and set missing status */
@@ -714,9 +729,9 @@ void gfx_thumbnail_process_streams(
          if (request_right || request_left)
          {
             /* Update thumbnail content */
-            if (!path_data ||
-                !playlist ||
-                !gfx_thumbnail_set_content_playlist(path_data, playlist, idx))
+            if (   !path_data
+                || !playlist
+                || !gfx_thumbnail_set_content_playlist(path_data, playlist, idx))
             {
                /* Content is invalid
                 * > Reset thumbnail and set missing status */
@@ -787,10 +802,10 @@ void gfx_thumbnail_get_draw_dimensions(
    video_driver_state_t *video_st = video_state_get_ptr();
 
    /* Sanity check */
-   if (   !thumbnail 
-       || (width             < 1) 
+   if (   !thumbnail
+       || (width             < 1)
        || (height            < 1)
-       || (thumbnail->width  < 1) 
+       || (thumbnail->width  < 1)
        || (thumbnail->height < 1))
    {
       *draw_width  = 0.0f;
@@ -802,9 +817,10 @@ void gfx_thumbnail_get_draw_dimensions(
     * differences */
    display_aspect   = (float)width            / (float)height;
    thumbnail_aspect = (float)thumbnail->width / (float)thumbnail->height;
-   core_aspect      = ((thumbnail->flags & GFX_THUMB_FLAG_CORE_ASPECT) 
-         && video_st)
-         ? video_st->av_info.geometry.aspect_ratio : thumbnail_aspect;
+   core_aspect      = ((thumbnail->flags & GFX_THUMB_FLAG_CORE_ASPECT)
+         && video_st && video_st->av_info.geometry.aspect_ratio > 0)
+               ? video_st->av_info.geometry.aspect_ratio
+               : thumbnail_aspect;
 
    if (thumbnail_aspect > display_aspect)
    {
@@ -980,8 +996,8 @@ void gfx_thumbnail_draw(
       if (shadow)
       {
          /* Sanity check */
-         if ((shadow->type != GFX_THUMBNAIL_SHADOW_NONE) &&
-               (shadow->alpha > 0.0f))
+         if (     (shadow->type != GFX_THUMBNAIL_SHADOW_NONE)
+               && (shadow->alpha > 0.0f))
          {
             float shadow_width;
             float shadow_height;
