@@ -19,7 +19,6 @@
 
 #include <stddef.h>
 
-
 #include <libretro.h>
 #include <retro_common_api.h>
 #include <boolean.h>
@@ -41,15 +40,10 @@
 
 #include "video_defines.h"
 
-#ifdef HAVE_VIDEO_LAYOUT
-#include "video_layout.h"
-#endif
-
 #ifdef HAVE_CRTSWITCHRES
 #include "video_crt_switch.h"
 #endif
 
-#include "video_coord_array.h"
 #include "video_shader_parse.h"
 #include "video_filter.h"
 
@@ -57,20 +51,21 @@
 
 #define MEASURE_FRAME_TIME_SAMPLES_COUNT (2 * 1024)
 
-#define VIDEO_SHADER_STOCK_BLEND (GFX_MAX_SHADERS - 1)
-#define VIDEO_SHADER_MENU        (GFX_MAX_SHADERS - 2)
-#define VIDEO_SHADER_MENU_2      (GFX_MAX_SHADERS - 3)
-#define VIDEO_SHADER_MENU_3      (GFX_MAX_SHADERS - 4)
-#define VIDEO_SHADER_MENU_4      (GFX_MAX_SHADERS - 5)
-#define VIDEO_SHADER_MENU_5      (GFX_MAX_SHADERS - 6)
-#define VIDEO_SHADER_MENU_6      (GFX_MAX_SHADERS - 7)
-#define VIDEO_SHADER_STOCK_HDR   (GFX_MAX_SHADERS - 8)
+#define VIDEO_SHADER_STOCK_BLEND   (GFX_MAX_SHADERS - 1)
+#define VIDEO_SHADER_MENU          (GFX_MAX_SHADERS - 2)
+#define VIDEO_SHADER_MENU_2        (GFX_MAX_SHADERS - 3)
+#define VIDEO_SHADER_MENU_3        (GFX_MAX_SHADERS - 4)
+#define VIDEO_SHADER_MENU_4        (GFX_MAX_SHADERS - 5)
+#define VIDEO_SHADER_MENU_5        (GFX_MAX_SHADERS - 6)
+#define VIDEO_SHADER_MENU_6        (GFX_MAX_SHADERS - 7)
+#define VIDEO_SHADER_STOCK_HDR     (GFX_MAX_SHADERS - 8)
+#define VIDEO_SHADER_STOCK_NOBLEND (GFX_MAX_SHADERS - 9)
 
 #define VIDEO_HDR_MAX_CONTRAST 10.0f
 
 #if defined(_XBOX360)
 #define DEFAULT_SHADER_TYPE RARCH_SHADER_HLSL
-#elif defined(__PSL1GHT__) || defined(HAVE_OPENGLES2) || defined(HAVE_GLSL)
+#elif defined(HAVE_OPENGLES2) || defined(HAVE_GLSL)
 #define DEFAULT_SHADER_TYPE RARCH_SHADER_GLSL
 #elif defined(HAVE_CG)
 #define DEFAULT_SHADER_TYPE RARCH_SHADER_CG
@@ -278,19 +273,6 @@ typedef struct shader_backend
    const char *ident;
 } shader_backend_t;
 
-typedef struct video_shader_ctx_init
-{
-   const char *path;
-   const shader_backend_t *shader;
-   void *data;
-   void *shader_data;
-   enum rarch_shader_type shader_type;
-   struct
-   {
-      bool core_context_enabled;
-   } gl;
-} video_shader_ctx_init_t;
-
 typedef struct video_shader_ctx_params
 {
    void *data;
@@ -308,48 +290,11 @@ typedef struct video_shader_ctx_params
    unsigned fbo_info_cnt;
 } video_shader_ctx_params_t;
 
-typedef struct video_shader_ctx_coords
-{
-   void *handle_data;
-   const void *data;
-} video_shader_ctx_coords_t;
-
-typedef struct video_shader_ctx_scale
-{
-   struct gfx_fbo_scale *scale;
-   unsigned idx;
-} video_shader_ctx_scale_t;
-
-typedef struct video_shader_ctx_info
-{
-   void *data;
-   unsigned num;
-   unsigned idx;
-   bool set_active;
-} video_shader_ctx_info_t;
-
-typedef struct video_shader_ctx_mvp
-{
-   void *data;
-   const void *matrix;
-} video_shader_ctx_mvp_t;
-
-typedef struct video_shader_ctx_filter
-{
-   bool *smooth;
-   unsigned index;
-} video_shader_ctx_filter_t;
-
 typedef struct video_shader_ctx
 {
    struct video_shader *data;
 } video_shader_ctx_t;
 
-typedef struct video_shader_ctx_texture
-{
-   unsigned id;
-} video_shader_ctx_texture_t;
-
 typedef struct video_pixel_scaler
 {
    struct scaler_ctx *scaler;
@@ -452,6 +397,7 @@ typedef struct video_frame_info
    int crt_switch_porch_adjust;
 
    unsigned hard_sync_frames;
+   unsigned runahead_frames;
    unsigned aspect_ratio_idx;
    unsigned max_swapchain_images;
    unsigned monitor_index;
@@ -469,6 +415,9 @@ typedef struct video_frame_info
    unsigned custom_vp_full_width;
    unsigned custom_vp_full_height;
    unsigned black_frame_insertion;
+   unsigned bfi_dark_frames;
+   unsigned shader_subframes;
+   unsigned current_subframe;
    unsigned fps_update_interval;
    unsigned memory_update_interval;
    unsigned msg_queue_delay;
@@ -478,6 +427,7 @@ typedef struct video_frame_info
    float menu_header_opacity;
    float menu_footer_opacity;
    float refresh_rate;
+   float font_size;
    float font_msg_pos_x;
    float font_msg_pos_y;
    float font_msg_color_r;
@@ -485,7 +435,6 @@ typedef struct video_frame_info
    float font_msg_color_b;
    float xmb_alpha_factor;
 
-
    struct
    {
       /* Drop shadow offset.
@@ -504,18 +453,21 @@ typedef struct video_frame_info
       bool full_screen;
    } osd_stat_params;
 
-   char stat_text[512];
+   uint32_t video_st_flags;
+   uint16_t menu_st_flags;
+
+   char stat_text[1024];
 
    bool widgets_active;
    bool notifications_hidden;
    bool menu_mouse_enable;
-   bool widgets_is_paused;
-   bool widgets_is_fast_forwarding;
-   bool widgets_is_rewinding;
    bool input_menu_swap_ok_cancel_buttons;
    bool input_driver_nonblock_state;
    bool input_driver_grab_mouse_state;
    bool hard_sync;
+   bool runahead;
+   bool runahead_second_instance;
+   bool preemptive_frames;
    bool fps_show;
    bool memory_show;
    bool statistics_show;
@@ -525,7 +477,6 @@ typedef struct video_frame_info
    bool windowed_fullscreen;
    bool fullscreen;
    bool font_enable;
-   bool use_rgba;
    bool hdr_support;
    bool libretro_running;
    bool xmb_shadows_enable;
@@ -534,12 +485,12 @@ typedef struct video_frame_info
    bool runloop_is_slowmotion;
    bool runloop_is_paused;
    bool fastforward_frameskip;
-   bool menu_is_alive;
-   bool menu_screensaver_active;
+   bool frame_rest;
    bool msg_bgcolor_enable;
    bool crt_switch_hires_menu;
    bool hdr_enable;
    bool overlay_behind_menu;
+   bool scan_subframes;
 } video_frame_info_t;
 
 typedef void (*update_window_title_cb)(void*);
@@ -659,14 +610,6 @@ typedef struct gfx_ctx_driver
    void (*make_current)(bool release);
 } gfx_ctx_driver_t;
 
-typedef struct gfx_ctx_size
-{
-   bool *quit;
-   bool *resize;
-   unsigned *width;
-   unsigned *height;
-} gfx_ctx_size_t;
-
 typedef struct gfx_ctx_mode
 {
    unsigned width;
@@ -680,30 +623,11 @@ typedef struct gfx_ctx_metrics
    enum display_metric_types type;
 } gfx_ctx_metrics_t;
 
-typedef struct gfx_ctx_aspect
-{
-   float *aspect;
-   unsigned width;
-   unsigned height;
-} gfx_ctx_aspect_t;
-
-typedef struct gfx_ctx_input
-{
-   input_driver_t **input;
-   void **input_data;
-} gfx_ctx_input_t;
-
 typedef struct gfx_ctx_ident
 {
    const char *ident;
 } gfx_ctx_ident_t;
 
-struct aspect_ratio_elem
-{
-   float value;
-   char name[64];
-};
-
 /* Optionally implemented interface to poke more
  * deeply into video driver. */
 
@@ -736,9 +660,9 @@ typedef struct video_poke_interface
          unsigned width, unsigned height, float alpha);
    /* Enable or disable rendering. */
    void (*set_texture_enable)(void *data, bool enable, bool full_screen);
-   void (*set_osd_msg)(void *data, 
+   void (*set_osd_msg)(void *data,
          const char *msg,
-         const void *params, void *font);
+         const struct font_params *params, void *font);
 
    void (*show_mouse)(void *data, bool state);
    void (*grab_mouse_toggle)(void *data);
@@ -749,11 +673,11 @@ typedef struct video_poke_interface
    bool (*get_hw_render_interface)(void *data,
          const struct retro_hw_render_interface **iface);
 
-   /* hdr settings */ 
+   /* hdr settings */
    void (*set_hdr_max_nits)(void *data, float max_nits);
    void (*set_hdr_paper_white_nits)(void *data, float paper_white_nits);
    void (*set_hdr_contrast)(void *data, float contrast);
-   void (*set_hdr_expand_gamut)(void *data, bool expand_gamut);         
+   void (*set_hdr_expand_gamut)(void *data, bool expand_gamut);
 } video_poke_interface_t;
 
 /* msg is for showing a message on the screen
@@ -835,9 +759,6 @@ typedef struct video_driver
 #ifdef HAVE_OVERLAY
    void (*overlay_interface)(void *data,
          const video_overlay_interface_t **iface);
-#endif
-#ifdef HAVE_VIDEO_LAYOUT
-   const video_layout_render_interface_t *(*video_layout_render_interface)(void *data);
 #endif
    void (*poke_interface)(void *data, const video_poke_interface_t **iface);
    unsigned (*wrap_type_to_enum)(enum gfx_wrap_type type);
@@ -856,9 +777,9 @@ typedef struct
 #endif
    struct retro_system_av_info av_info; /* double alignment */
    retro_time_t frame_time_samples[MEASURE_FRAME_TIME_SAMPLES_COUNT];
-   retro_time_t core_frame_time;
    uint64_t frame_time_count;
    uint64_t frame_count;
+   uint64_t frame_rest_time_count;
    uint8_t *record_gpu_buffer;
 #ifdef HAVE_VIDEO_FILTER
    rarch_softfilter_t *state_filter;
@@ -912,8 +833,6 @@ typedef struct
    unsigned state_scale;
    unsigned state_out_bpp;
 #endif
-   unsigned frame_delay_target;
-   unsigned frame_delay_effective;
    unsigned frame_cache_width;
    unsigned frame_cache_height;
    unsigned width;
@@ -941,46 +860,58 @@ typedef struct
 
    char cli_shader_path[PATH_MAX_LENGTH];
    char window_title[512];
-   char gpu_device_string[128];
+   char window_title_prev[512];
    char gpu_api_version_string[128];
    char title_buf[64];
    char cached_driver_id[32];
 
+   uint8_t frame_rest;
+   uint8_t frame_delay_target;
+   uint8_t frame_delay_effective;
+   bool frame_delay_pause;
+
    bool threaded;
 } video_driver_state_t;
 
-typedef struct video_frame_delay_auto {
+typedef struct video_frame_delay_auto
+{
    float refresh_rate;
-   unsigned frame_time_interval;
-   unsigned decrease;
-   unsigned target;
-   unsigned time;
+   uint16_t frame_time_target;
+   uint16_t frame_time_avg;
+   uint8_t frame_time_interval;
+   uint8_t delay_decrease;
 } video_frame_delay_auto_t;
 
+struct aspect_ratio_elem
+{
+   float value;
+   char name[64];
+};
+
 extern struct aspect_ratio_elem aspectratio_lut[ASPECT_RATIO_END];
 
+#if !(defined(RARCH_CONSOLE) || defined(RARCH_MOBILE))
 bool video_driver_has_windowed(void);
+#else
+#define video_driver_has_windowed() (false)
+#endif
+
+#ifdef HAVE_THREADS
+bool video_driver_is_threaded(void);
+#else
+#define video_driver_is_threaded() (false)
+#endif
 
 bool video_driver_has_focus(void);
 
-bool video_driver_cached_frame_has_valid_framebuffer(void);
-
-void video_driver_set_cached_frame_ptr(const void *data);
-
 void video_driver_set_stub_frame(void);
 
 void video_driver_unset_stub_frame(void);
 
-bool video_driver_supports_viewport_read(void);
-
-bool video_driver_prefer_viewport_read(void);
-
-bool video_driver_supports_read_frame_raw(void);
+float video_driver_get_core_aspect(void);
 
 void video_driver_set_viewport_core(void);
 
-void video_driver_reset_custom_viewport(settings_t *settings);
-
 void video_driver_set_rgba(void);
 
 void video_driver_unset_rgba(void);
@@ -993,14 +924,6 @@ void video_driver_unset_hdr_support(void);
 
 bool video_driver_supports_hdr(void);
 
-unsigned video_driver_get_hdr_color(unsigned color);
-
-float video_driver_get_hdr_luminance(float nits);
-
-unsigned video_driver_get_hdr_paper_white(void);
-
-float* video_driver_get_hdr_paper_white_float(void);
-
 bool video_driver_get_next_video_out(void);
 
 bool video_driver_get_prev_video_out(void);
@@ -1011,23 +934,14 @@ void video_driver_set_aspect_ratio(void);
 
 void video_driver_update_viewport(struct video_viewport* vp, bool force_full, bool keep_aspect);
 
-void video_driver_show_mouse(void);
-
-void video_driver_hide_mouse(void);
-
 void video_driver_apply_state_changes(void);
 
-bool video_driver_read_viewport(uint8_t *buffer, bool is_idle);
-
 void video_driver_cached_frame(void);
 
 bool video_driver_is_hw_context(void);
 
 struct retro_hw_render_callback *video_driver_get_hw_context(void);
 
-const struct retro_hw_render_context_negotiation_interface
-*video_driver_get_context_negotiation_interface(void);
-
 bool video_driver_get_viewport_info(struct video_viewport *viewport);
 
 /**
@@ -1059,46 +973,19 @@ bool video_driver_set_video_mode(unsigned width,
 bool video_driver_get_video_output_size(
       unsigned *width, unsigned *height, char *desc, size_t desc_len);
 
-void video_driver_set_texture_enable(bool enable, bool full_screen);
-
-void video_driver_set_texture_frame(const void *frame, bool rgb32,
-      unsigned width, unsigned height, float alpha);
-
-#ifdef HAVE_VIDEO_LAYOUT
-const video_layout_render_interface_t *video_driver_layout_render_interface(void);
-#endif
-
 void * video_driver_read_frame_raw(unsigned *width,
    unsigned *height, size_t *pitch);
 
 void video_driver_set_filtering(unsigned index, bool smooth, bool ctx_scaling);
 
-void video_driver_set_hdr_max_nits(float max_nits);
-void video_driver_set_hdr_paper_white_nits(float paper_white_nits);
-void video_driver_set_hdr_contrast(float contrast);
-void video_driver_set_hdr_expand_gamut(bool expand_gamut);
-
 const char *video_driver_get_ident(void);
 
-void video_driver_set_viewport(unsigned width, unsigned height,
-      bool force_fullscreen, bool allow_rotate);
-
 void video_driver_get_size(unsigned *width, unsigned *height);
 
 void video_driver_set_size(unsigned width, unsigned height);
 
 float video_driver_get_aspect_ratio(void);
 
-void video_driver_set_aspect_ratio_value(float value);
-
-enum retro_pixel_format video_driver_get_pixel_format(void);
-
-void video_driver_cached_frame_set(const void *data, unsigned width,
-      unsigned height, size_t pitch);
-
-void video_driver_cached_frame_get(const void **data, unsigned *width,
-      unsigned *height, size_t *pitch);
-
 void video_driver_menu_settings(void **list_data, void *list_info_data,
       void *group_data, void *subgroup_data, const char *parent_group);
 
@@ -1117,10 +1004,6 @@ void video_viewport_get_scaled_integer(struct video_viewport *vp,
       unsigned width, unsigned height,
       float aspect_ratio, bool keep_aspect);
 
-struct retro_system_av_info *video_viewport_get_system_av_info(void);
-
-struct video_viewport *video_viewport_get_custom(void);
-
 /**
  * video_monitor_set_refresh_rate:
  * @hz                 : New refresh rate for monitor.
@@ -1155,18 +1038,6 @@ void video_monitor_compute_fps_statistics(uint64_t
 bool video_monitor_fps_statistics(double *refresh_rate,
       double *deviation, unsigned *sample_points);
 
-bool video_driver_monitor_adjust_system_rates(
-      float timing_skew_hz,
-      float video_refresh_rate,
-      bool vrr_runloop_enable,
-      float audio_max_timing_skew,
-      unsigned video_swap_interval,
-      double input_fps);
-
-void crt_switch_driver_refresh(void);
-
-char* crt_switch_core_name(void);
-
 #define video_driver_translate_coord_viewport_wrap(vp, mouse_x, mouse_y, res_x, res_y, res_screen_x, res_screen_y) \
    (video_driver_get_viewport_info(vp) ? video_driver_translate_coord_viewport(vp, mouse_x, mouse_y, res_x, res_y, res_screen_x, res_screen_y) : false)
 
@@ -1223,7 +1094,16 @@ bool *video_driver_get_threaded(void);
 
 void video_driver_set_threaded(bool val);
 
-void video_frame_delay_auto(video_driver_state_t *video_st, video_frame_delay_auto_t *vfda);
+void video_frame_delay(video_driver_state_t *video_st,
+      settings_t *settings,
+      bool core_paused);
+
+void video_frame_delay_auto(video_driver_state_t *video_st,
+      video_frame_delay_auto_t *vfda);
+
+void video_frame_rest(video_driver_state_t *video_st,
+      settings_t *settings,
+      retro_time_t current_time);
 
 /**
  * video_context_driver_init:
@@ -1291,11 +1171,6 @@ bool video_shader_driver_get_current_shader(video_shader_ctx_t *shader);
 
 float video_driver_get_refresh_rate(void);
 
-#if defined(HAVE_GFX_WIDGETS)
-bool video_driver_has_widgets(void);
-#endif
-
-bool video_driver_is_threaded(void);
 
 bool video_context_driver_get_flags(gfx_ctx_flags_t *flags);
 
@@ -1303,10 +1178,6 @@ bool video_driver_test_all_flags(enum display_flags testflag);
 
 gfx_ctx_flags_t video_driver_get_flags_wrapper(void);
 
-void video_driver_set_gpu_device_string(const char *str);
-
-const char* video_driver_get_gpu_device_string(void);
-
 void video_driver_set_gpu_api_version_string(const char *str);
 
 const char* video_driver_get_gpu_api_version_string(void);
@@ -1318,8 +1189,6 @@ void video_driver_set_gpu_api_devices(enum gfx_ctx_api api, struct string_list *
 
 struct string_list* video_driver_get_gpu_api_devices(enum gfx_ctx_api api);
 
-enum retro_hw_context_type hw_render_context_type(const char *s);
-
 const char *hw_render_context_name(
       enum retro_hw_context_type type, int major, int minor);
 
@@ -1395,8 +1264,6 @@ void video_driver_set_viewport_config(
 
 void video_driver_set_viewport_square_pixel(struct retro_game_geometry *geom);
 
-uint32_t video_driver_get_st_flags(void);
-
 bool video_driver_init_internal(bool *video_is_threaded, bool verbosity_enabled);
 
 /**
@@ -1411,6 +1278,13 @@ bool video_driver_init_internal(bool *video_is_threaded, bool verbosity_enabled)
 void video_driver_frame(const void *data, unsigned width,
       unsigned height, size_t pitch);
 
+void video_driver_update_title(void *data);
+
+bool video_coord_array_append(video_coord_array_t *ca,
+      const video_coords_t *coords, unsigned count);
+
+void video_coord_array_free(video_coord_array_t *ca);
+
 extern const video_driver_t *video_drivers[];
 
 extern video_driver_t video_gl3;
