@@ -390,7 +390,8 @@ bool gfx_widget_start_load_content_animation(void)
       /* If content was found in playlist but the entry
        * did not have a db_name, use playlist name itself
        * as the system name */
-      if (playlist_entry_found && !has_system)
+      if (      playlist_entry_found 
+            && !has_system)
       {
          const char *playlist_path = playlist_get_conf_path(playlist);
 
@@ -451,11 +452,9 @@ bool gfx_widget_start_load_content_animation(void)
    {
       size_t len = strlcpy(state->icon_file, state->system_name,
             sizeof(state->icon_file));
-      state->icon_file[len]   = '.';
-      state->icon_file[len+1] = 'p';
-      state->icon_file[len+2] = 'n';
-      state->icon_file[len+3] = 'g';
-      state->icon_file[len+4] = '\0';
+      strlcpy(state->icon_file       + len,
+            ".png",
+            sizeof(state->icon_file) - len);
 
       fill_pathname_join_special(icon_path,
             state->icon_directory, state->icon_file,
@@ -480,16 +479,14 @@ bool gfx_widget_start_load_content_animation(void)
           (databases_list->size == 1))
          core_db_name = databases_list->elems[0].data;
 
-      if (!string_is_empty(core_db_name) &&
-          !string_is_equal(core_db_name, state->system_name))
+      if (   !string_is_empty(core_db_name)
+          && !string_is_equal(core_db_name, state->system_name))
       {
          size_t len = strlcpy(state->icon_file, core_db_name,
                sizeof(state->icon_file));
-         state->icon_file[len]   = '.';
-         state->icon_file[len+1] = 'p';
-         state->icon_file[len+2] = 'n';
-         state->icon_file[len+3] = 'g';
-         state->icon_file[len+4] = '\0';
+         strlcpy(state->icon_file       + len,
+               ".png",
+               sizeof(state->icon_file) - len);
 
          fill_pathname_join_special(icon_path,
                state->icon_directory, state->icon_file,
@@ -569,6 +566,42 @@ static void gfx_widget_load_content_animation_layout(
          (float)font_regular->line_centre_offset;
    /* > Note: cannot determine state->text_x_end
     *   until text strings are set */
+
+   /* Recalculate end positions if layout changes after start */
+   if (state->status > GFX_WIDGET_LOAD_CONTENT_BEGIN)
+   {
+      int content_name_width;
+      int system_name_width;
+      int text_width;
+
+      /* Get overall text width */
+      content_name_width = font_driver_get_message_width(
+            font_bold->font, state->content_name,
+            strlen(state->content_name), 1.0f);
+      system_name_width = font_driver_get_message_width(
+            font_regular->font, state->system_name,
+            state->system_name_len, 1.0f);
+
+      state->content_name_width = (content_name_width > 0) ?
+            (unsigned)content_name_width : 0;
+      state->system_name_width  = (system_name_width > 0) ?
+            (unsigned)system_name_width : 0;
+
+      text_width = (state->content_name_width > state->system_name_width) ?
+            (int)state->content_name_width : (int)state->system_name_width;
+
+      /* Now we have the text width, can determine
+       * final icon/text x draw positions */
+      state->icon_x_end = ((int)last_video_width - text_width -
+            (int)state->icon_size - (3 * (int)widget_padding)) >> 1;
+      if (state->icon_x_end < (int)widget_padding)
+         state->icon_x_end = widget_padding;
+
+      state->text_x_end = state->icon_x_end +
+            (float)(state->icon_size + widget_padding);
+
+   }
+
 }
 
 /* Widget iterate() */
