@@ -111,23 +111,22 @@ void dinput_destroy_context(void)
 
 bool dinput_init_context(void)
 {
-   if (g_dinput_ctx)
-      return true;
-
-   /* Who said we shouldn't have same call signature in a COM API? <_< */
+   if (!g_dinput_ctx)
+   {
+      /* Who said we shouldn't have same call signature in a COM API? <_< */
 #ifdef __cplusplus
-   if (!(SUCCEEDED(DirectInput8Create(
-                  GetModuleHandle(NULL), DIRECTINPUT_VERSION,
-                  IID_IDirectInput8,
-                  (void**)&g_dinput_ctx, NULL))))
-#else
       if (!(SUCCEEDED(DirectInput8Create(
                      GetModuleHandle(NULL), DIRECTINPUT_VERSION,
-                     &IID_IDirectInput8,
+                     IID_IDirectInput8,
                      (void**)&g_dinput_ctx, NULL))))
+#else
+         if (!(SUCCEEDED(DirectInput8Create(
+                        GetModuleHandle(NULL), DIRECTINPUT_VERSION,
+                        &IID_IDirectInput8,
+                        (void**)&g_dinput_ctx, NULL))))
 #endif
-         return false;
-
+            return false;
+   }
    return true;
 }
 
@@ -138,8 +137,7 @@ static void *dinput_init(const char *joypad_driver)
    if (!dinput_init_context())
       return NULL;
 
-   di = (struct dinput_input*)calloc(1, sizeof(*di));
-   if (!di)
+   if (!(di = (struct dinput_input*)calloc(1, sizeof(*di))))
       return NULL;
 
    if (!string_is_empty(joypad_driver))
@@ -307,13 +305,14 @@ static void dinput_poll(void *data)
       POINT point;
       DIMOUSESTATE2 mouse_state;
       BYTE *rgb_buttons_ptr     = &mouse_state.rgbButtons[0];
-      
-      point.x = 0;
-      point.y = 0;
+      bool swap_mouse_buttons   = (g_win32_flags & WIN32_CMN_FLAG_SWAP_MOUSE_BTNS) ? true : false;
 
-      mouse_state.lX = 0;
-      mouse_state.lY = 0;
-      mouse_state.lZ = 0;
+      point.x                   = 0;
+      point.y                   = 0;
+
+      mouse_state.lX            = 0;
+      mouse_state.lY            = 0;
+      mouse_state.lZ            = 0;
 
       for (
             ; rgb_buttons_ptr < mouse_state.rgbButtons + 8
@@ -340,32 +339,59 @@ static void dinput_poll(void *data)
       di->mouse_rel_x = mouse_state.lX;
       di->mouse_rel_y = mouse_state.lY;
 
-      if (!mouse_state.rgbButtons[0])
-         di->flags &= ~DINP_FLAG_DBCLK_ON_TITLEBAR;
+      if (swap_mouse_buttons)
+      {
+         if (!mouse_state.rgbButtons[1])
+            di->flags &= ~DINP_FLAG_DBCLK_ON_TITLEBAR;
 
-      if (di->flags & DINP_FLAG_DBCLK_ON_TITLEBAR)
-         di->flags &= ~DINP_FLAG_MOUSE_L_BTN;
+         if (di->flags & DINP_FLAG_DBCLK_ON_TITLEBAR)
+            di->flags &= ~DINP_FLAG_MOUSE_R_BTN;
+         else
+         {
+            if (mouse_state.rgbButtons[0])
+               di->flags |=  DINP_FLAG_MOUSE_R_BTN;
+            else
+               di->flags &= ~DINP_FLAG_MOUSE_R_BTN;
+         }
+
+         if (mouse_state.rgbButtons[1])
+            di->flags    |=  DINP_FLAG_MOUSE_L_BTN;
+         else
+            di->flags    &= ~DINP_FLAG_MOUSE_L_BTN;
+      }
       else
       {
-         if (mouse_state.rgbButtons[0])
-            di->flags |=  DINP_FLAG_MOUSE_L_BTN;
-         else
+         if (!mouse_state.rgbButtons[0])
+            di->flags &= ~DINP_FLAG_DBCLK_ON_TITLEBAR;
+
+         if (di->flags & DINP_FLAG_DBCLK_ON_TITLEBAR)
             di->flags &= ~DINP_FLAG_MOUSE_L_BTN;
+         else
+         {
+            if (mouse_state.rgbButtons[0])
+               di->flags |=  DINP_FLAG_MOUSE_L_BTN;
+            else
+               di->flags &= ~DINP_FLAG_MOUSE_L_BTN;
+         }
+
+         if (mouse_state.rgbButtons[1])
+            di->flags    |=  DINP_FLAG_MOUSE_R_BTN;
+         else
+            di->flags    &= ~DINP_FLAG_MOUSE_R_BTN;
       }
-      if (mouse_state.rgbButtons[1])
-         di->flags    |= DINP_FLAG_MOUSE_R_BTN;
-      else
-         di->flags    &= ~DINP_FLAG_MOUSE_R_BTN;
+
       if (mouse_state.rgbButtons[2])
-         di->flags    |= DINP_FLAG_MOUSE_M_BTN;
+         di->flags    |=  DINP_FLAG_MOUSE_M_BTN;
       else
          di->flags    &= ~DINP_FLAG_MOUSE_M_BTN;
+
       if (mouse_state.rgbButtons[3])
-         di->flags    |= DINP_FLAG_MOUSE_B4_BTN;
+         di->flags    |=  DINP_FLAG_MOUSE_B4_BTN;
       else
          di->flags    &= ~DINP_FLAG_MOUSE_B4_BTN;
+
       if (mouse_state.rgbButtons[4])
-         di->flags    |= DINP_FLAG_MOUSE_B5_BTN;
+         di->flags    |=  DINP_FLAG_MOUSE_B5_BTN;
       else
          di->flags    &= ~DINP_FLAG_MOUSE_B5_BTN;
 
@@ -381,39 +407,39 @@ static void dinput_poll(void *data)
 static bool dinput_mouse_button_pressed(
       struct dinput_input *di, unsigned port, unsigned key)
 {
-	bool result;
+   bool result = false;
 
-	switch (key)
+   switch (key)
    {
       case RETRO_DEVICE_ID_MOUSE_LEFT:
-         return (di->flags & DINP_FLAG_MOUSE_L_BTN);
+         return (di->flags & DINP_FLAG_MOUSE_L_BTN)  ? true : false;
       case RETRO_DEVICE_ID_MOUSE_RIGHT:
-         return (di->flags & DINP_FLAG_MOUSE_R_BTN);
+         return (di->flags & DINP_FLAG_MOUSE_R_BTN)  ? true : false;
       case RETRO_DEVICE_ID_MOUSE_MIDDLE:
-         return (di->flags & DINP_FLAG_MOUSE_M_BTN);
+         return (di->flags & DINP_FLAG_MOUSE_M_BTN)  ? true : false;
       case RETRO_DEVICE_ID_MOUSE_BUTTON_4:
-         return (di->flags & DINP_FLAG_MOUSE_B4_BTN);
+         return (di->flags & DINP_FLAG_MOUSE_B4_BTN) ? true : false;
       case RETRO_DEVICE_ID_MOUSE_BUTTON_5:
-         return (di->flags & DINP_FLAG_MOUSE_B5_BTN);
+         return (di->flags & DINP_FLAG_MOUSE_B5_BTN) ? true : false;
       case RETRO_DEVICE_ID_MOUSE_WHEELUP:
-         result        = di->flags & DINP_FLAG_MOUSE_WU_BTN;
+         result        = (di->flags & DINP_FLAG_MOUSE_WU_BTN)  ? true : false;
          di->flags    &= ~DINP_FLAG_MOUSE_WU_BTN;
-         return result;
+         break;
       case RETRO_DEVICE_ID_MOUSE_WHEELDOWN:
-         result        = di->flags & DINP_FLAG_MOUSE_WD_BTN;
+         result        = (di->flags & DINP_FLAG_MOUSE_WD_BTN)  ? true : false;
          di->flags    &= ~DINP_FLAG_MOUSE_WD_BTN;
-         return result;
+         break;
       case RETRO_DEVICE_ID_MOUSE_HORIZ_WHEELUP:
-         result        = di->flags & DINP_FLAG_MOUSE_HWU_BTN;
+         result        = (di->flags & DINP_FLAG_MOUSE_HWU_BTN) ? true : false;
          di->flags    &= ~DINP_FLAG_MOUSE_HWU_BTN;
-         return result;
+         break;
       case RETRO_DEVICE_ID_MOUSE_HORIZ_WHEELDOWN:
-         result        = di->flags & DINP_FLAG_MOUSE_HWD_BTN;
+         result        = (di->flags & DINP_FLAG_MOUSE_HWD_BTN) ? true : false;
          di->flags    &= ~DINP_FLAG_MOUSE_HWD_BTN;
-         return result;
+         break;
    }
 
-	return false;
+   return result;
 }
 
 static int16_t dinput_lightgun_aiming_state(
@@ -421,7 +447,6 @@ static int16_t dinput_lightgun_aiming_state(
 {
    struct video_viewport vp;
    const int edge_detect       = 32700;
-   bool inside                 = false;
    int16_t res_x               = 0;
    int16_t res_y               = 0;
    int16_t res_screen_x        = 0;
@@ -431,7 +456,7 @@ static int16_t dinput_lightgun_aiming_state(
    int y                       = 0;
    unsigned num                = 0;
 
-   struct dinput_pointer_status 
+   struct dinput_pointer_status
       *check_pos               = di->pointer_head.next;
 
    vp.x                        = 0;
@@ -444,7 +469,7 @@ static int16_t dinput_lightgun_aiming_state(
    while (check_pos && num < idx)
    {
       num++;
-      check_pos = check_pos->next;
+      check_pos                = check_pos->next;
    }
 
    if (!check_pos && idx > 0) /* idx = 0 has mouse fallback. */
@@ -459,30 +484,31 @@ static int16_t dinput_lightgun_aiming_state(
       y = check_pos->pointer_y;
    }
 
-   if (!(video_driver_translate_coord_viewport_wrap(
+   if (video_driver_translate_coord_viewport_wrap(
                &vp, x, y,
-               &res_x, &res_y, &res_screen_x, &res_screen_y)))
-      return 0;
-
-   inside =    (res_x >= -edge_detect) 
-            && (res_y >= -edge_detect)
-            && (res_x <= edge_detect)
-            && (res_y <= edge_detect);
-
-   switch ( id )
+               &res_x, &res_y, &res_screen_x, &res_screen_y))
    {
-      case RETRO_DEVICE_ID_LIGHTGUN_SCREEN_X:
-         if (inside)
-            return res_x;
-         break;
-      case RETRO_DEVICE_ID_LIGHTGUN_SCREEN_Y:
-         if (inside)
-            return res_y;
-         break;
-      case RETRO_DEVICE_ID_LIGHTGUN_IS_OFFSCREEN:
-         return !inside;
-      default:
-         break;
+      bool inside =
+            (res_x >= -edge_detect)
+         && (res_y >= -edge_detect)
+         && (res_x <=  edge_detect)
+         && (res_y <=  edge_detect);
+
+      switch (id)
+      {
+         case RETRO_DEVICE_ID_LIGHTGUN_SCREEN_X:
+            if (inside)
+               return res_x;
+            break;
+         case RETRO_DEVICE_ID_LIGHTGUN_SCREEN_Y:
+            if (inside)
+               return res_y;
+            break;
+         case RETRO_DEVICE_ID_LIGHTGUN_IS_OFFSCREEN:
+            return !inside;
+         default:
+            break;
+      }
    }
 
    return 0;
@@ -536,304 +562,304 @@ static int16_t dinput_input_state(
       unsigned id)
 {
    settings_t *settings;
-   int16_t ret                = 0;
    struct dinput_input *di    = (struct dinput_input*)data;
 
-	if (port >= MAX_USERS)
-		return 0;
-
-   switch (device)
+   if (port < MAX_USERS)
    {
-      case RETRO_DEVICE_JOYPAD:
-         {
-            settings                   = config_get_ptr();
-
-            if (id == RETRO_DEVICE_ID_JOYPAD_MASK)
+      switch (device)
+      {
+         case RETRO_DEVICE_JOYPAD:
             {
-               unsigned i;
+               int16_t ret = 0;
+               settings    = config_get_ptr();
 
-               if (settings->uints.input_mouse_index[port] == 0)
+               if (id == RETRO_DEVICE_ID_JOYPAD_MASK)
                {
-                  for (i = 0; i < RARCH_FIRST_CUSTOM_BIND; i++)
+                  unsigned i;
+
+                  if (settings->uints.input_mouse_index[port] == 0)
                   {
-                     if (binds[port][i].valid)
+                     for (i = 0; i < RARCH_FIRST_CUSTOM_BIND; i++)
                      {
-                        if (dinput_mouse_button_pressed(
-                                 di, port, binds[port][i].mbutton)
-                           )
-                           ret |= (1 << i);
+                        if (binds[port][i].valid)
+                        {
+                           if (dinput_mouse_button_pressed(di, port, binds[port][i].mbutton))
+                              ret |= (1 << i);
+                        }
                      }
                   }
-               }
 
-               if (!keyboard_mapping_blocked)
-               {
-                  for (i = 0; i < RARCH_FIRST_CUSTOM_BIND; i++)
+                  if (!keyboard_mapping_blocked)
                   {
-                     if (binds[port][i].valid)
+                     for (i = 0; i < RARCH_FIRST_CUSTOM_BIND; i++)
                      {
-                        if ((binds[port][i].key < RETROK_LAST) &&
-                              di->state[rarch_keysym_lut
-                              [(enum retro_key)binds[port][i].key]] & 0x80)
-                           ret |= (1 << i);
+                        if (binds[port][i].valid)
+                        {
+                           if (     (binds[port][i].key && binds[port][i].key < RETROK_LAST)
+                                 && di->state[rarch_keysym_lut[(enum retro_key)binds[port][i].key]] & 0x80)
+                              ret |= (1 << i);
+                        }
                      }
                   }
-               }
-               return ret;
-            }
 
-            if (id < RARCH_BIND_LIST_END)
-            {
-               if (binds[port][id].valid)
+                  return ret;
+               }
+
+               if (id < RARCH_BIND_LIST_END)
                {
-                  if  (binds[port][id].key < RETROK_LAST
-                        && (di->state[rarch_keysym_lut
-                           [(enum retro_key)binds[port][id].key]] & 0x80)
-                        && (   (id == RARCH_GAME_FOCUS_TOGGLE) 
-                           || !keyboard_mapping_blocked)
-                      )
-                     return 1;
-                  else if (
-                        settings->uints.input_mouse_index[port] == 0
-                        && dinput_mouse_button_pressed(
-                           di, port, binds[port][id].mbutton)
+                  if (binds[port][id].valid)
+                  {
+                     if (     binds[port][id].key && binds[port][id].key < RETROK_LAST
+                           && (di->state[rarch_keysym_lut[(enum retro_key)binds[port][id].key]] & 0x80)
+                           && (id == RARCH_GAME_FOCUS_TOGGLE || !keyboard_mapping_blocked)
                         )
-                     return 1;
-               }
-            }
-         }
-         break;
-      case RETRO_DEVICE_KEYBOARD:
-         return (id < RETROK_LAST) && 
-            di->state[rarch_keysym_lut[(enum retro_key)id]] & 0x80;
-      case RETRO_DEVICE_ANALOG:
-         {
-            int id_minus_key      = 0;
-            int id_plus_key       = 0;
-            unsigned id_minus     = 0;
-            unsigned id_plus      = 0;
-            bool id_plus_valid    = false;
-            bool id_minus_valid   = false;
-
-            input_conv_analog_id_to_bind_id(idx, id, id_minus, id_plus);
-
-            id_minus_valid        = binds[port][id_minus].valid;
-            id_plus_valid         = binds[port][id_plus].valid;
-            id_minus_key          = binds[port][id_minus].key;
-            id_plus_key           = binds[port][id_plus].key;
-
-            if (id_plus_valid && id_plus_key < RETROK_LAST)
-            {
-               unsigned sym = rarch_keysym_lut[(enum retro_key)id_plus_key];
-               if (di->state[sym] & 0x80)
-                  ret = 0x7fff;
-            }
-            if (id_minus_valid && id_minus_key < RETROK_LAST)
-            {
-               unsigned sym = rarch_keysym_lut[(enum retro_key)id_minus_key];
-               if (di->state[sym] & 0x80)
-                  ret += -0x7fff;
-            }
-         }
-         return ret;
-      case RARCH_DEVICE_MOUSE_SCREEN:
-         settings                   = config_get_ptr();
-         if (settings->uints.input_mouse_index[ port ] != 0)
-            break;
-
-         switch (id)
-         {
-            case RETRO_DEVICE_ID_MOUSE_X:
-               return di->mouse_x;
-            case RETRO_DEVICE_ID_MOUSE_Y:
-               return di->mouse_y;
-            default:
-               break;
-         }
-         /* fall-through */
-      case RETRO_DEVICE_MOUSE:
-         settings                   = config_get_ptr();
-         if (settings->uints.input_mouse_index[port] == 0)
-         {
-            int16_t        state = 0;
-
-            switch (id)
-            {
-               case RETRO_DEVICE_ID_MOUSE_X:
-                  return di->mouse_rel_x;
-               case RETRO_DEVICE_ID_MOUSE_Y:
-                  return di->mouse_rel_y;
-               case RETRO_DEVICE_ID_MOUSE_LEFT:
-                  return (di->flags & DINP_FLAG_MOUSE_L_BTN);
-               case RETRO_DEVICE_ID_MOUSE_RIGHT:
-                  return (di->flags & DINP_FLAG_MOUSE_R_BTN);
-               case RETRO_DEVICE_ID_MOUSE_WHEELUP:
-                  if (di->flags & DINP_FLAG_MOUSE_WU_BTN)
-                     state = 1;
-                  di->flags &= ~DINP_FLAG_MOUSE_WU_BTN;
-                  return state;
-               case RETRO_DEVICE_ID_MOUSE_WHEELDOWN:
-                  if (di->flags & DINP_FLAG_MOUSE_WD_BTN)
-                     state = 1;
-                  di->flags &= ~DINP_FLAG_MOUSE_WD_BTN;
-                  return state;
-               case RETRO_DEVICE_ID_MOUSE_HORIZ_WHEELUP:
-                  if (di->flags & DINP_FLAG_MOUSE_HWU_BTN)
-                     state = 1;
-                  di->flags &= ~DINP_FLAG_MOUSE_HWU_BTN;
-                  return state;
-               case RETRO_DEVICE_ID_MOUSE_HORIZ_WHEELDOWN:
-                  if (di->flags & DINP_FLAG_MOUSE_HWD_BTN)
-                     state = 1;
-                  di->flags &= ~DINP_FLAG_MOUSE_HWD_BTN;
-                  return state;
-               case RETRO_DEVICE_ID_MOUSE_MIDDLE:
-                  return (di->flags & DINP_FLAG_MOUSE_M_BTN);
-               case RETRO_DEVICE_ID_MOUSE_BUTTON_4:
-                  return (di->flags & DINP_FLAG_MOUSE_B4_BTN);
-               case RETRO_DEVICE_ID_MOUSE_BUTTON_5:
-                  return (di->flags & DINP_FLAG_MOUSE_B5_BTN);
-            }
-         }
-         break;
-      case RETRO_DEVICE_POINTER:
-      case RARCH_DEVICE_POINTER_SCREEN:
-         {
-            struct video_viewport vp;
-            bool pointer_down           = false;
-            bool inside                 = false;
-            int x                       = 0;
-            int y                       = 0;
-            int16_t res_x               = 0;
-            int16_t res_y               = 0;
-            int16_t res_screen_x        = 0;
-            int16_t res_screen_y        = 0;
-            unsigned num                = 0;
-            struct dinput_pointer_status *
-               check_pos                = di->pointer_head.next;
-
-            vp.x                        = 0;
-            vp.y                        = 0;
-            vp.width                    = 0;
-            vp.height                   = 0;
-            vp.full_width               = 0;
-            vp.full_height              = 0;
-
-            while (check_pos && num < idx)
-            {
-               num++;
-               check_pos    = check_pos->next;
-            }
-            if (!check_pos && idx > 0) /* idx = 0 has mouse fallback. */
-               return 0;
-
-            x               = di->mouse_x;
-            y               = di->mouse_y;
-            pointer_down    = di->flags & DINP_FLAG_MOUSE_L_BTN;
-
-            if (check_pos)
-            {
-               x            = check_pos->pointer_x;
-               y            = check_pos->pointer_y;
-               pointer_down = true;
-            }
-
-            if (!(video_driver_translate_coord_viewport_wrap(&vp, x, y,
-                        &res_x, &res_y, &res_screen_x, &res_screen_y)))
-               return 0;
-
-            if (device == RARCH_DEVICE_POINTER_SCREEN)
-            {
-               res_x        = res_screen_x;
-               res_y        = res_screen_y;
-            }
-
-            if (!(inside = (res_x >= -0x7fff) && (res_y >= -0x7fff)))
-               return 0;
-
-            switch (id)
-            {
-               case RETRO_DEVICE_ID_POINTER_X:
-                  return res_x;
-               case RETRO_DEVICE_ID_POINTER_Y:
-                  return res_y;
-               case RETRO_DEVICE_ID_POINTER_PRESSED:
-                  return pointer_down;
-               default:
-                  break;
-            }
-         }
-         break;
-      case RETRO_DEVICE_LIGHTGUN:
-         switch (id)
-         {
-            /*aiming*/
-            case RETRO_DEVICE_ID_LIGHTGUN_SCREEN_X:
-            case RETRO_DEVICE_ID_LIGHTGUN_SCREEN_Y:
-            case RETRO_DEVICE_ID_LIGHTGUN_IS_OFFSCREEN:
-               return dinput_lightgun_aiming_state(di, idx, id);
-
-               /*buttons*/
-            case RETRO_DEVICE_ID_LIGHTGUN_TRIGGER:
-            case RETRO_DEVICE_ID_LIGHTGUN_RELOAD:
-            case RETRO_DEVICE_ID_LIGHTGUN_AUX_A:
-            case RETRO_DEVICE_ID_LIGHTGUN_AUX_B:
-            case RETRO_DEVICE_ID_LIGHTGUN_AUX_C:
-            case RETRO_DEVICE_ID_LIGHTGUN_START:
-            case RETRO_DEVICE_ID_LIGHTGUN_SELECT:
-            case RETRO_DEVICE_ID_LIGHTGUN_DPAD_UP:
-            case RETRO_DEVICE_ID_LIGHTGUN_DPAD_DOWN:
-            case RETRO_DEVICE_ID_LIGHTGUN_DPAD_LEFT:
-            case RETRO_DEVICE_ID_LIGHTGUN_DPAD_RIGHT:
-            case RETRO_DEVICE_ID_LIGHTGUN_PAUSE:
-               {
-                  unsigned new_id                = dinput_retro_id_to_rarch(id);
-                  const uint64_t bind_joykey     = input_config_binds[port][new_id].joykey;
-                  const uint64_t bind_joyaxis    = input_config_binds[port][new_id].joyaxis;
-                  const uint64_t autobind_joykey = input_autoconf_binds[port][new_id].joykey;
-                  const uint64_t autobind_joyaxis= input_autoconf_binds[port][new_id].joyaxis;
-                  uint16_t port                  = joypad_info->joy_idx;
-                  float axis_threshold           = joypad_info->axis_threshold;
-                  const uint64_t joykey          = (bind_joykey != NO_BTN)
-                     ? bind_joykey  : autobind_joykey;
-                  const uint32_t joyaxis         = (bind_joyaxis != AXIS_NONE)
-                     ? bind_joyaxis : autobind_joyaxis;
-                  if (binds[port][new_id].valid)
-                  {
-                     if ((uint16_t)joykey != NO_BTN && joypad->button(
-                              port, (uint16_t)joykey))
                         return 1;
-                     if (joyaxis != AXIS_NONE &&
-                           ((float)abs(joypad->axis(port, joyaxis)) 
-                            / 0x8000) > axis_threshold)
-                        return 1;
-                     else if (
-                           binds[port][new_id].key < RETROK_LAST
-                           && !keyboard_mapping_blocked
-                           && di->state[rarch_keysym_lut
-                           [(enum retro_key)binds[port][new_id].key]] & 0x80
-                           )
-                        return 1;
-                     else
+                     else if (settings->uints.input_mouse_index[port] == 0)
                      {
-                        settings = config_get_ptr();
-                        if (
-                              settings->uints.input_mouse_index[port] == 0
-                              && dinput_mouse_button_pressed(
-                                 di, port, binds[port][new_id].mbutton)
-                           )
+                        if (dinput_mouse_button_pressed(di, port, binds[port][id].mbutton))
                            return 1;
                      }
                   }
                }
+            }
+            break;
+         case RETRO_DEVICE_KEYBOARD:
+            return (id && id < RETROK_LAST) && di->state[rarch_keysym_lut[(enum retro_key)id]] & 0x80;
+         case RETRO_DEVICE_ANALOG:
+            {
+               int16_t ret           = 0;
+               int id_minus_key      = 0;
+               int id_plus_key       = 0;
+               unsigned id_minus     = 0;
+               unsigned id_plus      = 0;
+               bool id_plus_valid    = false;
+               bool id_minus_valid   = false;
+
+               input_conv_analog_id_to_bind_id(idx, id, id_minus, id_plus);
+
+               id_minus_valid        = binds[port][id_minus].valid;
+               id_plus_valid         = binds[port][id_plus].valid;
+               id_minus_key          = binds[port][id_minus].key;
+               id_plus_key           = binds[port][id_plus].key;
+
+               if (id_plus_valid && id_plus_key && id_plus_key < RETROK_LAST)
+               {
+                  unsigned sym = rarch_keysym_lut[(enum retro_key)id_plus_key];
+                  if (di->state[sym] & 0x80)
+                     ret = 0x7fff;
+               }
+               if (id_minus_valid && id_minus_key && id_minus_key < RETROK_LAST)
+               {
+                  unsigned sym = rarch_keysym_lut[(enum retro_key)id_minus_key];
+                  if (di->state[sym] & 0x80)
+                     ret += -0x7fff;
+               }
+               return ret;
+            }
+            break;
+         case RARCH_DEVICE_MOUSE_SCREEN:
+            settings                   = config_get_ptr();
+            if (settings->uints.input_mouse_index[port] != 0)
                break;
-               /*deprecated*/
-            case RETRO_DEVICE_ID_LIGHTGUN_X:
-               return di->mouse_rel_x;
-            case RETRO_DEVICE_ID_LIGHTGUN_Y:
-               return di->mouse_rel_y;
-         }
-			break;
+
+            switch (id)
+            {
+               case RETRO_DEVICE_ID_MOUSE_X:
+                  return di->mouse_x;
+               case RETRO_DEVICE_ID_MOUSE_Y:
+                  return di->mouse_y;
+               default:
+                  break;
+            }
+            /* fall-through */
+         case RETRO_DEVICE_MOUSE:
+            settings                   = config_get_ptr();
+            if (settings->uints.input_mouse_index[port] == 0)
+            {
+               switch (id)
+               {
+                  case RETRO_DEVICE_ID_MOUSE_X:
+                     return di->mouse_rel_x;
+                  case RETRO_DEVICE_ID_MOUSE_Y:
+                     return di->mouse_rel_y;
+                  case RETRO_DEVICE_ID_MOUSE_LEFT:
+                     return (di->flags & DINP_FLAG_MOUSE_L_BTN) > 0;
+                  case RETRO_DEVICE_ID_MOUSE_RIGHT:
+                     return (di->flags & DINP_FLAG_MOUSE_R_BTN) > 0;
+                  case RETRO_DEVICE_ID_MOUSE_WHEELUP:
+                     if (di->flags & DINP_FLAG_MOUSE_WU_BTN)
+                     {
+                        di->flags &= ~DINP_FLAG_MOUSE_WU_BTN;
+                        return 1;
+                     }
+                     di->flags &= ~DINP_FLAG_MOUSE_WU_BTN;
+                     break;
+                  case RETRO_DEVICE_ID_MOUSE_WHEELDOWN:
+                     if (di->flags & DINP_FLAG_MOUSE_WD_BTN)
+                     {
+                        di->flags &= ~DINP_FLAG_MOUSE_WD_BTN;
+                        return 1;
+                     }
+                     di->flags &= ~DINP_FLAG_MOUSE_WD_BTN;
+                     break;
+                  case RETRO_DEVICE_ID_MOUSE_HORIZ_WHEELUP:
+                     if (di->flags & DINP_FLAG_MOUSE_HWU_BTN)
+                     {
+                        di->flags &= ~DINP_FLAG_MOUSE_HWU_BTN;
+                        return 1;
+                     }
+                     di->flags &= ~DINP_FLAG_MOUSE_HWU_BTN;
+                     break;
+                  case RETRO_DEVICE_ID_MOUSE_HORIZ_WHEELDOWN:
+                     if (di->flags & DINP_FLAG_MOUSE_HWD_BTN)
+                     {
+                        di->flags &= ~DINP_FLAG_MOUSE_HWD_BTN;
+                        return 1;
+                     }
+                     di->flags &= ~DINP_FLAG_MOUSE_HWD_BTN;
+                     break;
+                  case RETRO_DEVICE_ID_MOUSE_MIDDLE:
+                     return (di->flags & DINP_FLAG_MOUSE_M_BTN) > 0;
+                  case RETRO_DEVICE_ID_MOUSE_BUTTON_4:
+                     return (di->flags & DINP_FLAG_MOUSE_B4_BTN) > 0;
+                  case RETRO_DEVICE_ID_MOUSE_BUTTON_5:
+                     return (di->flags & DINP_FLAG_MOUSE_B5_BTN) > 0;
+               }
+            }
+            break;
+         case RETRO_DEVICE_POINTER:
+         case RARCH_DEVICE_POINTER_SCREEN:
+            {
+               struct video_viewport vp;
+               bool inside                 = false;
+               int x                       = 0;
+               int y                       = 0;
+               int16_t res_x               = 0;
+               int16_t res_y               = 0;
+               int16_t res_screen_x        = 0;
+               int16_t res_screen_y        = 0;
+               unsigned num                = 0;
+               struct dinput_pointer_status *
+                  check_pos                = di->pointer_head.next;
+
+               vp.x                        = 0;
+               vp.y                        = 0;
+               vp.width                    = 0;
+               vp.height                   = 0;
+               vp.full_width               = 0;
+               vp.full_height              = 0;
+
+               while (check_pos && num < idx)
+               {
+                  num++;
+                  check_pos    = check_pos->next;
+               }
+               if (!check_pos && idx > 0) /* idx = 0 has mouse fallback. */
+                  return 0;
+
+               x               = di->mouse_x;
+               y               = di->mouse_y;
+
+               if (check_pos)
+               {
+                  x            = check_pos->pointer_x;
+                  y            = check_pos->pointer_y;
+               }
+
+               if (video_driver_translate_coord_viewport_wrap(&vp, x, y,
+                           &res_x, &res_y, &res_screen_x, &res_screen_y))
+               {
+                  if (device == RARCH_DEVICE_POINTER_SCREEN)
+                  {
+                     res_x        = res_screen_x;
+                     res_y        = res_screen_y;
+                  }
+
+                  if ((inside = (res_x >= -0x7fff) && (res_y >= -0x7fff)))
+                  {
+                     switch (id)
+                     {
+                        case RETRO_DEVICE_ID_POINTER_X:
+                           return res_x;
+                        case RETRO_DEVICE_ID_POINTER_Y:
+                           return res_y;
+                        case RETRO_DEVICE_ID_POINTER_PRESSED:
+                           return check_pos ? 1 : (di->flags & DINP_FLAG_MOUSE_L_BTN);
+                        default:
+                           break;
+                     }
+                  }
+               }
+            }
+            break;
+         case RETRO_DEVICE_LIGHTGUN:
+            switch (id)
+            {
+               /*aiming*/
+               case RETRO_DEVICE_ID_LIGHTGUN_SCREEN_X:
+               case RETRO_DEVICE_ID_LIGHTGUN_SCREEN_Y:
+               case RETRO_DEVICE_ID_LIGHTGUN_IS_OFFSCREEN:
+                  return dinput_lightgun_aiming_state(di, idx, id);
+
+                  /*buttons*/
+               case RETRO_DEVICE_ID_LIGHTGUN_TRIGGER:
+               case RETRO_DEVICE_ID_LIGHTGUN_RELOAD:
+               case RETRO_DEVICE_ID_LIGHTGUN_AUX_A:
+               case RETRO_DEVICE_ID_LIGHTGUN_AUX_B:
+               case RETRO_DEVICE_ID_LIGHTGUN_AUX_C:
+               case RETRO_DEVICE_ID_LIGHTGUN_START:
+               case RETRO_DEVICE_ID_LIGHTGUN_SELECT:
+               case RETRO_DEVICE_ID_LIGHTGUN_DPAD_UP:
+               case RETRO_DEVICE_ID_LIGHTGUN_DPAD_DOWN:
+               case RETRO_DEVICE_ID_LIGHTGUN_DPAD_LEFT:
+               case RETRO_DEVICE_ID_LIGHTGUN_DPAD_RIGHT:
+               case RETRO_DEVICE_ID_LIGHTGUN_PAUSE:
+                  {
+                     unsigned new_id                = dinput_retro_id_to_rarch(id);
+                     const uint64_t bind_joykey     = input_config_binds[port][new_id].joykey;
+                     const uint64_t bind_joyaxis    = input_config_binds[port][new_id].joyaxis;
+                     const uint64_t autobind_joykey = input_autoconf_binds[port][new_id].joykey;
+                     const uint64_t autobind_joyaxis= input_autoconf_binds[port][new_id].joyaxis;
+                     uint16_t joyport               = joypad_info->joy_idx;
+                     float axis_threshold           = joypad_info->axis_threshold;
+                     const uint64_t joykey          = (bind_joykey != NO_BTN)
+                        ? bind_joykey  : autobind_joykey;
+                     const uint32_t joyaxis         = (bind_joyaxis != AXIS_NONE)
+                        ? bind_joyaxis : autobind_joyaxis;
+
+                     if (binds[port][new_id].valid)
+                     {
+                        if ((uint16_t)joykey != NO_BTN && joypad->button(
+                                 joyport, (uint16_t)joykey))
+                           return 1;
+                        if (joyaxis != AXIS_NONE &&
+                              ((float)abs(joypad->axis(joyport, joyaxis))
+                               / 0x8000) > axis_threshold)
+                           return 1;
+                        else if ((binds[port][new_id].key && binds[port][new_id].key < RETROK_LAST)
+                              && !keyboard_mapping_blocked
+                              && di->state[rarch_keysym_lut[(enum retro_key)binds[port][new_id].key]] & 0x80)
+                           return 1;
+                        else
+                        {
+                           settings = config_get_ptr();
+                           if (settings->uints.input_mouse_index[port] == 0)
+                           {
+                              if (dinput_mouse_button_pressed(di, port, binds[port][new_id].mbutton))
+                                 return 1;
+                           }
+                        }
+                     }
+                  }
+                  break;
+                  /*deprecated*/
+               case RETRO_DEVICE_ID_LIGHTGUN_X:
+                  return di->mouse_rel_x;
+               case RETRO_DEVICE_ID_LIGHTGUN_Y:
+                  return di->mouse_rel_y;
+            }
+            break;
+      }
    }
 
    return 0;
@@ -857,14 +883,13 @@ static int16_t dinput_input_state(
 #define GET_POINTERID_WPARAM(wParam)   (LOWORD(wParam))
 #endif
 
-/* Stores x/y in client coordinates. */
+/* Stores X/Y in client coordinates. */
 static void dinput_pointer_store_pos(
       struct dinput_pointer_status *pointer, WPARAM lParam)
 {
    POINT point;
-
-   point.x = GET_X_LPARAM(lParam);
-   point.y = GET_Y_LPARAM(lParam);
+   point.x            = GET_X_LPARAM(lParam);
+   point.y            = GET_Y_LPARAM(lParam);
    ScreenToClient((HWND)video_driver_window_get(), &point);
    pointer->pointer_x = point.x;
    pointer->pointer_y = point.y;
@@ -875,12 +900,12 @@ static void dinput_add_pointer(struct dinput_input *di,
 {
    struct dinput_pointer_status *insert_pos = NULL;
 
-   new_pointer->next                 = NULL;
-   insert_pos                        = &di->pointer_head;
+   new_pointer->next                        = NULL;
+   insert_pos                               = &di->pointer_head;
 
    while (insert_pos->next)
-      insert_pos                     = insert_pos->next;
-   insert_pos->next                  = new_pointer;
+      insert_pos                            = insert_pos->next;
+   insert_pos->next                         = new_pointer;
 }
 
 static void dinput_delete_pointer(struct dinput_input *di, int pointer_id)
@@ -920,7 +945,7 @@ static void dinput_clear_pointers(struct dinput_input *di)
 
    while (pointer->next)
    {
-      struct dinput_pointer_status *del = pointer->next;
+      struct dinput_pointer_status *del  = pointer->next;
 
       pointer->next = pointer->next->next;
       free(del);
@@ -953,45 +978,39 @@ bool dinput_handle_message(void *data,
             struct dinput_pointer_status *new_pointer =
                (struct dinput_pointer_status *)malloc(sizeof(struct dinput_pointer_status));
 
-            if (!new_pointer)
-               return false;
-
-            new_pointer->pointer_id = GET_POINTERID_WPARAM(wParam);
-            dinput_pointer_store_pos(new_pointer, lParam);
-            dinput_add_pointer(di, new_pointer);
-            return true;
+            if (new_pointer)
+            {
+               new_pointer->pointer_id = GET_POINTERID_WPARAM(wParam);
+               dinput_pointer_store_pos(new_pointer, lParam);
+               dinput_add_pointer(di, new_pointer);
+               return true;
+            }
          }
+         break;
       case WM_POINTERUP:
          {
             int pointer_id = GET_POINTERID_WPARAM(wParam);
             dinput_delete_pointer(di, pointer_id);
-            return true;
          }
+         return true;
       case WM_POINTERUPDATE:
          {
             int pointer_id                 = GET_POINTERID_WPARAM(wParam);
             struct dinput_pointer_status *pointer = dinput_find_pointer(di, pointer_id);
             if (pointer)
                dinput_pointer_store_pos(pointer, lParam);
-            return true;
          }
+         return true;
       case WM_DEVICECHANGE:
 #if defined(_WIN32_WINNT) && _WIN32_WINNT >= 0x0500 /* 2K */
-         if (  wParam == DBT_DEVICEARRIVAL  || 
+         if (  wParam == DBT_DEVICEARRIVAL  ||
                wParam == DBT_DEVICEREMOVECOMPLETE)
          {
             PDEV_BROADCAST_HDR pHdr = (PDEV_BROADCAST_HDR)lParam;
-            if(pHdr->dbch_devicetype == DBT_DEVTYP_DEVICEINTERFACE)
-            {
-#if 0
-               PDEV_BROADCAST_DEVICEINTERFACE pDevInf = 
-                  (PDEV_BROADCAST_DEVICEINTERFACE)pHdr;
-#endif
-
-               /* TODO/FIXME: Don't destroy everything, let's just 
-                * handle new devices gracefully */
+            /* TODO/FIXME: Don't destroy everything, let's just
+             * handle new devices gracefully */
+            if (pHdr->dbch_devicetype == DBT_DEVTYP_DEVICEINTERFACE)
                joypad_driver_reinit(di, di->joypad_drv_name);
-            }
          }
 #endif
          break;
