@@ -19,7 +19,6 @@
 #include <string.h>
 
 #include <boolean.h>
-#include <retro_assert.h>
 #include <retro_miscellaneous.h>
 #include <encodings/crc32.h>
 #include <encodings/utf.h>
@@ -205,12 +204,8 @@ static void rwebinput_generate_lut(void)
    int i;
    struct rarch_key_map *key_map;
 
-   retro_assert(ARRAY_SIZE(rarch_key_map_rwebinput) ==
-      ARRAY_SIZE(rwebinput_key_to_code_map) + 1);
-
    for (i = 0; i < ARRAY_SIZE(rwebinput_key_to_code_map); i++)
    {
-      int j;
       uint32_t crc;
       const rwebinput_key_to_code_map_entry_t *key_to_code =
          &rwebinput_key_to_code_map[i];
@@ -218,10 +213,6 @@ static void rwebinput_generate_lut(void)
       crc = encoding_crc32(0, (const uint8_t *)key_to_code->key,
          strlen(key_to_code->key));
 
-      /* sanity check: make sure there's no collisions */
-      for (j = 0; j < i; j++)
-         retro_assert(rarch_key_map_rwebinput[j].sym != crc);
-
       key_map->rk  = key_to_code->rk;
       key_map->sym = crc;
    }
@@ -246,7 +237,6 @@ static EM_BOOL rwebinput_keyboard_cb(int event_type,
       size_t new_max = MAX(1, rwebinput->keyboard.max_size << 1);
       rwebinput->keyboard.events = realloc(rwebinput->keyboard.events,
          new_max * sizeof(rwebinput->keyboard.events[0]));
-      retro_assert(rwebinput->keyboard.events != NULL);
       rwebinput->keyboard.max_size = new_max;
    }
 
@@ -301,7 +291,7 @@ static void *rwebinput_input_init(const char *joypad_driver)
    rwebinput_generate_lut();
 
    r = emscripten_set_keydown_callback(
-         EMSCRIPTEN_EVENT_TARGET_DOCUMENT, rwebinput, false,
+         "!canvas", rwebinput, false,
          rwebinput_keyboard_cb);
    if (r != EMSCRIPTEN_RESULT_SUCCESS)
    {
@@ -310,7 +300,7 @@ static void *rwebinput_input_init(const char *joypad_driver)
    }
 
    r = emscripten_set_keyup_callback(
-         EMSCRIPTEN_EVENT_TARGET_DOCUMENT, rwebinput, false,
+         "!canvas", rwebinput, false,
          rwebinput_keyboard_cb);
    if (r != EMSCRIPTEN_RESULT_SUCCESS)
    {
@@ -319,7 +309,7 @@ static void *rwebinput_input_init(const char *joypad_driver)
    }
 
    r = emscripten_set_keypress_callback(
-         EMSCRIPTEN_EVENT_TARGET_DOCUMENT, rwebinput, false,
+         "!canvas", rwebinput, false,
          rwebinput_keyboard_cb);
    if (r != EMSCRIPTEN_RESULT_SUCCESS)
    {
@@ -327,7 +317,7 @@ static void *rwebinput_input_init(const char *joypad_driver)
          "[EMSCRIPTEN/INPUT] failed to create keypress callback: %d\n", r);
    }
 
-   r = emscripten_set_mousedown_callback("#canvas", rwebinput, false,
+   r = emscripten_set_mousedown_callback("!canvas", rwebinput, false,
          rwebinput_mouse_cb);
    if (r != EMSCRIPTEN_RESULT_SUCCESS)
    {
@@ -335,7 +325,7 @@ static void *rwebinput_input_init(const char *joypad_driver)
          "[EMSCRIPTEN/INPUT] failed to create mousedown callback: %d\n", r);
    }
 
-   r = emscripten_set_mouseup_callback("#canvas", rwebinput, false,
+   r = emscripten_set_mouseup_callback("!canvas", rwebinput, false,
          rwebinput_mouse_cb);
    if (r != EMSCRIPTEN_RESULT_SUCCESS)
    {
@@ -343,7 +333,7 @@ static void *rwebinput_input_init(const char *joypad_driver)
          "[EMSCRIPTEN/INPUT] failed to create mouseup callback: %d\n", r);
    }
 
-   r = emscripten_set_mousemove_callback("#canvas", rwebinput, false,
+   r = emscripten_set_mousemove_callback("!canvas", rwebinput, false,
          rwebinput_mouse_cb);
    if (r != EMSCRIPTEN_RESULT_SUCCESS)
    {
@@ -352,7 +342,7 @@ static void *rwebinput_input_init(const char *joypad_driver)
    }
 
    r = emscripten_set_wheel_callback(
-         EMSCRIPTEN_EVENT_TARGET_DOCUMENT, rwebinput, false,
+         "!canvas", rwebinput, false,
          rwebinput_wheel_cb);
    if (r != EMSCRIPTEN_RESULT_SUCCESS)
    {
@@ -394,13 +384,13 @@ static int16_t rwebinput_mouse_state(
       case RETRO_DEVICE_ID_MOUSE_BUTTON_5:
          return !!(mouse->buttons & (1 << RWEBINPUT_MOUSE_BTN5));
       case RETRO_DEVICE_ID_MOUSE_WHEELUP:
-         return mouse->scroll_y < 0.0;
+         return mouse->scroll_y < 0.0f;
       case RETRO_DEVICE_ID_MOUSE_WHEELDOWN:
-         return mouse->scroll_y > 0.0;
+         return mouse->scroll_y > 0.0f;
       case RETRO_DEVICE_ID_MOUSE_HORIZ_WHEELUP:
-         return mouse->scroll_x < 0.0;
+         return mouse->scroll_x < 0.0f;
       case RETRO_DEVICE_ID_MOUSE_HORIZ_WHEELDOWN:
-         return mouse->scroll_x > 0.0;
+         return mouse->scroll_x > 0.0f;
    }
 
    return 0;
@@ -415,11 +405,12 @@ static int16_t rwebinput_is_pressed(
    const struct retro_keybind *bind = &binds[id];
    int key                          = bind->key;
 
-   if ((key < RETROK_LAST) && rwebinput_key_pressed(rwebinput, key))
-      if ((id == RARCH_GAME_FOCUS_TOGGLE) || !keyboard_mapping_blocked)
-         return 1;
-   if (port == 0 && !!rwebinput_mouse_state(&rwebinput->mouse,
-            bind->mbutton, false))
+   if (     (key && key < RETROK_LAST)
+         && rwebinput_key_pressed(rwebinput, key)
+         && (id == RARCH_GAME_FOCUS_TOGGLE || !keyboard_mapping_blocked)
+      )
+      return 1;
+   if (port == 0 && !!rwebinput_mouse_state(&rwebinput->mouse, bind->mbutton, false))
       return 1;
    return 0;
 }
@@ -489,14 +480,14 @@ static int16_t rwebinput_input_state(
             id_minus_key          = binds[port][id_minus].key;
             id_plus_key           = binds[port][id_plus].key;
 
-            if (id_plus_valid && id_plus_key < RETROK_LAST)
+            if (id_plus_valid && id_plus_key && id_plus_key < RETROK_LAST)
             {
                if (rwebinput_is_pressed(rwebinput,
                         binds[port], idx, id_plus,
                         keyboard_mapping_blocked))
                   ret = 0x7fff;
             }
-            if (id_minus_valid && id_minus_key < RETROK_LAST)
+            if (id_minus_valid && id_minus_key && id_minus_key < RETROK_LAST)
             {
                if (rwebinput_is_pressed(rwebinput, 
                         binds[port], idx, id_minus,
@@ -508,11 +499,10 @@ static int16_t rwebinput_input_state(
          }
          break;
       case RETRO_DEVICE_KEYBOARD:
-         return ((id < RETROK_LAST) && rwebinput->keys[id]);
+         return (id && id < RETROK_LAST) && rwebinput->keys[id];
       case RETRO_DEVICE_MOUSE:
       case RARCH_DEVICE_MOUSE_SCREEN:
-         return rwebinput_mouse_state(&rwebinput->mouse, id,
-               device == RARCH_DEVICE_MOUSE_SCREEN);
+         return rwebinput_mouse_state(&rwebinput->mouse, id, device == RARCH_DEVICE_MOUSE_SCREEN);
       case RETRO_DEVICE_POINTER:
       case RARCH_DEVICE_POINTER_SCREEN:
          if (idx == 0)
@@ -537,7 +527,7 @@ static int16_t rwebinput_input_state(
             vp.full_height              = 0;
 
             if (!(video_driver_translate_coord_viewport_wrap(
-                        &vp, mouse->x, mouse->x,
+                        &vp, mouse->x, mouse->y,
                         &res_x, &res_y, &res_screen_x, &res_screen_y)))
                return 0;
 
@@ -661,7 +651,7 @@ static void rwebinput_input_poll(void *data)
 static void rwebinput_grab_mouse(void *data, bool state)
 {
    if (state)
-      emscripten_request_pointerlock("#canvas", EM_TRUE);
+      emscripten_request_pointerlock("!canvas", EM_TRUE);
    else
       emscripten_exit_pointerlock();
 }
