@@ -976,6 +976,7 @@ static void btpad_queue_reset(void)
    can_run         = 1;
 }
 
+#if 0
 static void btpad_queue_btstack_set_power_mode(
       struct btpad_queue_command *cmd, uint8_t on)
 {
@@ -988,6 +989,7 @@ static void btpad_queue_btstack_set_power_mode(
    btpad_increment_position(&insert_position);
    btpad_queue_process();
 }
+#endif
 
 static void btpad_set_inquiry_state(bool on)
 {
@@ -1388,7 +1390,7 @@ static int16_t btstack_hid_joypad_axis(void *data,
       if (val < 0)
          return val;
    }
-   else if(AXIS_POS_GET(joyaxis) < 4)
+   else if (AXIS_POS_GET(joyaxis) < 4)
    {
       int16_t val = pad_connection_get_axis(
             &slots[port], port, AXIS_POS_GET(joyaxis));
@@ -1409,7 +1411,6 @@ static int16_t btstack_hid_joypad_state(
    unsigned i;
    int16_t ret                          = 0;
    const struct retro_keybind *binds    = (const struct retro_keybind*)binds_data;
-   btstack_hid_t         *hid            = (btstack_hid_t*)data;
    uint16_t port_idx                     = joypad_info->joy_idx;
    joypad_connection_t              *pad = &slots[port_idx];
 
@@ -1456,7 +1457,6 @@ static void btstack_hid_free(const void *data)
    btpad_set_inquiry_state(true);
    btstack_set_poweron(false);
 
-   free(slots);
    if (hid)
       free(hid);
 }
