@@ -84,6 +84,8 @@
  * - B:      SDLK_LCTRL
  * - Y:      SDLK_SPACE
  * - Menu:   SDLK_RCTRL
+ * - L3:     SDLK_RALT
+ * - R3:     SDLK_RSHIFT
  */
 #if defined(MIYOO)
 #define SDL_DINGUX_SDLK_X      SDLK_LSHIFT
@@ -102,8 +104,13 @@
 #define SDL_DINGUX_SDLK_R2     SDLK_PAGEDOWN
 #define SDL_DINGUX_SDLK_SELECT SDLK_ESCAPE
 #define SDL_DINGUX_SDLK_START  SDLK_RETURN
+#if defined(MIYOO)
+#define SDL_DINGUX_SDLK_L3     SDLK_RALT
+#define SDL_DINGUX_SDLK_R3     SDLK_RSHIFT
+#else
 #define SDL_DINGUX_SDLK_L3     SDLK_KP_DIVIDE
 #define SDL_DINGUX_SDLK_R3     SDLK_KP_PERIOD
+#endif
 #define SDL_DINGUX_SDLK_UP     SDLK_UP
 #define SDL_DINGUX_SDLK_RIGHT  SDLK_RIGHT
 #define SDL_DINGUX_SDLK_DOWN   SDLK_DOWN
@@ -510,49 +517,48 @@ static void sdl_dingux_joypad_get_buttons(unsigned port, input_bits_t *state)
 static int16_t sdl_dingux_joypad_axis_state(unsigned port, uint32_t joyaxis)
 {
 #if defined(SDL_DINGUX_HAS_ANALOG)
-   dingux_joypad_t *joypad = (dingux_joypad_t*)&dingux_joypad;
-   int val                 = 0;
-   int axis                = -1;
-   bool is_neg             = false;
-   bool is_pos             = false;
-
-   if (port != 0)
-      return 0;
-
-   if (AXIS_NEG_GET(joyaxis) < 4)
+   if (port == 0)
    {
-      axis   = AXIS_NEG_GET(joyaxis);
-      is_neg = true;
+      dingux_joypad_t *joypad = (dingux_joypad_t*)&dingux_joypad;
+      if (AXIS_NEG_GET(joyaxis) < 4)
+      {
+         int16_t val  = 0;
+         int16_t axis = AXIS_NEG_GET(joyaxis);
+         switch (axis)
+         {
+            case 0:
+            case 1:
+               val = joypad->analog_state[0][axis];
+               break;
+            case 2:
+            case 3:
+               val = joypad->analog_state[1][axis - 2];
+               break;
+         }
+         if (val < 0)
+            return val;
+      }
+      else if (AXIS_POS_GET(joyaxis) < 4)
+      {
+         int16_t val  = 0;
+         int16_t axis = AXIS_POS_GET(joyaxis);
+         switch (axis)
+         {
+            case 0:
+            case 1:
+               val = joypad->analog_state[0][axis];
+               break;
+            case 2:
+            case 3:
+               val = joypad->analog_state[1][axis - 2];
+               break;
+         }
+         if (val > 0)
+            return val;
+      }
    }
-   else if (AXIS_POS_GET(joyaxis) < 4)
-   {
-      axis   = AXIS_POS_GET(joyaxis);
-      is_pos = true;
-   }
-   else
-      return 0;
-
-   switch (axis)
-   {
-      case 0:
-      case 1:
-         val = joypad->analog_state[0][axis];
-         break;
-      case 2:
-      case 3:
-         val = joypad->analog_state[1][axis - 2];
-         break;
-   }
-
-   if (is_neg && val > 0)
-      return 0;
-   else if (is_pos && val < 0)
-      return 0;
-
-   return val;
-#else
-   return 0;
 #endif
+   return 0;
 }
 
 static int16_t sdl_dingux_joypad_axis(unsigned port, uint32_t joyaxis)
