@@ -207,7 +207,7 @@ void manual_content_scan_scrub_system_name_custom(void)
     * and/or violate the No-Intro filename standard:
     * http://datomatic.no-intro.org/stuff/The%20Official%20No-Intro%20Convention%20(20071030).zip
     * Replace these characters with underscores */
-   while ((scrub_char_pointer = 
+   while ((scrub_char_pointer =
             strpbrk(scan_settings.system_name_custom, "&*/:`\"<>?\\|")))
       *scrub_char_pointer = '_';
 }
@@ -513,6 +513,7 @@ enum manual_content_scan_playlist_refresh_status
    bool search_recursively      = false;
    bool search_archives         = false;
    bool filter_dat_content      = false;
+   bool overwrite_playlist      = false;
 #ifdef HAVE_LIBRETRODB
    struct string_list *rdb_list = NULL;
 #endif
@@ -542,6 +543,7 @@ enum manual_content_scan_playlist_refresh_status
    search_recursively = playlist_get_scan_search_recursively(playlist);
    search_archives    = playlist_get_scan_search_archives(playlist);
    filter_dat_content = playlist_get_scan_filter_dat_content(playlist);
+   overwrite_playlist = playlist_get_scan_overwrite_playlist(playlist);
 
    /* Determine system name (playlist basename
     * without extension) */
@@ -654,8 +656,8 @@ enum manual_content_scan_playlist_refresh_status
    }
 
    /* Set core path/name */
-   if (!string_is_empty(core_name) &&
-       !string_is_equal(core_name, FILE_PATH_DETECT))
+   if (   !string_is_empty(core_name)
+       && !string_is_equal(core_name, FILE_PATH_DETECT))
       core_type = MANUAL_CONTENT_SCAN_CORE_SET;
 
    if (!manual_content_scan_set_menu_core_name(
@@ -712,11 +714,10 @@ enum manual_content_scan_playlist_refresh_status
    scan_settings.search_recursively = search_recursively;
    scan_settings.search_archives    = search_archives;
    scan_settings.filter_dat_content = filter_dat_content;
+   scan_settings.overwrite_playlist = overwrite_playlist;
    /* When refreshing a playlist:
-    * > We never overwrite the existing file
     * > We always validate entries in the
     *   existing file */
-   scan_settings.overwrite_playlist = false;
    scan_settings.validate_entries   = true;
 
 end:
@@ -748,29 +749,28 @@ bool manual_content_scan_get_menu_content_dir(const char **content_dir)
  * actual system name used when generating the playlist */
 bool manual_content_scan_get_menu_system_name(const char **system_name)
 {
-   if (!system_name)
-      return false;
-
-   switch (scan_settings.system_name_type)
+   if (system_name)
    {
-      case MANUAL_CONTENT_SCAN_SYSTEM_NAME_CONTENT_DIR:
-         *system_name = msg_hash_to_str(
-               MENU_ENUM_LABEL_VALUE_MANUAL_CONTENT_SCAN_SYSTEM_NAME_USE_CONTENT_DIR);
-         return true;
-      case MANUAL_CONTENT_SCAN_SYSTEM_NAME_CUSTOM:
-         *system_name = msg_hash_to_str(
-               MENU_ENUM_LABEL_VALUE_MANUAL_CONTENT_SCAN_SYSTEM_NAME_USE_CUSTOM);
-         return true;
-      case MANUAL_CONTENT_SCAN_SYSTEM_NAME_DATABASE:
-         if (string_is_empty(scan_settings.system_name_database))
-            return false;
-         else
-         {
-            *system_name = scan_settings.system_name_database;
+      switch (scan_settings.system_name_type)
+      {
+         case MANUAL_CONTENT_SCAN_SYSTEM_NAME_CONTENT_DIR:
+            *system_name = msg_hash_to_str(
+                  MENU_ENUM_LABEL_VALUE_MANUAL_CONTENT_SCAN_SYSTEM_NAME_USE_CONTENT_DIR);
             return true;
-         }
-      default:
-         break;
+         case MANUAL_CONTENT_SCAN_SYSTEM_NAME_CUSTOM:
+            *system_name = msg_hash_to_str(
+                  MENU_ENUM_LABEL_VALUE_MANUAL_CONTENT_SCAN_SYSTEM_NAME_USE_CUSTOM);
+            return true;
+         case MANUAL_CONTENT_SCAN_SYSTEM_NAME_DATABASE:
+            if (!string_is_empty(scan_settings.system_name_database))
+            {
+               *system_name = scan_settings.system_name_database;
+               return true;
+            }
+            break;
+         default:
+            break;
+      }
    }
 
    return false;
@@ -780,25 +780,24 @@ bool manual_content_scan_get_menu_system_name(const char **system_name)
  * Returns true if core name is valid. */
 bool manual_content_scan_get_menu_core_name(const char **core_name)
 {
-   if (!core_name)
-      return false;
-
-   switch (scan_settings.core_type)
+   if (core_name)
    {
-      case MANUAL_CONTENT_SCAN_CORE_DETECT:
-         *core_name = msg_hash_to_str(
-               MENU_ENUM_LABEL_VALUE_MANUAL_CONTENT_SCAN_CORE_NAME_DETECT);
-         return true;
-      case MANUAL_CONTENT_SCAN_CORE_SET:
-         if (string_is_empty(scan_settings.core_name))
-            return false;
-         else
-         {
-            *core_name = scan_settings.core_name;
+      switch (scan_settings.core_type)
+      {
+         case MANUAL_CONTENT_SCAN_CORE_DETECT:
+            *core_name = msg_hash_to_str(
+                  MENU_ENUM_LABEL_VALUE_MANUAL_CONTENT_SCAN_CORE_NAME_DETECT);
             return true;
-         }
-      default:
-         break;
+         case MANUAL_CONTENT_SCAN_CORE_SET:
+            if (!string_is_empty(scan_settings.core_name))
+            {
+               *core_name = scan_settings.core_name;
+               return true;
+            }
+            break;
+         default:
+            break;
+      }
    }
 
    return false;
@@ -924,8 +923,8 @@ struct string_list *manual_content_scan_get_menu_core_name_list(void)
 
    if (core_info_list)
    {
-      core_info_t *core_info = NULL;
       size_t i;
+      core_info_t *core_info = NULL;
 
       /* Sort cores alphabetically */
       core_info_qsort(core_info_list, CORE_INFO_LIST_SORT_DISPLAY_NAME);
@@ -933,10 +932,7 @@ struct string_list *manual_content_scan_get_menu_core_name_list(void)
       /* Loop through cores */
       for (i = 0; i < core_info_list->count; i++)
       {
-         core_info = NULL;
-         core_info = core_info_get(core_info_list, i);
-
-         if (core_info)
+         if ((core_info = core_info_get(core_info_list, i)))
          {
             if (string_is_empty(core_info->display_name))
                continue;
@@ -1002,32 +998,26 @@ bool manual_content_scan_get_task_config(
       case MANUAL_CONTENT_SCAN_SYSTEM_NAME_CONTENT_DIR:
          if (string_is_empty(scan_settings.system_name_content_dir))
             return false;
-
          strlcpy(
                task_config->system_name,
                scan_settings.system_name_content_dir,
                sizeof(task_config->system_name));
-
          break;
       case MANUAL_CONTENT_SCAN_SYSTEM_NAME_CUSTOM:
          if (string_is_empty(scan_settings.system_name_custom))
             return false;
-
          strlcpy(
                task_config->system_name,
                scan_settings.system_name_custom,
                sizeof(task_config->system_name));
-
          break;
       case MANUAL_CONTENT_SCAN_SYSTEM_NAME_DATABASE:
          if (string_is_empty(scan_settings.system_name_database))
             return false;
-
          strlcpy(
                task_config->system_name,
                scan_settings.system_name_database,
                sizeof(task_config->system_name));
-
          break;
       default:
          return false;
@@ -1039,11 +1029,9 @@ bool manual_content_scan_get_task_config(
          task_config->database_name,
          task_config->system_name,
          sizeof(task_config->database_name));
-   task_config->database_name[len  ] = '.';
-   task_config->database_name[len+1] = 'l';
-   task_config->database_name[len+2] = 'p';
-   task_config->database_name[len+3] = 'l';
-   task_config->database_name[len+4] = '\0';
+   strlcpy(task_config->database_name       + len,
+         ".lpl",
+         sizeof(task_config->database_name) - len);
 
    /* ...which can in turn be used to generate the
     * playlist path */
@@ -1144,9 +1132,9 @@ bool manual_content_scan_get_task_config(
 struct string_list *manual_content_scan_get_content_list(
       manual_content_scan_task_config_t *task_config)
 {
-   struct string_list *dir_list = NULL;
    bool filter_exts;
    bool include_compressed;
+   struct string_list *dir_list = NULL;
 
    /* Sanity check */
    if (!task_config)
@@ -1227,8 +1215,8 @@ static bool manual_content_scan_get_playlist_content_path(
 
    /* Check whether this is an archive file
     * requiring special attention... */
-   if ((content_type == RARCH_COMPRESSED_ARCHIVE) &&
-       task_config->search_archives)
+   if (  (content_type == RARCH_COMPRESSED_ARCHIVE)
+       && task_config->search_archives)
    {
       bool filter_exts         = !string_is_empty(task_config->file_exts);
       const char *archive_file = NULL;
@@ -1274,9 +1262,9 @@ static bool manual_content_scan_get_playlist_content_path(
       if (filter_exts || (archive_list->size == 1))
       {
          /* Build path to file inside archive */
-         s[_len  ] = '#';
-         s[_len+1] = '\0';
-         strlcat(s, archive_file, len);
+         s[  _len] = '#';
+         s[++_len] = '\0';
+         strlcpy(s + _len, archive_file, len - _len);
       }
 
       string_list_free(archive_list);
@@ -1306,7 +1294,7 @@ static bool manual_content_scan_get_playlist_content_label(
 
    /* In most cases, content label is just the
     * filename without extension */
-   fill_pathname(content_label, path_basename(content_path), 
+   fill_pathname(content_label, path_basename(content_path),
          "", len);
 
    if (string_is_empty(content_label))
