@@ -144,8 +144,8 @@ static int action_left_input_desc(unsigned type, const char *label,
    unsigned bind_idx;
    unsigned mapped_port;
    settings_t *settings                  = config_get_ptr();
-   rarch_system_info_t *system           = &runloop_state_get_ptr()->system;
-   if (!settings || !system)
+   rarch_system_info_t *sys_info         = &runloop_state_get_ptr()->system;
+   if (!settings || !sys_info)
       return 0;
 
    user_idx    = (type - MENU_SETTINGS_INPUT_DESC_BEGIN) / (RARCH_FIRST_CUSTOM_BIND + 8);
@@ -184,7 +184,7 @@ static int action_left_input_desc(unsigned type, const char *label,
       also skip all the axes until analog remapping is implemented */
    if (remap_idx != RARCH_UNMAPPED)
    {
-      if ((string_is_empty(system->input_desc_btn[mapped_port][remap_idx]) && remap_idx < RARCH_CUSTOM_BIND_LIST_END) /*||
+      if ((string_is_empty(sys_info->input_desc_btn[mapped_port][remap_idx]) && remap_idx < RARCH_CUSTOM_BIND_LIST_END) /*||
           (remap_idx >= RARCH_FIRST_CUSTOM_BIND && remap_idx < RARCH_CUSTOM_BIND_LIST_END)*/)
          action_left_input_desc(type, label, wraparound);
    }
@@ -227,67 +227,60 @@ static int action_left_input_desc_kbd(unsigned type, const char *label,
 static int action_left_scroll(unsigned type, const char *label,
       bool wraparound)
 {
-   size_t scroll_accel   = 0;
-   unsigned scroll_speed = 0, fast_scroll_speed = 0;
-   size_t selection      = menu_navigation_get_selection();
-
-   if (!menu_driver_ctl(MENU_NAVIGATION_CTL_GET_SCROLL_ACCEL, &scroll_accel))
-      return false;
-
-   scroll_speed          = (unsigned)((MAX(scroll_accel, 2) - 2) / 4 + 1);
-   fast_scroll_speed     = 10 * scroll_speed;
+   struct menu_state *menu_st    = menu_state_get_ptr();
+   size_t selection              = menu_st->selection_ptr;
+   size_t scroll_accel           = menu_st->scroll.acceleration;
+   unsigned scroll_speed         = (unsigned)((MAX(scroll_accel, 2) - 2) / 4 + 1);
+   unsigned fast_scroll_speed    = 10 * scroll_speed;
 
    if (selection > fast_scroll_speed)
    {
-      size_t idx  = selection - fast_scroll_speed;
-      menu_navigation_set_selection(idx);
-      menu_driver_navigation_set(true);
+      size_t idx                 = selection - fast_scroll_speed;
+      menu_st->selection_ptr     = idx;
+      if (menu_st->driver_ctx->navigation_set)
+         menu_st->driver_ctx->navigation_set(menu_st->userdata, true);
    }
    else
    {
-      bool pending_push = false;
+      bool pending_push          = false;
       menu_driver_ctl(MENU_NAVIGATION_CTL_CLEAR, &pending_push);
    }
-
+#ifdef HAVE_AUDIOMIXER
+   if (selection != menu_st->selection_ptr) /* Changed? */
+      audio_driver_mixer_play_scroll_sound(true);
+#endif
    return 0;
 }
 
-static int action_left_goto_tab(void)
-{
-   menu_ctx_list_t list_info;
-   file_list_t *selection_buf = menu_entries_get_selection_buf_ptr(0);
-
-   list_info.type             = MENU_LIST_HORIZONTAL;
-   list_info.action           = MENU_ACTION_LEFT;
-
-   menu_driver_list_cache(&list_info);
-
-   return menu_driver_deferred_push_content_list(selection_buf);
-}
-
 static int action_left_mainmenu(unsigned type, const char *label,
       bool wraparound)
 {
-   menu_ctx_list_t list_info;
-   settings_t            *settings = config_get_ptr();
-   bool menu_nav_wraparound_enable = settings->bools.menu_navigation_wraparound_enable;
-   const char *menu_ident          = menu_driver_ident();
-
-   menu_driver_list_get_selection(&list_info);
-
-   list_info.type = MENU_LIST_PLAIN;
-
-   menu_driver_list_get_size(&list_info);
-
+#ifdef HAVE_XMB
+   struct menu_state    *menu_st       = menu_state_get_ptr();
+   const menu_ctx_driver_t *driver_ctx = menu_st->driver_ctx;
+   size_t size                         = (driver_ctx && driver_ctx->list_get_size) ? driver_ctx->list_get_size(menu_st->userdata, MENU_LIST_PLAIN) : 0;
+   const char *menu_ident              = (driver_ctx && driver_ctx->ident) ? driver_ctx->ident : NULL;
    /* Tab switching functionality only applies
     * to XMB */
-   if ((list_info.size == 1) &&
-       string_is_equal(menu_ident, "xmb"))
+   if (  (size == 1)
+       && string_is_equal(menu_ident, "xmb"))
    {
-      if ((list_info.selection != 0) || menu_nav_wraparound_enable)
-         return action_left_goto_tab();
+      settings_t            *settings  = config_get_ptr();
+      bool menu_nav_wraparound_enable  = settings->bools.menu_navigation_wraparound_enable;
+      size_t selection                 = (driver_ctx && driver_ctx->list_get_selection) ? driver_ctx->list_get_selection(menu_st->userdata) : 0;
+      if ((selection != 0) || menu_nav_wraparound_enable)
+      {
+         menu_list_t *menu_list        = menu_st->entries.list;
+         file_list_t *selection_buf    = menu_list ? MENU_LIST_GET_SELECTION(menu_list, 0) : NULL;
+
+         if (menu_st->driver_ctx && menu_st->driver_ctx->list_cache)
+            menu_st->driver_ctx->list_cache(menu_st->userdata,
+                  MENU_LIST_HORIZONTAL, MENU_ACTION_LEFT);
+         return menu_driver_deferred_push_content_list(selection_buf);
+      }
    }
    else
+#endif
       action_left_scroll(0, "", false);
 
    return 0;
@@ -318,7 +311,7 @@ static int action_left_shader_scale_pass(unsigned type, const char *label,
    shader_pass->fbo.scale_x   = current_scale;
    shader_pass->fbo.scale_y   = current_scale;
 
-   shader->flags           |= SHDR_FLAG_MODIFIED;
+   shader->flags             |= SHDR_FLAG_MODIFIED;
 
    return 0;
 }
@@ -326,7 +319,7 @@ static int action_left_shader_scale_pass(unsigned type, const char *label,
 static int action_left_shader_filter_pass(unsigned type, const char *label,
       bool wraparound)
 {
-   unsigned delta = 2;
+   unsigned delta                        = 2;
    unsigned pass                         = type - MENU_SETTINGS_SHADER_PASS_FILTER_0;
    struct video_shader *shader           = menu_shader_get();
    struct video_shader_pass *shader_pass = shader ? &shader->pass[pass] : NULL;
@@ -356,13 +349,14 @@ static int action_left_shader_filter_default(unsigned type, const char *label,
 static int action_left_cheat_num_passes(unsigned type, const char *label,
       bool wraparound)
 {
-   bool refresh      = false;
-   unsigned new_size = 0;
+   unsigned new_size          = 0;
+   struct menu_state *menu_st = menu_state_get_ptr();
+   unsigned cheat_size        = cheat_manager_get_size();
 
-   if (cheat_manager_get_size())
-      new_size = cheat_manager_get_size() - 1;
-   menu_entries_ctl(MENU_ENTRIES_CTL_SET_REFRESH, &refresh);
-   menu_driver_ctl(RARCH_MENU_CTL_SET_PREVENT_POPULATE, NULL);
+   if (cheat_size)
+      new_size         = cheat_size - 1;
+   menu_st->flags     |= MENU_ST_FLAG_ENTRIES_NEED_REFRESH
+                       | MENU_ST_FLAG_PREVENT_POPULATE;
    cheat_manager_realloc(new_size, CHEAT_HANDLER_TYPE_EMU);
 
    return 0;
@@ -373,7 +367,7 @@ static int action_left_cheat_num_passes(unsigned type, const char *label,
 static int action_left_shader_num_passes(unsigned type, const char *label,
       bool wraparound)
 {
-   bool refresh      = false;
+   struct menu_state *menu_st  = menu_state_get_ptr();
    struct video_shader *shader = menu_shader_get();
    unsigned pass_count         = shader ? shader->passes : 0;
 
@@ -383,8 +377,8 @@ static int action_left_shader_num_passes(unsigned type, const char *label,
    if (pass_count > 0)
       shader->passes--;
 
-   menu_entries_ctl(MENU_ENTRIES_CTL_SET_REFRESH, &refresh);
-   menu_driver_ctl(RARCH_MENU_CTL_SET_PREVENT_POPULATE, NULL);
+   menu_st->flags     |= MENU_ST_FLAG_ENTRIES_NEED_REFRESH
+                       | MENU_ST_FLAG_PREVENT_POPULATE;
    video_shader_resolve_parameters(shader);
 
    shader->flags                        |= SHDR_FLAG_MODIFIED;
@@ -683,17 +677,16 @@ static int manual_content_scan_core_name_left(unsigned type, const char *label,
    return 0;
 }
 
-#ifndef HAVE_LAKKA_SWITCH
 #ifdef HAVE_LAKKA
 static int cpu_policy_mode_change(unsigned type, const char *label,
       bool wraparound)
 {
-   bool refresh = false;
+   struct menu_state *menu_st = menu_state_get_ptr();
    enum cpu_scaling_mode mode = get_cpu_scaling_mode(NULL);
    if (mode != CPUSCALING_MANAGED_PERFORMANCE)
       mode--;
    set_cpu_scaling_mode(mode, NULL);
-   menu_entries_ctl(MENU_ENTRIES_CTL_SET_REFRESH, &refresh);
+   menu_st->flags |=  MENU_ST_FLAG_ENTRIES_NEED_REFRESH;
    return 0;
 }
 
@@ -798,7 +791,6 @@ static int cpu_policy_freq_tweak(unsigned type, const char *label,
    return 0;
 }
 #endif
-#endif /* #ifndef HAVE_LAKKA_SWITCH */
 
 static int core_setting_left(unsigned type, const char *label,
       bool wraparound)
@@ -861,7 +853,7 @@ static int action_left_video_gpu_index(unsigned type, const char *label,
             {
                configuration_set_int(settings,
                      settings->ints.vulkan_gpu_index,
-                     list->size - 1);
+                     (int)list->size - 1);
             }
          }
 
@@ -956,14 +948,44 @@ static int action_left_video_gpu_index(unsigned type, const char *label,
 static int action_left_state_slot(unsigned type, const char *label,
       bool wraparound)
 {
+   struct menu_state *menu_st     = menu_state_get_ptr();
    settings_t           *settings = config_get_ptr();
 
    settings->ints.state_slot--;
    if (settings->ints.state_slot < -1)
       settings->ints.state_slot = 999;
 
-   menu_driver_ctl(RARCH_MENU_CTL_UPDATE_SAVESTATE_THUMBNAIL_PATH, NULL);
-   menu_driver_ctl(RARCH_MENU_CTL_UPDATE_SAVESTATE_THUMBNAIL_IMAGE, NULL);
+   if (menu_st->driver_ctx)
+   {
+      size_t selection            = menu_st->selection_ptr;
+      if (menu_st->driver_ctx->update_savestate_thumbnail_path)
+         menu_st->driver_ctx->update_savestate_thumbnail_path(
+               menu_st->userdata, (unsigned)selection);
+      if (menu_st->driver_ctx->update_savestate_thumbnail_image)
+         menu_st->driver_ctx->update_savestate_thumbnail_image(menu_st->userdata);
+   }
+
+   return 0;
+}
+static int action_left_replay_slot(unsigned type, const char *label,
+      bool wraparound)
+{
+   struct menu_state *menu_st     = menu_state_get_ptr();
+   settings_t           *settings = config_get_ptr();
+
+   settings->ints.replay_slot--;
+   if (settings->ints.replay_slot < -1)
+      settings->ints.replay_slot = 999;
+
+   if (menu_st->driver_ctx)
+   {
+      size_t selection            = menu_st->selection_ptr;
+      if (menu_st->driver_ctx->update_savestate_thumbnail_path)
+         menu_st->driver_ctx->update_savestate_thumbnail_path(
+               menu_st->userdata, (unsigned)selection);
+      if (menu_st->driver_ctx->update_savestate_thumbnail_image)
+         menu_st->driver_ctx->update_savestate_thumbnail_image(menu_st->userdata);
+   }
 
    return 0;
 }
@@ -975,43 +997,9 @@ static int bind_left_generic(unsigned type, const char *label,
 }
 
 static int menu_cbs_init_bind_left_compare_label(menu_file_list_cbs_t *cbs,
-      const char *label, const char *menu_label)
+      const char *label, size_t lbl_len, const char *menu_label, size_t menu_lbl_len)
 {
 
-   if (cbs->setting)
-   {
-      const char *parent_group   = cbs->setting->parent_group;
-
-      if (string_is_equal(parent_group,
-               msg_hash_to_str(MENU_ENUM_LABEL_MAIN_MENU))
-               && (cbs->setting->type == ST_GROUP))
-      {
-         BIND_ACTION_LEFT(cbs, action_left_mainmenu);
-         return 0;
-      }
-   }
-
-   if (  string_starts_with_size(label, "input_player", STRLEN_CONST("input_player")) && 
-         string_ends_with_size(label, "_joypad_index", strlen(label),
-            STRLEN_CONST("_joypad_index")))
-   {
-      unsigned i;
-      for (i = 0; i < MAX_USERS; i++)
-      {
-         char label_setting[128];
-         label_setting[0] = '\0';
-
-         snprintf(label_setting,
-               sizeof(label_setting), "input_player%d_joypad_index", i + 1);
-
-         if (!string_is_equal(label, label_setting))
-            continue;
-
-         BIND_ACTION_LEFT(cbs, bind_left_generic);
-         return 0;
-      }
-   }
-
    if (string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_PLAYLISTS_TAB)))
    {
       BIND_ACTION_LEFT(cbs, action_left_mainmenu);
@@ -1074,7 +1062,7 @@ static int menu_cbs_init_bind_left_compare_label(menu_file_list_cbs_t *cbs,
             case MENU_ENUM_LABEL_EXPLORE_INITIALISING_LIST:
                if (
                         string_ends_with_size(menu_label, "_tab",
-                           strlen(menu_label),
+                           menu_lbl_len,
                            STRLEN_CONST("_tab")
                            )
                      || string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_MAIN_MENU))
@@ -1112,7 +1100,6 @@ static int menu_cbs_init_bind_left_compare_label(menu_file_list_cbs_t *cbs,
             case MENU_ENUM_LABEL_MANUAL_CONTENT_SCAN_CORE_NAME:
                BIND_ACTION_LEFT(cbs, manual_content_scan_core_name_left);
                break;
-            #ifndef HAVE_LAKKA_SWITCH
             #ifdef HAVE_LAKKA
             case MENU_ENUM_LABEL_CPU_PERF_MODE:
                BIND_ACTION_LEFT(cbs, cpu_policy_mode_change);
@@ -1131,7 +1118,6 @@ static int menu_cbs_init_bind_left_compare_label(menu_file_list_cbs_t *cbs,
                BIND_ACTION_LEFT(cbs, cpu_policy_freq_managed_gov);
                break;
             #endif
-            #endif
             default:
                return -1;
          }
@@ -1146,7 +1132,7 @@ static int menu_cbs_init_bind_left_compare_label(menu_file_list_cbs_t *cbs,
 }
 
 static int menu_cbs_init_bind_left_compare_type(menu_file_list_cbs_t *cbs,
-      unsigned type, const char *menu_label)
+      unsigned type, const char *menu_label, size_t menu_lbl_len)
 {
 #ifdef HAVE_CHEATS
    if (type >= MENU_SETTINGS_CHEAT_BEGIN
@@ -1208,9 +1194,7 @@ static int menu_cbs_init_bind_left_compare_type(menu_file_list_cbs_t *cbs,
          case FILE_TYPE_SHADER_PRESET:
          case FILE_TYPE_IMAGE:
          case FILE_TYPE_OVERLAY:
-#ifdef HAVE_VIDEO_LAYOUT
-         case FILE_TYPE_VIDEO_LAYOUT:
-#endif
+         case FILE_TYPE_OSK_OVERLAY:
          case FILE_TYPE_VIDEOFILTER:
          case FILE_TYPE_AUDIOFILTER:
          case FILE_TYPE_CONFIG:
@@ -1236,7 +1220,7 @@ static int menu_cbs_init_bind_left_compare_type(menu_file_list_cbs_t *cbs,
          case MENU_SETTINGS_CORE_INFO_NONE:
             if (  
                   string_ends_with_size(menu_label, "_tab",
-                     strlen(menu_label), STRLEN_CONST("_tab"))
+                     menu_lbl_len, STRLEN_CONST("_tab"))
                   || string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_HORIZONTAL_MENU))
                )
             {
@@ -1270,6 +1254,11 @@ static int menu_cbs_init_bind_left_compare_type(menu_file_list_cbs_t *cbs,
          case MENU_SETTING_ACTION_LOADSTATE:
             BIND_ACTION_LEFT(cbs, action_left_state_slot);
             break;
+         case MENU_SETTING_ACTION_RECORDREPLAY:
+         case MENU_SETTING_ACTION_PLAYREPLAY:
+         case MENU_SETTING_ACTION_HALTREPLAY:
+            BIND_ACTION_LEFT(cbs, action_left_replay_slot);
+            break;
          default:
             return -1;
       }
@@ -1279,8 +1268,10 @@ static int menu_cbs_init_bind_left_compare_type(menu_file_list_cbs_t *cbs,
 }
 
 int menu_cbs_init_bind_left(menu_file_list_cbs_t *cbs,
-      const char *path, const char *label, unsigned type, size_t idx,
-      const char *menu_label)
+      const char *path,
+      const char *label, size_t lbl_len,
+      unsigned type, size_t idx,
+      const char *menu_label, size_t menu_lbl_len)
 {
    if (!cbs)
       return -1;
@@ -1291,7 +1282,7 @@ int menu_cbs_init_bind_left(menu_file_list_cbs_t *cbs,
    {
       if (  
                string_ends_with_size(menu_label, "_tab",
-                  strlen(menu_label), STRLEN_CONST("_tab"))
+                  menu_lbl_len, STRLEN_CONST("_tab"))
             || string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_MAIN_MENU))
             || string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_HORIZONTAL_MENU))
          )
@@ -1301,10 +1292,23 @@ int menu_cbs_init_bind_left(menu_file_list_cbs_t *cbs,
       }
    }
 
-   if (menu_cbs_init_bind_left_compare_label(cbs, label, menu_label) == 0)
+   if (cbs->setting)
+   {
+      const char *parent_group   = cbs->setting->parent_group;
+
+      if (string_is_equal(parent_group,
+               msg_hash_to_str(MENU_ENUM_LABEL_MAIN_MENU))
+               && (cbs->setting->type == ST_GROUP))
+      {
+         BIND_ACTION_LEFT(cbs, action_left_mainmenu);
+         return 0;
+      }
+   }
+
+   if (menu_cbs_init_bind_left_compare_label(cbs, label, lbl_len, menu_label, menu_lbl_len) == 0)
       return 0;
 
-   if (menu_cbs_init_bind_left_compare_type(cbs, type, menu_label) == 0)
+   if (menu_cbs_init_bind_left_compare_type(cbs, type, menu_label, menu_lbl_len) == 0)
       return 0;
 
    return -1;
