@@ -181,9 +181,10 @@ static int64_t task_database_cue_get_token(intfstream_t *fd, char *token, uint64
    }
 }
 
+#define DISC_DATA_SIZE_PS1 60000
+
 int detect_ps1_game(intfstream_t *fd, char *s, size_t len, const char *filename)
 {
-   #define DISC_DATA_SIZE_PS1 60000
    int pos;
    char raw_game_id[50];
    char disc_data[DISC_DATA_SIZE_PS1];
@@ -254,7 +255,7 @@ int detect_ps1_game(intfstream_t *fd, char *s, size_t len, const char *filename)
 
          string_remove_all_whitespace(s, raw_game_id);
          cue_append_multi_disc_suffix(s, filename);
-         return true;
+         return false;
       }
    }
 
@@ -457,10 +458,10 @@ int detect_psp_game(intfstream_t *fd, char *s, size_t len, const char *filename)
 
 int detect_gc_game(intfstream_t *fd, char *s, size_t len, const char *filename)
 {
-   size_t _len;
    char region_id;
    char pre_game_id[20];
    char raw_game_id[20];
+   size_t _len = 0;
 
    /* Load raw serial or quit */
    if (intfstream_seek(fd, 0, SEEK_SET) < 0)
@@ -469,7 +470,7 @@ int detect_gc_game(intfstream_t *fd, char *s, size_t len, const char *filename)
    if (intfstream_read(fd, raw_game_id, 4) <= 0)
       return false;
 
-   if (string_is_equal_fast(raw_game_id, "RVZ", STRLEN_CONST("RVZ"))
+   if (     string_is_equal_fast(raw_game_id, "RVZ", STRLEN_CONST("RVZ"))
          || string_is_equal_fast(raw_game_id, "WIA", STRLEN_CONST("WIA")))
    {
       if (intfstream_seek(fd, 0x0058, SEEK_SET) < 0)
@@ -497,17 +498,17 @@ int detect_gc_game(intfstream_t *fd, char *s, size_t len, const char *filename)
    will not match redump.**/
 
    /** insert prefix **/
-   pre_game_id[0  ] = 'D';
-   pre_game_id[1  ] = 'L';
-   pre_game_id[2  ] = '-';
-   pre_game_id[3  ] = 'D';
-   pre_game_id[4  ] = 'O';
-   pre_game_id[5  ] = 'L';
-   pre_game_id[6  ] = '-';
-   pre_game_id[7  ] = '\0';
+   pre_game_id[  _len] = 'D';
+   pre_game_id[++_len] = 'L';
+   pre_game_id[++_len] = '-';
+   pre_game_id[++_len] = 'D';
+   pre_game_id[++_len] = 'O';
+   pre_game_id[++_len] = 'L';
+   pre_game_id[++_len] = '-';
+   pre_game_id[++_len] = '\0';
 
    /** add raw serial **/
-   strlcat(pre_game_id, raw_game_id, sizeof(pre_game_id));
+   strlcpy(pre_game_id + _len, raw_game_id, sizeof(pre_game_id) - _len);
 
    /** check region **/
    region_id = pre_game_id[10];
@@ -519,68 +520,68 @@ int detect_gc_game(intfstream_t *fd, char *s, size_t len, const char *filename)
    switch (region_id)
    {
       case 'E':
-         s[_len  ] = '-';
-         s[_len+1] = 'U';
-         s[_len+2] = 'S';
-         s[_len+3] = 'A';
-         s[_len+4] = '\0';
+         s[  _len] = '-';
+         s[++_len] = 'U';
+         s[++_len] = 'S';
+         s[++_len] = 'A';
+         s[++_len] = '\0';
          return true;
       case 'J':
-         s[_len  ] = '-';
-         s[_len+1] = 'J';
-         s[_len+2] = 'P';
-         s[_len+3] = 'N';
-         s[_len+4] = '\0';
+         s[  _len] = '-';
+         s[++_len] = 'J';
+         s[++_len] = 'P';
+         s[++_len] = 'N';
+         s[++_len] = '\0';
          return true;
       case 'P': /** NYI: P can also be P-UKV, P-AUS **/
       case 'X': /** NYI: X can also be X-UKV, X-EUU **/
-         s[_len  ] = '-';
-         s[_len+1] = 'E';
-         s[_len+2] = 'U';
-         s[_len+3] = 'R';
-         s[_len+4] = '\0';
+         s[  _len] = '-';
+         s[++_len] = 'E';
+         s[++_len] = 'U';
+         s[++_len] = 'R';
+         s[++_len] = '\0';
          return true;
       case 'Y':
-         s[_len  ] = '-';
-         s[_len+1] = 'F';
-         s[_len+2] = 'A';
-         s[_len+3] = 'H';
-         s[_len+4] = '\0';
+         s[  _len] = '-';
+         s[++_len] = 'F';
+         s[++_len] = 'A';
+         s[++_len] = 'H';
+         s[++_len] = '\0';
          return true;
       case 'D':
-         s[_len  ] = '-';
-         s[_len+1] = 'N';
-         s[_len+2] = 'O';
-         s[_len+3] = 'E';
-         s[_len+4] = '\0';
+         s[  _len] = '-';
+         s[++_len] = 'N';
+         s[++_len] = 'O';
+         s[++_len] = 'E';
+         s[++_len] = '\0';
          return true;
       case 'S':
-         s[_len  ] = '-';
-         s[_len+1] = 'E';
-         s[_len+2] = 'S';
-         s[_len+3] = 'P';
-         s[_len+4] = '\0';
+         s[  _len] = '-';
+         s[++_len] = 'E';
+         s[++_len] = 'S';
+         s[++_len] = 'P';
+         s[++_len] = '\0';
          return true;
       case 'F':
-         s[_len  ] = '-';
-         s[_len+1] = 'F';
-         s[_len+2] = 'R';
-         s[_len+3] = 'A';
-         s[_len+4] = '\0';
+         s[  _len] = '-';
+         s[++_len] = 'F';
+         s[++_len] = 'R';
+         s[++_len] = 'A';
+         s[++_len] = '\0';
          return true;
       case 'I':
-         s[_len  ] = '-';
-         s[_len+1] = 'I';
-         s[_len+2] = 'T';
-         s[_len+3] = 'A';
-         s[_len+4] = '\0';
+         s[  _len] = '-';
+         s[++_len] = 'I';
+         s[++_len] = 'T';
+         s[++_len] = 'A';
+         s[++_len] = '\0';
          return true;
       case 'H':
-         s[_len  ] = '-';
-         s[_len+1] = 'H';
-         s[_len+2] = 'O';
-         s[_len+3] = 'L';
-         s[_len+4] = '\0';
+         s[  _len] = '-';
+         s[++_len] = 'H';
+         s[++_len] = 'O';
+         s[++_len] = 'L';
+         s[++_len] = '\0';
          return true;
       default:
     break;
@@ -637,7 +638,8 @@ int detect_scd_game(intfstream_t *fd, char *s, size_t len, const char *filename)
    check_suffix_50[2] = '\0';
 
    /** redump serials are built differently for each prefix **/
-   if (pre_game_id[0] == 'T' && pre_game_id[1] == '-')
+   if (     pre_game_id[0] == 'T' 
+         && pre_game_id[1] == '-')
    {
       if (region_id == 'U' || region_id == 'J')
       {
@@ -653,14 +655,15 @@ int detect_scd_game(intfstream_t *fd, char *s, size_t len, const char *filename)
       strncpy(lgame_id, pre_game_id, index);
       lgame_id[index] = '\0';
       _len            = strlcat(s, lgame_id, len);
-      s[_len  ]       = '-';
-      s[_len+1]       = '5';
-      s[_len+2]       = '0';
-      s[_len+3]       = '\0';
+      s[  _len]       = '-';
+      s[++_len]       = '5';
+      s[++_len]       = '0';
+      s[++_len]       = '\0';
       cue_append_multi_disc_suffix(s, filename);
       return true;
    }
-   else if (pre_game_id[0] == 'G' && pre_game_id[1] == '-')
+   else if (pre_game_id[0] == 'G' 
+         && pre_game_id[1] == '-')
    {
       if ((index = string_index_last_occurance(pre_game_id, '-')) == -1)
          return false;
@@ -679,15 +682,16 @@ int detect_scd_game(intfstream_t *fd, char *s, size_t len, const char *filename)
          strncpy(lgame_id, &pre_game_id[3], 4);
          lgame_id[4]     = '\0';
          _len            = strlcat(s, lgame_id, len);
-         s[_len  ]       = '-';
-         s[_len+1]       = '5';
-         s[_len+2]       = '0';
-         s[_len+3]       = '\0';
-         cue_append_multi_disc_suffix(s, filename);
-         return true;
+         s[  _len]       = '-';
+         s[++_len]       = '5';
+         s[++_len]       = '0';
+         s[++_len]       = '\0';
+      }
+      else
+      {
+         strncpy(s, &pre_game_id[3], 4);
+         s[4]            = '\0';
       }
-      strncpy(s, &pre_game_id[3], 4);
-      s[4] = '\0';
       cue_append_multi_disc_suffix(s, filename);
       return true;
    }
@@ -757,13 +761,10 @@ int detect_sat_game(intfstream_t *fd, char *s, size_t len, const char *filename)
          {
             strncpy(s, &raw_game_id[3], length - 3);
             s[length - 3] = '\0';
-            cue_append_multi_disc_suffix(s, filename);
          }
          else
-         {
             strlcpy(s, raw_game_id, len);
-            cue_append_multi_disc_suffix(s, filename);
-         }
+         cue_append_multi_disc_suffix(s, filename);
          return true;
       case 'E':
          strncpy(lgame_id, &raw_game_id[0], 2);
@@ -779,12 +780,12 @@ int detect_sat_game(intfstream_t *fd, char *s, size_t len, const char *filename)
             strncpy(rgame_id, &raw_game_id[2], length - 1);
             rgame_id[length - 1] = '\0';
          }
-         strlcat(s, lgame_id, len);
-         _len      = strlcat(s, rgame_id, len);
-         s[_len  ] = '-';
-         s[_len+1] = '5';
-         s[_len+2] = '0';
-         s[_len+3] = '\0';
+         _len      = strlcat(s, lgame_id, len);
+         _len     += strlcpy(s + _len, rgame_id, len - _len);
+         s[  _len] = '-';
+         s[++_len] = '5';
+         s[++_len] = '0';
+         s[++_len] = '\0';
          cue_append_multi_disc_suffix(s, filename);
          return true;
       case 'J':
@@ -822,7 +823,8 @@ int detect_dc_game(intfstream_t *fd, char *s, size_t len, const char *filename)
    raw_game_id[10] = '\0';
 
    /** Scrub files with bad data and log **/
-   if (raw_game_id[0] == '\0' || raw_game_id[0] == ' ')
+   if (     raw_game_id[0] == '\0' 
+         || raw_game_id[0] == ' ')
    {
 #ifdef DEBUG
       RARCH_LOG("[Scanner]: Scrubbing: %s\n", filename);
@@ -851,27 +853,26 @@ int detect_dc_game(intfstream_t *fd, char *s, size_t len, const char *filename)
          strncpy(rgame_id, &raw_game_id[index + 1], length - 1);
          rgame_id[length - 1] = '\0';
          _len                 = strlcat(s, lgame_id, len);
-         s[_len  ]            = '-';
-         s[_len+1]            = '\0';
-         strlcat(s, rgame_id, len);
-         cue_append_multi_disc_suffix(s, filename);
-         return true;
+         s[  _len]            = '-';
+         s[++_len]            = '\0';
+         strlcpy(s + _len, rgame_id, len - _len);
       }
-      if (length <= 7)
+      else if (length <= 7)
       {
          strncpy(s, raw_game_id, 7);
          s[7] = '\0';
-         cue_append_multi_disc_suffix(s, filename);
-         return true;
       }
-      strncpy(lgame_id, raw_game_id, 7);
-      lgame_id[7]          = '\0';
-      strncpy(rgame_id, &raw_game_id[length - 2], length - 1);
-      rgame_id[length - 1] = '\0';
-      _len                 = strlcat(s, lgame_id, len);
-      s[_len  ]            = '-';
-      s[_len+1]            = '\0';
-      strlcat(s, rgame_id, len);
+      else
+      {
+         strncpy(lgame_id, raw_game_id, 7);
+         lgame_id[7]          = '\0';
+         strncpy(rgame_id, &raw_game_id[length - 2], length - 1);
+         rgame_id[length - 1] = '\0';
+         _len                 = strlcat(s, lgame_id, len);
+         s[  _len]            = '-';
+         s[++_len]            = '\0';
+         strlcpy(s + _len, rgame_id, len - _len);
+      }
       cue_append_multi_disc_suffix(s, filename);
       return true;
    }
@@ -882,9 +883,9 @@ int detect_dc_game(intfstream_t *fd, char *s, size_t len, const char *filename)
       strncpy(rgame_id, &raw_game_id[1], length - 1);
       rgame_id[length - 1] = '\0';
       _len                 = strlcpy(pre_game_id, lgame_id, sizeof(pre_game_id));
-      pre_game_id[_len  ]  = '-';
-      pre_game_id[_len+1]  = '\0';
-      strlcat(pre_game_id, rgame_id, sizeof(pre_game_id));
+      pre_game_id[  _len]  = '-';
+      pre_game_id[++_len]  = '\0';
+      strlcpy(pre_game_id + _len, rgame_id, sizeof(pre_game_id) - _len);
       total_hyphens_recalc = string_count_occurrences_single_character(pre_game_id, '-');
 
       if (total_hyphens_recalc >= 2)
@@ -896,13 +897,6 @@ int detect_dc_game(intfstream_t *fd, char *s, size_t len, const char *filename)
          strncpy(lgame_id, pre_game_id, size_t_var);
          lgame_id[index]             = '\0';
          length_recalc               = strlen(pre_game_id);
-         strncpy(rgame_id, &pre_game_id[length_recalc - 2], length_recalc - 1);
-         rgame_id[length_recalc - 1] = '\0';
-         _len                        = strlcat(s, lgame_id, len);
-         s[_len  ]                   = '-';
-         s[_len+1]                   = '\0';
-         strlcat(s, rgame_id, len);
-         cue_append_multi_disc_suffix(s, filename);
       }
       else
       {
@@ -916,14 +910,14 @@ int detect_dc_game(intfstream_t *fd, char *s, size_t len, const char *filename)
          }
          strncpy(lgame_id, pre_game_id, 7);
          lgame_id[7] = '\0';
-         strncpy(rgame_id, &pre_game_id[length_recalc - 2], length_recalc - 1);
-         rgame_id[length_recalc - 1] = '\0';
-         _len                        = strlcat(s, lgame_id, len);
-         s[_len  ]                   = '-';
-         s[_len+1]                   = '\0';
-         strlcat(s, rgame_id ,len);
-         cue_append_multi_disc_suffix(s, filename);
       }
+      strncpy(rgame_id, &pre_game_id[length_recalc - 2], length_recalc - 1);
+      rgame_id[length_recalc - 1] = '\0';
+      _len                        = strlcat(s, lgame_id, len);
+      s[  _len]                   = '-';
+      s[++_len]                   = '\0';
+      strlcpy(s + _len, rgame_id, len - _len);
+      cue_append_multi_disc_suffix(s, filename);
       return true;
    }
    else if (raw_game_id[0] == 'H'
@@ -941,13 +935,12 @@ int detect_dc_game(intfstream_t *fd, char *s, size_t len, const char *filename)
          strncpy(rgame_id, &raw_game_id[length - 4], length - 3);
          rgame_id[length - 3] = '\0';
          _len                 = strlcat(s, lgame_id, len);
-         s[_len  ]            = '-';
-         s[_len+1]            = '\0';
-         strlcat(s, rgame_id, len);
-         cue_append_multi_disc_suffix(s, filename);
-         return true;
+         s[  _len]            = '-';
+         s[++_len]            = '\0';
+         strlcpy(s + _len, rgame_id, len - _len);
       }
-      strlcpy(s, raw_game_id, len);
+      else
+         strlcpy(s, raw_game_id, len);
       cue_append_multi_disc_suffix(s, filename);
       return true;
    }
@@ -963,22 +956,24 @@ int detect_dc_game(intfstream_t *fd, char *s, size_t len, const char *filename)
          {
             strncpy(s, raw_game_id + 3, 5);
             s[5] = '\0';
-            cue_append_multi_disc_suffix(s, filename);
-            return true;
          }
-         strncpy(s, raw_game_id, 8);
-         s[8] = '\0';
-         cue_append_multi_disc_suffix(s, filename);
-         return true;
+         else
+         {
+            strncpy(s, raw_game_id, 8);
+            s[8] = '\0';
+         }
+      }
+      else
+      {
+         strncpy(lgame_id, raw_game_id, 8);
+         lgame_id[8]          = '\0';
+         strncpy(rgame_id, &raw_game_id[length - 2], length - 1);
+         rgame_id[length - 1] = '\0';
+         _len                 = strlcat(s, lgame_id, len);
+         s[  _len]            = '-';
+         s[++_len]            = '\0';
+         strlcpy(s + _len, rgame_id, len - _len);
       }
-      strncpy(lgame_id, raw_game_id, 8);
-      lgame_id[8]          = '\0';
-      strncpy(rgame_id, &raw_game_id[length - 2], length - 1);
-      rgame_id[length - 1] = '\0';
-      _len                 = strlcat(s, lgame_id, len);
-      s[_len  ]            = '-';
-      s[_len+1]            = '\0';
-      strlcat(s, rgame_id, len);
       cue_append_multi_disc_suffix(s, filename);
       return true;
    }
@@ -1010,7 +1005,7 @@ int detect_wii_game(intfstream_t *fd, char *s, size_t len, const char *filename)
          return false;
    }
 
-   if (string_is_equal_fast(raw_game_id, "RVZ", STRLEN_CONST("RVZ"))
+   if (     string_is_equal_fast(raw_game_id, "RVZ", STRLEN_CONST("RVZ"))
          || string_is_equal_fast(raw_game_id, "WIA", STRLEN_CONST("WIA")))
    {
       if (intfstream_seek(fd, 0x0058, SEEK_SET) < 0)
@@ -1021,7 +1016,8 @@ int detect_wii_game(intfstream_t *fd, char *s, size_t len, const char *filename)
    raw_game_id[6] = '\0';
 
    /** Scrub files with bad data and log **/
-   if (raw_game_id[0] == '\0' || raw_game_id[0] == ' ')
+   if (     raw_game_id[0] == '\0' 
+         || raw_game_id[0] == ' ')
    {
 #ifdef DEBUG
       RARCH_LOG("[Scanner]: Scrubbing: %s\n", filename);
@@ -1062,9 +1058,9 @@ static int detect_serial_ascii_game(intfstream_t *fd, char *s, size_t len)
          for (i = 0; i < 15; i++)
          {
             /* Is the given character ASCII? A-Z, 0-9, - */
-            if (  (s[i] == 45) ||
-                  (s[i] >= 48 && s[i] <= 57) ||
-                  (s[i] >= 65 && s[i] <= 90))
+            if (     (s[i] == 45)
+                  || (s[i] >= 48 && s[i] <= 57)
+                  || (s[i] >= 65 && s[i] <= 90))
                number_of_ascii++;
             else
                break;
@@ -1247,8 +1243,9 @@ int cue_find_track(const char *cue_path, bool first,
          last_index = (size_t) (((m * 60 + s) * 75) + f) * 2352;
 
          /* If we've changed tracks since the candidate, update it */
-         if (cand_track != -1 && track != cand_track &&
-             update_cand(&cand_index, &last_index, &largest,
+         if (     (cand_track != -1)
+               && (track != cand_track)
+               && update_cand(&cand_index, &last_index, &largest,
                 last_file, offset,
                 size, track_path, max_len))
          {
