@@ -16,9 +16,9 @@
  *  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
-#include <stdio.h>
 #include <ctype.h>
 #include <boolean.h>
 
@@ -38,6 +38,7 @@
 #ifdef HAVE_MENU
 #include "../menu/menu_driver.h"
 #endif
+#include "../runloop.h"
 #endif
 
 enum manual_scan_status
@@ -119,7 +120,7 @@ static void cb_task_manual_content_scan(
    manual_scan_handle_t *manual_scan = NULL;
    playlist_t *cached_playlist       = playlist_get_cached();
 #if defined(RARCH_INTERNAL) && defined(HAVE_MENU)
-   menu_ctx_environment_t menu_environ;
+   struct menu_state *menu_st        = menu_state_get_ptr();
    if (!task)
       goto end;
 #else
@@ -127,9 +128,7 @@ static void cb_task_manual_content_scan(
       return;
 #endif
 
-   manual_scan = (manual_scan_handle_t*)task->state;
-
-   if (!manual_scan)
+   if (!(manual_scan = (manual_scan_handle_t*)task->state))
    {
 #if defined(RARCH_INTERNAL) && defined(HAVE_MENU)
       goto end;
@@ -169,10 +168,9 @@ static void cb_task_manual_content_scan(
 end:
    /* When creating playlists, the playlist tabs of
     * any active menu driver must be refreshed */
-   menu_environ.type = MENU_ENVIRON_RESET_HORIZONTAL_LIST;
-   menu_environ.data = NULL;
-
-   menu_driver_ctl(RARCH_MENU_CTL_ENVIRONMENT, &menu_environ);
+   if (menu_st->driver_ctx->environ_cb)
+      menu_st->driver_ctx->environ_cb(MENU_ENVIRON_RESET_HORIZONTAL_LIST,
+            NULL, menu_st->userdata);
 #endif
 }
 
@@ -211,7 +209,7 @@ static void task_manual_content_scan_handler(retro_task_t *task)
                      manual_scan->task_config->file_exts, "|");
 
             /* Get content list */
-            if (!(manual_scan->content_list 
+            if (!(manual_scan->content_list
                      = manual_content_scan_get_content_list(
                         manual_scan->task_config)))
             {
@@ -249,7 +247,7 @@ static void task_manual_content_scan_handler(retro_task_t *task)
                playlist_clear(manual_scan->playlist);
 
             /* Get initial playlist size */
-            manual_scan->playlist_size = 
+            manual_scan->playlist_size =
                playlist_size(manual_scan->playlist);
 
             /* Set default core, if required */
@@ -276,6 +274,8 @@ static void task_manual_content_scan_handler(retro_task_t *task)
                   manual_scan->task_config->search_archives);
             playlist_set_scan_filter_dat_content(manual_scan->playlist,
                   manual_scan->task_config->filter_dat_content);
+            playlist_set_scan_overwrite_playlist(manual_scan->playlist,
+                  manual_scan->task_config->overwrite_playlist);
 
             /* All good - can start iterating
              * > If playlist has content and 'validate
@@ -299,20 +299,23 @@ static void task_manual_content_scan_handler(retro_task_t *task)
 
             if (entry)
             {
+               size_t _len;
                const char *entry_file     = NULL;
                const char *entry_file_ext = NULL;
-               char task_title[PATH_MAX_LENGTH];
+               char task_title[128];
 
                /* Update progress display */
                task_free_title(task);
 
-               strlcpy(task_title,
+               _len = strlcpy(task_title,
                      msg_hash_to_str(MSG_MANUAL_CONTENT_SCAN_PLAYLIST_CLEANUP),
                      sizeof(task_title));
 
                if (!string_is_empty(entry->path) &&
                    (entry_file = path_basename(entry->path)))
-                  strlcat(task_title, entry_file, sizeof(task_title));
+                  strlcpy(task_title       + _len,
+                        entry_file,
+                        sizeof(task_title) - _len);
 
                task_set_title(task, strdup(task_title));
                task_set_progress(task, (manual_scan->playlist_index * 100) /
@@ -362,18 +365,21 @@ static void task_manual_content_scan_handler(retro_task_t *task)
 
             if (!string_is_empty(content_path))
             {
-               char task_title[PATH_MAX_LENGTH];
+               size_t _len;
+               char task_title[128];
                const char *content_file = path_basename(content_path);
 
                /* Update progress display */
                task_free_title(task);
 
-               strlcpy(task_title,
+               _len = strlcpy(task_title,
                      msg_hash_to_str(MSG_MANUAL_CONTENT_SCAN_IN_PROGRESS),
                      sizeof(task_title));
 
                if (!string_is_empty(content_file))
-                  strlcat(task_title, content_file, sizeof(task_title));
+                  strlcpy(task_title       + _len,
+                        content_file,
+                        sizeof(task_title) - _len);
 
                task_set_title(task, strdup(task_title));
                task_set_progress(task,
@@ -420,19 +426,22 @@ static void task_manual_content_scan_handler(retro_task_t *task)
 
             if (!string_is_empty(m3u_path))
             {
-               char task_title[PATH_MAX_LENGTH];
+               size_t _len;
+               char task_title[128];
                const char *m3u_name = path_basename_nocompression(m3u_path);
                m3u_file_t *m3u_file = NULL;
 
                /* Update progress display */
                task_free_title(task);
 
-               strlcpy(task_title,
+               _len = strlcpy(task_title,
                      msg_hash_to_str(MSG_MANUAL_CONTENT_SCAN_M3U_CLEANUP),
                      sizeof(task_title));
 
                if (!string_is_empty(m3u_name))
-                  strlcat(task_title, m3u_name, sizeof(task_title));
+                  strlcpy(task_title       + _len,
+                        m3u_name,
+                        sizeof(task_title) - _len);
 
                task_set_title(task, strdup(task_title));
                task_set_progress(task, (manual_scan->m3u_index * 100) /
@@ -467,7 +476,8 @@ static void task_manual_content_scan_handler(retro_task_t *task)
          break;
       case MANUAL_SCAN_END:
          {
-            char task_title[PATH_MAX_LENGTH];
+            size_t _len;
+            char task_title[128];
 
             /* Ensure playlist is alphabetically sorted
              * > Override user settings here */
@@ -480,11 +490,12 @@ static void task_manual_content_scan_handler(retro_task_t *task)
             /* Update progress display */
             task_free_title(task);
 
-            strlcpy(
+            _len = strlcpy(
                   task_title, msg_hash_to_str(MSG_MANUAL_CONTENT_SCAN_END),
                   sizeof(task_title));
-            strlcat(task_title, manual_scan->task_config->system_name,
-                  sizeof(task_title));
+            strlcpy(task_title       + _len,
+                  manual_scan->task_config->system_name,
+                  sizeof(task_title) - _len);
 
             task_set_title(task, strdup(task_title));
          }
@@ -493,9 +504,9 @@ static void task_manual_content_scan_handler(retro_task_t *task)
          task_set_progress(task, 100);
          goto task_finished;
    }
-   
+
    return;
-   
+
 task_finished:
 
    if (task)
@@ -520,8 +531,9 @@ bool task_push_manual_content_scan(
       const playlist_config_t *playlist_config,
       const char *playlist_directory)
 {
+   size_t _len;
    task_finder_data_t find_data;
-   char task_title[PATH_MAX_LENGTH];
+   char task_title[128];
    retro_task_t *task                = NULL;
    manual_scan_handle_t *manual_scan = NULL;
 
@@ -588,11 +600,12 @@ bool task_push_manual_content_scan(
       goto error;
 
    /* > Get task title */
-   strlcpy(
+   _len = strlcpy(
          task_title, msg_hash_to_str(MSG_MANUAL_CONTENT_SCAN_START),
          sizeof(task_title));
-   strlcat(task_title, manual_scan->task_config->system_name,
-         sizeof(task_title));
+   strlcpy(task_title       + _len,
+         manual_scan->task_config->system_name,
+         sizeof(task_title) - _len);
 
    /* > Configure task */
    task->handler                 = task_manual_content_scan_handler;
