@@ -47,9 +47,6 @@ struct overlay_loader
    unsigned pos;
    unsigned pos_increment;
 
-   float overlay_opacity;
-   overlay_layout_desc_t layout_desc;
-
    enum overlay_status state;
    enum overlay_image_transfer_status loading_status;
 
@@ -90,7 +87,7 @@ static void task_overlay_load_desc_image(
       fill_pathname_resolve_relative(path, loader->overlay_path,
             image_path, sizeof(path));
 
-      image_tex.supports_rgba = (loader->flags & OVERLAY_LOADER_RGBA_SUPPORT);
+      image_tex.supports_rgba = (loader->flags & OVERLAY_LOADER_RGBA_SUPPORT) ? true : false;
 
       if (image_texture_load(&image_tex, path))
       {
@@ -125,8 +122,10 @@ static void task_overlay_desc_populate_eightway_config(
       struct overlay_desc *desc,
       unsigned ol_idx, unsigned desc_idx)
 {
+   size_t _len;
    input_driver_state_t *input_st = input_state_get_ptr();
    overlay_eightway_config_t *eightway;
+   char conf_key_base[20];
    char conf_key[64];
    char *str;
 
@@ -164,40 +163,39 @@ static void task_overlay_desc_populate_eightway_config(
          return;
    }
 
+   snprintf(conf_key_base, sizeof(conf_key_base), "overlay%u_desc%u", ol_idx, desc_idx);
+
    /* Redefine eightway vals if specified in conf
     */
-   snprintf(conf_key, sizeof(conf_key),
-         "overlay%u_desc%u_up", ol_idx, desc_idx);
+   _len = strlcpy(conf_key, conf_key_base, sizeof(conf_key));
+   strlcpy(conf_key + _len, "_up", sizeof(conf_key) - _len);
    if (config_get_string(loader->conf, conf_key, &str))
    {
       task_overlay_redefine_eightway_direction(str, &eightway->up);
       free(str);
    }
 
-   snprintf(conf_key, sizeof(conf_key),
-         "overlay%u_desc%u_down", ol_idx, desc_idx);
+   strlcpy(conf_key + _len, "_down", sizeof(conf_key) - _len);
    if (config_get_string(loader->conf, conf_key, &str))
    {
       task_overlay_redefine_eightway_direction(str, &eightway->down);
       free(str);
    }
 
-   snprintf(conf_key, sizeof(conf_key),
-         "overlay%u_desc%u_right", ol_idx, desc_idx);
-   if (config_get_string(loader->conf, conf_key, &str))
-   {
-      task_overlay_redefine_eightway_direction(str, &eightway->right);
-      free(str);
-   }
-
-   snprintf(conf_key, sizeof(conf_key),
-         "overlay%u_desc%u_left", ol_idx, desc_idx);
+   strlcpy(conf_key + _len, "_left", sizeof(conf_key) - _len);
    if (config_get_string(loader->conf, conf_key, &str))
    {
       task_overlay_redefine_eightway_direction(str, &eightway->left);
       free(str);
    }
 
+   strlcpy(conf_key + _len, "_right", sizeof(conf_key) - _len);
+   if (config_get_string(loader->conf, conf_key, &str))
+   {
+      task_overlay_redefine_eightway_direction(str, &eightway->right);
+      free(str);
+   }
+
    /* Prepopulate diagonals.
     */
    bits_or_bits(eightway->up_right.data, eightway->up.data,
@@ -229,10 +227,11 @@ static bool task_overlay_load_desc(
       unsigned width, unsigned height,
       bool normalized, float alpha_mod, float range_mod)
 {
+   size_t _len;
    float width_mod, height_mod;
    char conf_key[64];
    char overlay_desc_key[32];
-   char overlay_desc_normalized_key[32];
+   char overlay_key[64];
    char overlay[256];
    float tmp_float                      = 0.0f;
    bool tmp_bool                        = false;
@@ -245,17 +244,17 @@ static bool task_overlay_load_desc(
    const char *box                      = NULL;
    config_file_t *conf                  = loader->conf;
 
-   overlay_desc_key[0]                  = 
+   overlay_desc_key[0]                  =
+      overlay_key[0]                    =
       conf_key[0]                       =
-      overlay_desc_normalized_key[0]    = 
       overlay[0]                        = '\0';
 
    snprintf(overlay_desc_key, sizeof(overlay_desc_key),
          "overlay%u_desc%u", ol_idx, desc_idx);
 
-   snprintf(overlay_desc_normalized_key, sizeof(overlay_desc_normalized_key),
-         "overlay%u_desc%u_normalized", ol_idx, desc_idx);
-   if (config_get_bool(conf, overlay_desc_normalized_key, &tmp_bool))
+   _len = strlcpy(overlay_key, overlay_desc_key, sizeof(overlay_key));
+   strlcpy(overlay_key + _len, "_normalized", sizeof(overlay_key) - _len);
+   if (config_get_bool(conf, overlay_key, &tmp_bool))
       normalized = tmp_bool;
 
    by_pixel = !normalized;
@@ -308,7 +307,7 @@ static bool task_overlay_load_desc(
    else if (strstr(key, "retrok_") == key)
    {
       desc->type          = OVERLAY_TYPE_KEYBOARD;
-      desc->retro_key_idx = input_config_translate_str_to_rk(key + 7);
+      desc->retro_key_idx = input_config_translate_str_to_rk(key + 7, strlen(key + 7));
    }
    else
    {
@@ -325,13 +324,13 @@ static bool task_overlay_load_desc(
 
       if (BIT256_GET(desc->button_mask, RARCH_OVERLAY_NEXT))
       {
-         char overlay_target_key[64];
-
-         snprintf(overlay_target_key, sizeof(overlay_target_key),
-               "overlay%u_desc%u_next_target", ol_idx, desc_idx);
-         config_get_array(conf, overlay_target_key,
+         strlcpy(overlay_key + _len, "_next_target",
+               sizeof(overlay_key) - _len);
+         config_get_array(conf, overlay_key,
                desc->next_index_name, sizeof(desc->next_index_name));
       }
+      else if (BIT256_GET(desc->button_mask, RARCH_OSK))
+         BIT16_SET(loader->overlay_types, OVERLAY_TYPE_OSK_TOGGLE);
    }
 
    BIT16_SET(loader->overlay_types, desc->type);
@@ -365,27 +364,20 @@ static bool task_overlay_load_desc(
    {
       case OVERLAY_TYPE_ANALOG_LEFT:
       case OVERLAY_TYPE_ANALOG_RIGHT:
+         if (desc->hitbox != OVERLAY_HITBOX_RADIAL)
          {
-            char overlay_analog_saturate_key[64];
-
-            overlay_analog_saturate_key[0] = '\0';
-
-            if (desc->hitbox != OVERLAY_HITBOX_RADIAL)
-            {
-               RARCH_ERR("[Overlay]: Analog hitbox type must be \"radial\".\n");
-               ret = false;
-               goto end;
-            }
-
-            snprintf(overlay_analog_saturate_key,
-                  sizeof(overlay_analog_saturate_key),
-                  "overlay%u_desc%u_saturate_pct", ol_idx, desc_idx);
-            if (config_get_float(conf, overlay_analog_saturate_key,
-                     &tmp_float))
-               desc->analog_saturate_pct = tmp_float;
-            else
-               desc->analog_saturate_pct = 1.0f;
+            RARCH_ERR("[Overlay]: Analog hitbox type must be \"radial\".\n");
+            ret = false;
+            goto end;
          }
+
+         strlcpy(overlay_key + _len, "_saturate_pct",
+               sizeof(overlay_key) - _len);
+         if (config_get_float(conf, overlay_key,
+                  &tmp_float))
+            desc->analog_saturate_pct = tmp_float;
+         else
+            desc->analog_saturate_pct = 1.0f;
          break;
       case OVERLAY_TYPE_DPAD_AREA:
       case OVERLAY_TYPE_ABXY_AREA:
@@ -401,46 +393,72 @@ static bool task_overlay_load_desc(
    desc->range_x = (float)strtod(list.elems[4].data, NULL) * width_mod;
    desc->range_y = (float)strtod(list.elems[5].data, NULL) * height_mod;
 
-   snprintf(conf_key, sizeof(conf_key),
-         "overlay%u_desc%u_reach_right", ol_idx, desc_idx);
+   _len = strlcpy(conf_key, overlay_desc_key, sizeof(conf_key));
+
+   strlcpy(conf_key + _len, "_reach_x",   sizeof(conf_key) - _len);
    desc->reach_right = 1.0f;
-   if (config_get_float(conf, conf_key, &tmp_float))
-      desc->reach_right = tmp_float;
-
-   snprintf(conf_key, sizeof(conf_key),
-         "overlay%u_desc%u_reach_left", ol_idx, desc_idx);
-   desc->reach_left = 1.0f;
-   if (config_get_float(conf, conf_key, &tmp_float))
-      desc->reach_left = tmp_float;
-
-   snprintf(conf_key, sizeof(conf_key),
-         "overlay%u_desc%u_reach_up", ol_idx, desc_idx);
-   desc->reach_up = 1.0f;
-   if (config_get_float(conf, conf_key, &tmp_float))
-      desc->reach_up = tmp_float;
-
-   snprintf(conf_key, sizeof(conf_key),
-         "overlay%u_desc%u_reach_down", ol_idx, desc_idx);
-   desc->reach_down = 1.0f;
-   if (config_get_float(conf, conf_key, &tmp_float))
-      desc->reach_down = tmp_float;
-
-   snprintf(conf_key, sizeof(conf_key),
-         "overlay%u_desc%u_reach_x", ol_idx, desc_idx);
+   desc->reach_left  = 1.0f;
    if (config_get_float(conf, conf_key, &tmp_float))
    {
       desc->reach_right = tmp_float;
       desc->reach_left  = tmp_float;
    }
 
-   snprintf(conf_key, sizeof(conf_key),
-         "overlay%u_desc%u_reach_y", ol_idx, desc_idx);
+   strlcpy(conf_key + _len, "_reach_y",   sizeof(conf_key) - _len);
+   desc->reach_up   = 1.0f;
+   desc->reach_down = 1.0f;
    if (config_get_float(conf, conf_key, &tmp_float))
    {
       desc->reach_up   = tmp_float;
       desc->reach_down = tmp_float;
    }
 
+   strlcpy(conf_key + _len, "_movable",   sizeof(conf_key) - _len);
+   desc->flags    &= ~OVERLAY_DESC_MOVABLE;
+   desc->delta_x   = 0.0f;
+   desc->delta_y   = 0.0f;
+   if (config_get_bool(conf, conf_key, &tmp_bool)
+         && tmp_bool)
+      desc->flags |= OVERLAY_DESC_MOVABLE;
+
+   strlcpy(conf_key + _len, "_reach_up", sizeof(conf_key) - _len);
+   if (config_get_float(conf, conf_key, &tmp_float))
+      desc->reach_up = tmp_float;
+
+   strlcpy(conf_key + _len, "_alpha_mod",   sizeof(conf_key) - _len);
+   desc->alpha_mod = alpha_mod;
+   if (config_get_float(conf, conf_key, &tmp_float))
+         desc->alpha_mod = tmp_float;
+
+   strlcpy(conf_key + _len, "_range_mod",   sizeof(conf_key) - _len);
+   desc->range_mod = range_mod;
+   if (config_get_float(conf, conf_key, &tmp_float))
+      desc->range_mod = tmp_float;
+
+   strlcpy(conf_key + _len, "_exclusive",   sizeof(conf_key) - _len);
+   desc->flags &= ~OVERLAY_DESC_EXCLUSIVE;
+   if (config_get_bool(conf, conf_key, &tmp_bool)
+         && tmp_bool)
+      desc->flags |= OVERLAY_DESC_EXCLUSIVE;
+
+   strlcpy(conf_key + _len, "_reach_down",   sizeof(conf_key) - _len);
+   if (config_get_float(conf, conf_key, &tmp_float))
+      desc->reach_down = tmp_float;
+
+   strlcpy(conf_key + _len, "_reach_left",   sizeof(conf_key) - _len);
+   if (config_get_float(conf, conf_key, &tmp_float))
+      desc->reach_left = tmp_float;
+
+   strlcpy(conf_key + _len, "_reach_right",   sizeof(conf_key) - _len);
+   if (config_get_float(conf, conf_key, &tmp_float))
+      desc->reach_right = tmp_float;
+
+   strlcpy(conf_key + _len, "_range_mod_exclusive", sizeof(conf_key) - _len);
+   desc->flags &= ~OVERLAY_DESC_RANGE_MOD_EXCLUSIVE;
+   if (config_get_bool(conf, conf_key, &tmp_bool)
+         && tmp_bool)
+      desc->flags |= OVERLAY_DESC_RANGE_MOD_EXCLUSIVE;
+
    if (     (desc->reach_left == 0.0f && desc->reach_right == 0.0f)
          || (desc->reach_up   == 0.0f && desc->reach_down  == 0.0f))
       desc->hitbox = OVERLAY_HITBOX_NONE;
@@ -450,42 +468,6 @@ static bool task_overlay_load_desc(
    desc->mod_y   = desc->y - desc->range_y;
    desc->mod_h   = 2.0f * desc->range_y;
 
-   snprintf(conf_key, sizeof(conf_key),
-         "overlay%u_desc%u_alpha_mod", ol_idx, desc_idx);
-   desc->alpha_mod = alpha_mod;
-   if (config_get_float(conf, conf_key, &tmp_float))
-         desc->alpha_mod = tmp_float;
-
-   snprintf(conf_key, sizeof(conf_key),
-         "overlay%u_desc%u_range_mod", ol_idx, desc_idx);
-   desc->range_mod = range_mod;
-   if (config_get_float(conf, conf_key, &tmp_float))
-      desc->range_mod = tmp_float;
-
-   snprintf(conf_key, sizeof(conf_key),
-         "overlay%u_desc%u_exclusive", ol_idx, desc_idx);
-   desc->flags &= ~OVERLAY_DESC_EXCLUSIVE;
-   if (config_get_bool(conf, conf_key, &tmp_bool)
-         && tmp_bool)
-      desc->flags |= OVERLAY_DESC_EXCLUSIVE;
-
-   snprintf(conf_key, sizeof(conf_key),
-         "overlay%u_desc%u_range_mod_exclusive", ol_idx, desc_idx);
-   desc->flags &= ~OVERLAY_DESC_RANGE_MOD_EXCLUSIVE;
-   if (config_get_bool(conf, conf_key, &tmp_bool)
-         && tmp_bool)
-      desc->flags |= OVERLAY_DESC_RANGE_MOD_EXCLUSIVE;
-
-   snprintf(conf_key, sizeof(conf_key),
-         "overlay%u_desc%u_movable", ol_idx, desc_idx);
-   desc->flags    &= ~OVERLAY_DESC_MOVABLE;
-   desc->delta_x   = 0.0f;
-   desc->delta_y   = 0.0f;
-
-   if (config_get_bool(conf, conf_key, &tmp_bool)
-         && tmp_bool)
-      desc->flags |= OVERLAY_DESC_MOVABLE;
-
    input_overlay->pos ++;
 
 end:
@@ -561,16 +543,8 @@ static void task_overlay_resolve_iterate(retro_task_t *task)
    }
 
    if (loader->resolve_pos == 0)
-   {
       loader->active = &loader->overlays[0];
 
-#if 0
-      /* TODO: MOVE TO MAIN THREAD / CALLBACK */
-      input_overlay_load_active(loader->deferred.opacity);
-      input_overlay_enable(loader->deferred.enable);
-#endif
-   }
-
    loader->resolve_pos += 1;
 }
 
@@ -592,9 +566,7 @@ static void task_overlay_deferred_loading(retro_task_t *task)
       case OVERLAY_IMAGE_TRANSFER_NONE:
       case OVERLAY_IMAGE_TRANSFER_BUSY:
          loader->loading_status = OVERLAY_IMAGE_TRANSFER_DONE;
-#if 0
-         break;
-#endif
+	 /* fall-through */
       case OVERLAY_IMAGE_TRANSFER_DONE:
          task_overlay_image_done(&loader->overlays[loader->pos]);
          loader->loading_status = OVERLAY_IMAGE_TRANSFER_DESC_IMAGE_ITERATE;
@@ -667,8 +639,9 @@ static void task_overlay_deferred_load(retro_task_t *task)
 
    for (i = 0; i < loader->pos_increment; i++, loader->pos++)
    {
+      size_t _len;
+      char conf_key_base[10];
       char conf_key[32];
-      char overlay_full_screen_key[32];
       char tmp_str[PATH_MAX_LENGTH];
       float tmp_float                   = 0.0;
       bool tmp_bool                     = false;
@@ -684,12 +657,28 @@ static void task_overlay_deferred_load(retro_task_t *task)
          break;
       }
 
-      tmp_str[0] = conf_key[0] = overlay_full_screen_key[0] = '\0';
+      tmp_str[0] = '\0';
 
       overlay = &loader->overlays[loader->pos];
 
-      snprintf(overlay->config.descs.key,
-            sizeof(overlay->config.descs.key), "overlay%u_descs", loader->pos);
+      snprintf(conf_key_base, sizeof(conf_key_base), "overlay%u", loader->pos);
+      _len = strlcpy(conf_key, conf_key_base,  sizeof(conf_key));
+
+      strlcpy(conf_key + _len, "_rect", sizeof(conf_key) - _len);
+      strlcpy(overlay->config.rect.key, conf_key,
+            sizeof(overlay->config.rect.key));
+
+      strlcpy(conf_key + _len, "_name", sizeof(conf_key) - _len);
+      strlcpy(overlay->config.names.key, conf_key,
+            sizeof(overlay->config.names.key));
+
+      strlcpy(conf_key + _len, "_descs", sizeof(conf_key) - _len);
+      strlcpy(overlay->config.descs.key, conf_key,
+            sizeof(overlay->config.descs.key));
+
+      strlcpy(conf_key + _len, "_overlay", sizeof(conf_key) - _len);
+      strlcpy(overlay->config.paths.key, conf_key,
+            sizeof(overlay->config.paths.key));
 
       if (!config_get_uint(conf, overlay->config.descs.key,
                &overlay->config.descs.size))
@@ -711,31 +700,31 @@ static void task_overlay_deferred_load(retro_task_t *task)
       overlay->descs = overlay_desc;
       overlay->size  = overlay->config.descs.size;
 
-      snprintf(overlay_full_screen_key, sizeof(overlay_full_screen_key),
-            "overlay%u_full_screen", loader->pos);
-      overlay->flags &= ~OVERLAY_FULL_SCREEN;
-      if (config_get_bool(conf, overlay_full_screen_key, &tmp_bool)
-            && tmp_bool)
-         overlay->flags |= OVERLAY_FULL_SCREEN;
+      strlcpy(conf_key + _len, "_alpha_mod", sizeof(conf_key) - _len);
+      if (config_get_float(conf, conf_key, &tmp_float))
+         overlay->config.alpha_mod = tmp_float;
+      else
+         overlay->config.alpha_mod = 1.0f;
 
-      overlay->config.normalized = false;
-      overlay->config.alpha_mod  = 1.0f;
-      overlay->config.range_mod  = 1.0f;
-
-      snprintf(conf_key, sizeof(conf_key),
-            "overlay%u_normalized", loader->pos);
+      strlcpy(conf_key + _len, "_range_mod", sizeof(conf_key) - _len);
+      if (config_get_float(conf, conf_key, &tmp_float))
+         overlay->config.range_mod = tmp_float;
+      else
+         overlay->config.range_mod = 1.0f;
 
+      strlcpy(conf_key + _len, "_normalized", sizeof(conf_key) - _len);
       if (config_get_bool(conf, conf_key, &tmp_bool)
             && tmp_bool)
          overlay->config.normalized = tmp_bool;
+      else
+         overlay->config.normalized = false;
 
-      snprintf(conf_key, sizeof(conf_key), "overlay%u_alpha_mod", loader->pos);
-      if (config_get_float(conf, conf_key, &tmp_float))
-         overlay->config.alpha_mod = tmp_float;
-
-      snprintf(conf_key, sizeof(conf_key), "overlay%u_range_mod", loader->pos);
-      if (config_get_float(conf, conf_key, &tmp_float))
-         overlay->config.range_mod = tmp_float;
+      strlcpy(conf_key + _len, "_full_screen", sizeof(conf_key) - _len);
+      if (config_get_bool(conf, conf_key, &tmp_bool)
+            && tmp_bool)
+         overlay->flags |=  OVERLAY_FULL_SCREEN;
+      else
+         overlay->flags &= ~OVERLAY_FULL_SCREEN;
 
       /* Precache load image array for simplicity. */
       texture_img = (struct texture_image*)
@@ -749,9 +738,6 @@ static void task_overlay_deferred_load(retro_task_t *task)
 
       overlay->load_images = texture_img;
 
-      snprintf(overlay->config.paths.key, sizeof(overlay->config.paths.key),
-            "overlay%u_overlay", loader->pos);
-
       if (config_get_path(conf, overlay->config.paths.key,
                tmp_str, sizeof(tmp_str)))
          strlcpy(overlay->config.paths.path,
@@ -769,7 +755,7 @@ static void task_overlay_deferred_load(retro_task_t *task)
                overlay->config.paths.path, sizeof(overlay_resolved_path));
 
          image_tex.supports_rgba =
-               (loader->flags & OVERLAY_LOADER_RGBA_SUPPORT);
+               (loader->flags & OVERLAY_LOADER_RGBA_SUPPORT) ? true : false;
 
          if (!image_texture_load(&image_tex, overlay_resolved_path))
          {
@@ -783,17 +769,15 @@ static void task_overlay_deferred_load(retro_task_t *task)
          overlay->image = image_tex;
       }
 
-      snprintf(overlay->config.names.key, sizeof(overlay->config.names.key),
-            "overlay%u_name", loader->pos);
       config_get_array(conf, overlay->config.names.key,
             overlay->name, sizeof(overlay->name));
 
       /* Attempt to determine native aspect ratio */
-      snprintf(conf_key, sizeof(conf_key),
-            "overlay%u_aspect_ratio", loader->pos);
-      overlay->aspect_ratio = 0.0f;
+      strlcpy(conf_key + _len, "_aspect_ratio", sizeof(conf_key) - _len);
       if (config_get_float(conf, conf_key, &tmp_float))
          overlay->aspect_ratio = tmp_float;
+      else
+         overlay->aspect_ratio = 0.0f;
 
       if (overlay->aspect_ratio <= 0.0f)
       {
@@ -812,9 +796,6 @@ static void task_overlay_deferred_load(retro_task_t *task)
       overlay->x = overlay->y = 0.0f;
       overlay->w = overlay->h = 1.0f;
 
-      snprintf(overlay->config.rect.key, sizeof(overlay->config.rect.key),
-            "overlay%u_rect", loader->pos);
-
       if (config_get_array(conf, overlay->config.rect.key,
                overlay->config.rect.array, sizeof(overlay->config.rect.array)))
       {
@@ -823,7 +804,7 @@ static void task_overlay_deferred_load(retro_task_t *task)
          string_list_initialize(&list);
 
          if (     !string_split_noalloc(
-                  &list, overlay->config.rect.array, ", ")  
+                  &list, overlay->config.rect.array, ", ")
                || list.size < 4)
          {
             RARCH_ERR("[Overlay]: Failed to split rect \"%s\" into at least four tokens.\n",
@@ -847,19 +828,42 @@ static void task_overlay_deferred_load(retro_task_t *task)
 
       /* Check whether x/y separation are force disabled
        * for this overlay */
-      snprintf(conf_key, sizeof(conf_key),
-            "overlay%u_block_x_separation", loader->pos);
-      overlay->flags    &= ~OVERLAY_BLOCK_X_SEPARATION;
+      strlcpy(conf_key + _len, "_block_x_separation", sizeof(conf_key) - _len);
       if (config_get_bool(conf, conf_key, &tmp_bool)
             && tmp_bool)
          overlay->flags |=  OVERLAY_BLOCK_X_SEPARATION;
+      else
+         overlay->flags &= ~OVERLAY_BLOCK_X_SEPARATION;
 
-      snprintf(conf_key, sizeof(conf_key),
-            "overlay%u_block_y_separation", loader->pos);
-      overlay->flags    &= ~OVERLAY_BLOCK_Y_SEPARATION;
+      strlcpy(conf_key + _len, "_block_y_separation", sizeof(conf_key) - _len);
       if (config_get_bool(conf, conf_key, &tmp_bool)
             && tmp_bool)
          overlay->flags |=  OVERLAY_BLOCK_Y_SEPARATION;
+      else
+         overlay->flags &= ~OVERLAY_BLOCK_Y_SEPARATION;
+
+      /* Check whether x/y separation are enabled
+       * for this overlay in auto-scale mode */
+      strlcpy(conf_key + _len, "_auto_x_separation", sizeof(conf_key) - _len);
+      overlay->flags    |=  OVERLAY_AUTO_X_SEPARATION;
+      if (config_get_bool(conf, conf_key, &tmp_bool))
+      {
+         if (!tmp_bool)
+            overlay->flags &= ~OVERLAY_AUTO_X_SEPARATION;
+      }
+      else
+      {
+         if (overlay->flags & OVERLAY_BLOCK_X_SEPARATION
+               || overlay->image.width != 0)
+            overlay->flags &= ~OVERLAY_AUTO_X_SEPARATION;
+      }
+
+      strlcpy(conf_key + _len, "_auto_y_separation", sizeof(conf_key) - _len);
+      if (config_get_bool(conf, conf_key, &tmp_bool)
+            && tmp_bool)
+         overlay->flags |=  OVERLAY_AUTO_Y_SEPARATION;
+      else
+         overlay->flags &= ~OVERLAY_AUTO_Y_SEPARATION;
    }
 
    return;
@@ -876,11 +880,11 @@ static void task_overlay_free(retro_task_t *task)
    overlay_loader_t *loader  = (overlay_loader_t*)task->state;
    struct overlay *overlay   = &loader->overlays[loader->pos];
 
-   if (loader->overlay_path)
-      free(loader->overlay_path);
-
    if (task_get_cancelled(task))
    {
+      if (loader->overlay_path)
+         free(loader->overlay_path);
+
       for (i = 0; i < overlay->load_images_size; i++)
       {
          struct texture_image *ti = &overlay->load_images[i];
@@ -932,12 +936,9 @@ static void task_overlay_handler(retro_task_t *task)
       data->overlays                    = loader->overlays;
       data->active                      = loader->active;
       data->size                        = loader->size;
-      data->overlay_opacity             = loader->overlay_opacity;
       data->flags                       = loader->flags;
       data->overlay_types               = loader->overlay_types;
-
-      memcpy(&data->layout_desc, &loader->layout_desc,
-            sizeof(overlay_layout_desc_t));
+      data->overlay_path                = loader->overlay_path;
 
       task_set_data(task, data);
    }
@@ -963,11 +964,7 @@ static bool task_overlay_finder(retro_task_t *task, void *user_data)
 bool task_push_overlay_load_default(
       retro_task_callback_t cb,
       const char *overlay_path,
-      bool overlay_hide_in_menu,
-      bool overlay_hide_when_gamepad_connected,
-      bool input_overlay_enable,
-      float input_overlay_opacity,
-      overlay_layout_desc_t *layout_desc,
+      bool is_osk,
       void *user_data)
 {
    task_finder_data_t find_data;
@@ -975,7 +972,7 @@ bool task_push_overlay_load_default(
    config_file_t *conf      = NULL;
    overlay_loader_t *loader = NULL;
 
-   if (string_is_empty(overlay_path) || !layout_desc)
+   if (string_is_empty(overlay_path))
       return false;
 
    /* Prevent overlay from being loaded if it already is being loaded */
@@ -1014,25 +1011,17 @@ bool task_push_overlay_load_default(
       return false;
    }
 
-   loader->overlay_opacity  = input_overlay_opacity;
    loader->conf             = conf;
    loader->state            = OVERLAY_STATUS_DEFERRED_LOAD;
    loader->pos_increment    = (loader->size / 4) ? (loader->size / 4) : 4;
 
-   if (overlay_hide_in_menu)
-      loader->flags        |= OVERLAY_LOADER_HIDE_IN_MENU;
-   if (overlay_hide_when_gamepad_connected)
-      loader->flags        |= OVERLAY_LOADER_HIDE_WHEN_GAMEPAD_CONNECTED;
-   if (input_overlay_enable)
-      loader->flags        |= OVERLAY_LOADER_ENABLE;
+   if (is_osk)
+      loader->flags        |= OVERLAY_LOADER_IS_OSK;
 #ifdef RARCH_INTERNAL
    if (video_driver_supports_rgba())
       loader->flags        |= OVERLAY_LOADER_RGBA_SUPPORT;
 #endif
 
-   memcpy(&loader->layout_desc, layout_desc,
-         sizeof(overlay_layout_desc_t));
-
    t                        = task_init();
 
    if (!t)
