@@ -49,6 +49,7 @@ def replace_global_in(source):
         ('IN.output_size', 'OutputSize'),
         ('IN.frame_count', 'FrameCount'),
         ('IN.frame_direction', 'FrameDirection'),
+        ('IN.rotation', 'Rotation'),
     ]
 
     for line in source.splitlines():
@@ -107,6 +108,7 @@ def no_uniform(elem):
         '_output_dummy_size',
         '_frame_count',
         '_frame_direction',
+        '_rotation',
         '_mvp_matrix',
         '_vertex_coord',
         'sampler2D'
