@@ -48,29 +48,68 @@ AchievementsPage::AchievementsPage(QObject *parent) :
 QWidget *AchievementsPage::widget()
 {
    unsigned i;
-   QWidget               *widget     = new QWidget;
-   QVBoxLayout           *layout     = new QVBoxLayout;
-   enum msg_hash_enums check_setting = MENU_ENUM_LABEL_CHEEVOS_ENABLE;
-   CheckableSettingsGroup *group     = new CheckableSettingsGroup(check_setting);
-   settings_t *settings              = config_get_ptr();
-   file_list_t *list = (file_list_t*)calloc(1, sizeof(*list));
-   menu_displaylist_build_list(list, settings,
-         DISPLAYLIST_RETRO_ACHIEVEMENTS_SETTINGS_LIST, true);
+   QWidget             *widget = new QWidget;
+   QVBoxLayout         *layout = new QVBoxLayout;
+   settings_t*        settings = config_get_ptr();
+   CheckBox   *enabledCheckBox = new CheckBox(MENU_ENUM_LABEL_CHEEVOS_ENABLE);
+   file_list_t    *generalList = (file_list_t*)calloc(1, sizeof(*generalList));
+   file_list_t *appearanceList = (file_list_t*)calloc(1, sizeof(*appearanceList));
+   file_list_t *visibilityList = (file_list_t*)calloc(1, sizeof(*visibilityList));
 
-   for (i = 0; i < list->size; i++)
+   m_generalGroup              = new SettingsGroup("General");
+   m_appearanceGroup           = new SettingsGroup(msg_hash_to_str(
+      MENU_ENUM_LABEL_VALUE_CHEEVOS_APPEARANCE_SETTINGS));
+   m_visibilityGroup           = new SettingsGroup(msg_hash_to_str(
+      MENU_ENUM_LABEL_VALUE_CHEEVOS_VISIBILITY_SETTINGS));
+
+   menu_displaylist_build_list(generalList, settings,
+      DISPLAYLIST_RETRO_ACHIEVEMENTS_SETTINGS_LIST, true);
+
+   for (i = 0; i < generalList->size; i++)
    {
       menu_file_list_cbs_t *cbs = (menu_file_list_cbs_t*)
-         file_list_get_actiondata_at_offset(list, i);
+         file_list_get_actiondata_at_offset(generalList, i);
 
-      if (cbs->enum_idx == check_setting)
+      if (cbs->enum_idx == MENU_ENUM_LABEL_CHEEVOS_ENABLE)
          continue;
 
-      group->add(cbs->enum_idx);
+      m_generalGroup->add(cbs->enum_idx);
    }
+   file_list_free(generalList);
 
-   file_list_free(list);
+   menu_displaylist_build_list(appearanceList, settings,
+      DISPLAYLIST_CHEEVOS_APPEARANCE_SETTINGS_LIST, true);
 
-   layout->addWidget(group);
+   for (i = 0; i < appearanceList->size; i++)
+   {
+      menu_file_list_cbs_t* cbs = (menu_file_list_cbs_t*)
+         file_list_get_actiondata_at_offset(appearanceList, i);
+
+      m_appearanceGroup->add(cbs->enum_idx);
+   }
+   file_list_free(appearanceList);
+
+   menu_displaylist_build_list(visibilityList, settings,
+      DISPLAYLIST_CHEEVOS_VISIBILITY_SETTINGS_LIST, true);
+
+   for (i = 0; i < visibilityList->size; i++)
+   {
+      menu_file_list_cbs_t* cbs = (menu_file_list_cbs_t*)
+         file_list_get_actiondata_at_offset(visibilityList, i);
+
+      m_visibilityGroup->add(cbs->enum_idx);
+   }
+   file_list_free(visibilityList);
+
+   connect(enabledCheckBox, SIGNAL(stateChanged(int)),
+           this,            SLOT(onAchievementEnabledChanged(int)));
+
+   onAchievementEnabledChanged(enabledCheckBox->checkState());
+
+   layout->addWidget(enabledCheckBox);
+   layout->addWidget(m_generalGroup);
+   layout->addWidget(m_appearanceGroup);
+   layout->addWidget(m_visibilityGroup);
 
    layout->addStretch();
 
@@ -79,6 +118,13 @@ QWidget *AchievementsPage::widget()
    return widget;
 }
 
+void AchievementsPage::onAchievementEnabledChanged(int state)
+{
+   m_generalGroup->setDisabled(state == Qt::Unchecked);
+   m_appearanceGroup->setDisabled(state == Qt::Unchecked);
+   m_visibilityGroup->setDisabled(state == Qt::Unchecked);
+}
+
 AudioCategory::AudioCategory(QWidget *parent) :
    OptionsCategory(parent)
 {
@@ -210,7 +256,7 @@ QWidget *InputPage::widget()
       if (cbs->enum_idx == MENU_ENUM_LABEL_INPUT_HOTKEY_BINDS)
          break;
 
-      layout->add(cbs->enum_idx);
+      layout->add(menu_setting_find_enum(cbs->enum_idx));
    }
 
    file_list_free(list);
@@ -243,7 +289,7 @@ QWidget *HotkeyBindsPage::widget()
       menu_file_list_cbs_t *cbs = (menu_file_list_cbs_t*)
          file_list_get_actiondata_at_offset(list, i);
 
-      mainLayout->add(cbs->enum_idx);
+      mainLayout->add(menu_setting_find_enum(cbs->enum_idx));
    }
 
    file_list_free(list);
@@ -281,9 +327,9 @@ QWidget *UserBindsPage::widget()
       for (retro_id = 0; retro_id < RARCH_FIRST_CUSTOM_BIND + 20; retro_id++)
       {
          char descriptor[300];
-         const struct retro_keybind *keybind   = 
+         const struct retro_keybind *keybind   =
             &input_config_binds[p][retro_id];
-         const struct retro_keybind *auto_bind = 
+         const struct retro_keybind *auto_bind =
             (const struct retro_keybind*)
             input_config_get_bind_auto(p, retro_id);
 
@@ -353,17 +399,17 @@ QWidget *LatencyPage::widget()
       layout->addRow(hardSyncGroup);
    }
 
-   layout->add(MENU_ENUM_LABEL_VIDEO_MAX_SWAPCHAIN_IMAGES);
-   layout->add(MENU_ENUM_LABEL_VIDEO_WAITABLE_SWAPCHAINS);
-   layout->add(MENU_ENUM_LABEL_VIDEO_MAX_FRAME_LATENCY);
+   layout->add(menu_setting_find_enum(MENU_ENUM_LABEL_VIDEO_MAX_SWAPCHAIN_IMAGES));
+   layout->add(menu_setting_find_enum(MENU_ENUM_LABEL_VIDEO_WAITABLE_SWAPCHAINS));
+   layout->add(menu_setting_find_enum(MENU_ENUM_LABEL_VIDEO_MAX_FRAME_LATENCY));
 
-   layout->add(MENU_ENUM_LABEL_VIDEO_FRAME_DELAY);
-   layout->add(MENU_ENUM_LABEL_AUDIO_LATENCY);
-   layout->add(MENU_ENUM_LABEL_INPUT_POLL_TYPE_BEHAVIOR);
+   layout->add(menu_setting_find_enum(MENU_ENUM_LABEL_VIDEO_FRAME_DELAY));
+   layout->add(menu_setting_find_enum(MENU_ENUM_LABEL_AUDIO_LATENCY));
+   layout->add(menu_setting_find_enum(MENU_ENUM_LABEL_INPUT_POLL_TYPE_BEHAVIOR));
 
-   runAheadGpuSync->add(MENU_ENUM_LABEL_RUN_AHEAD_FRAMES);
-   runAheadGpuSync->add(MENU_ENUM_LABEL_RUN_AHEAD_SECONDARY_INSTANCE);
-   runAheadGpuSync->add(MENU_ENUM_LABEL_RUN_AHEAD_HIDE_WARNINGS);
+   runAheadGpuSync->add(menu_setting_find_enum(MENU_ENUM_LABEL_RUN_AHEAD_FRAMES));
+   runAheadGpuSync->add(menu_setting_find_enum(MENU_ENUM_LABEL_RUN_AHEAD_SECONDARY_INSTANCE));
+   runAheadGpuSync->add(menu_setting_find_enum(MENU_ENUM_LABEL_RUN_AHEAD_HIDE_WARNINGS));
    layout->addRow(runAheadGpuSync);
 
    widget->setLayout(layout);
@@ -412,19 +458,19 @@ QWidget *NetplayPage::widget()
    unsigned row               = 0;
    unsigned column            = 0;
 
-   checksLayout->add(MENU_ENUM_LABEL_NETPLAY_PUBLIC_ANNOUNCE);
-   checksLayout->add(MENU_ENUM_LABEL_NETPLAY_START_AS_SPECTATOR);
-   checksLayout->add(MENU_ENUM_LABEL_NETPLAY_FADE_CHAT);
-   checksLayout->add(MENU_ENUM_LABEL_NETPLAY_ALLOW_PAUSING);
-   checksLayout->add(MENU_ENUM_LABEL_NETWORK_ON_DEMAND_THUMBNAILS);
+   checksLayout->add(menu_setting_find_enum(MENU_ENUM_LABEL_NETPLAY_PUBLIC_ANNOUNCE));
+   checksLayout->add(menu_setting_find_enum(MENU_ENUM_LABEL_NETPLAY_START_AS_SPECTATOR));
+   checksLayout->add(menu_setting_find_enum(MENU_ENUM_LABEL_NETPLAY_FADE_CHAT));
+   checksLayout->add(menu_setting_find_enum(MENU_ENUM_LABEL_NETPLAY_ALLOW_PAUSING));
+   checksLayout->add(menu_setting_find_enum(MENU_ENUM_LABEL_NETWORK_ON_DEMAND_THUMBNAILS));
 
-   serverForm->add(MENU_ENUM_LABEL_NETPLAY_IP_ADDRESS);
-   serverForm->add(MENU_ENUM_LABEL_NETPLAY_TCP_UDP_PORT);
-   serverForm->add(MENU_ENUM_LABEL_NETPLAY_MAX_CONNECTIONS);
-   serverForm->add(MENU_ENUM_LABEL_NETPLAY_MAX_PING);
-   serverForm->add(MENU_ENUM_LABEL_NETPLAY_PASSWORD);
-   serverForm->add(MENU_ENUM_LABEL_NETPLAY_SPECTATE_PASSWORD);
-   serverForm->add(MENU_ENUM_LABEL_NETPLAY_NAT_TRAVERSAL);
+   serverForm->add(menu_setting_find_enum(MENU_ENUM_LABEL_NETPLAY_IP_ADDRESS));
+   serverForm->add(menu_setting_find_enum(MENU_ENUM_LABEL_NETPLAY_TCP_UDP_PORT));
+   serverForm->add(menu_setting_find_enum(MENU_ENUM_LABEL_NETPLAY_MAX_CONNECTIONS));
+   serverForm->add(menu_setting_find_enum(MENU_ENUM_LABEL_NETPLAY_MAX_PING));
+   serverForm->add(menu_setting_find_enum(MENU_ENUM_LABEL_NETPLAY_PASSWORD));
+   serverForm->add(menu_setting_find_enum(MENU_ENUM_LABEL_NETPLAY_SPECTATE_PASSWORD));
+   serverForm->add(menu_setting_find_enum(MENU_ENUM_LABEL_NETPLAY_NAT_TRAVERSAL));
 
    serverLayout->addWidget(createMitmServerGroup());
    serverLayout->addSpacing(30);
@@ -509,7 +555,7 @@ QGroupBox *NetplayPage::createMitmServerGroup()
 
 void NetplayPage::onRadioButtonClicked(int id)
 {
-   rarch_setting_t *setting = 
+   rarch_setting_t *setting =
       menu_setting_find_enum(MENU_ENUM_LABEL_NETPLAY_MITM_SERVER);
 
    if (!setting)
@@ -571,11 +617,12 @@ QWidget *NotificationsPage::widget()
    notificationsGroup->add(MENU_ENUM_LABEL_VIDEO_FONT_SIZE);
    notificationsGroup->add(MENU_ENUM_LABEL_VIDEO_MESSAGE_POS_X);
    notificationsGroup->add(MENU_ENUM_LABEL_VIDEO_MESSAGE_POS_Y);
+   /* TODO/FIXME - localize */
    notificationsGroup->addRow("Notification Color: ", new FloatColorButton(
       MENU_ENUM_LABEL_VIDEO_MESSAGE_COLOR_RED,
       MENU_ENUM_LABEL_VIDEO_MESSAGE_COLOR_GREEN,
       MENU_ENUM_LABEL_VIDEO_MESSAGE_COLOR_BLUE));
-
+   /* TODO/FIXME - localize */
    bgGroup->addRow("Notification Background Color: ", new UIntColorButton(
       MENU_ENUM_LABEL_VIDEO_MESSAGE_BGCOLOR_RED,
       MENU_ENUM_LABEL_VIDEO_MESSAGE_BGCOLOR_GREEN,
@@ -595,6 +642,7 @@ QWidget *NotificationsPage::widget()
    notificationsGroup->add(MENU_ENUM_LABEL_NOTIFICATION_SHOW_REMAP_LOAD);
    notificationsGroup->add(MENU_ENUM_LABEL_NOTIFICATION_SHOW_CONFIG_OVERRIDE_LOAD);
    notificationsGroup->add(MENU_ENUM_LABEL_NOTIFICATION_SHOW_SET_INITIAL_DISK);
+   notificationsGroup->add(MENU_ENUM_LABEL_NOTIFICATION_SHOW_SAVE_STATE);
    notificationsGroup->add(MENU_ENUM_LABEL_NOTIFICATION_SHOW_FAST_FORWARD);
    notificationsGroup->add(MENU_ENUM_LABEL_NOTIFICATION_SHOW_CHEATS_APPLIED);
    notificationsGroup->add(MENU_ENUM_LABEL_NOTIFICATION_SHOW_SCREENSHOT);
@@ -690,14 +738,16 @@ QWidget *PlaylistsPage::widget()
 
    layout->addRow(history);
 
-   layout->add(MENU_ENUM_LABEL_PLAYLIST_ENTRY_RENAME);
-   /*layout->add(MENU_ENUM_LABEL_PLAYLIST_ENTRY_REMOVE); TOFIX */
-   layout->add(MENU_ENUM_LABEL_PLAYLIST_SORT_ALPHABETICAL);
-   layout->add(MENU_ENUM_LABEL_PLAYLIST_USE_OLD_FORMAT);
-   layout->add(MENU_ENUM_LABEL_PLAYLIST_COMPRESSION);
-   layout->add(MENU_ENUM_LABEL_PLAYLIST_SHOW_SUBLABELS);
-   layout->add(MENU_ENUM_LABEL_PLAYLIST_FUZZY_ARCHIVE_MATCH);
-   layout->add(MENU_ENUM_LABEL_SCAN_WITHOUT_CORE_MATCH);
+   layout->add(menu_setting_find_enum(MENU_ENUM_LABEL_PLAYLIST_ENTRY_RENAME));
+#if 0
+   layout->add(menu_setting_find_enum(MENU_ENUM_LABEL_PLAYLIST_ENTRY_REMOVE));
+#endif
+   layout->add(menu_setting_find_enum(MENU_ENUM_LABEL_PLAYLIST_SORT_ALPHABETICAL));
+   layout->add(menu_setting_find_enum(MENU_ENUM_LABEL_PLAYLIST_USE_OLD_FORMAT));
+   layout->add(menu_setting_find_enum(MENU_ENUM_LABEL_PLAYLIST_COMPRESSION));
+   layout->add(menu_setting_find_enum(MENU_ENUM_LABEL_PLAYLIST_SHOW_SUBLABELS));
+   layout->add(menu_setting_find_enum(MENU_ENUM_LABEL_PLAYLIST_FUZZY_ARCHIVE_MATCH));
+   layout->add(menu_setting_find_enum(MENU_ENUM_LABEL_SCAN_WITHOUT_CORE_MATCH));
 
    widget->setLayout(layout);
 
@@ -951,8 +1001,8 @@ QWidget *ViewsPage::widget()
          menu_file_list_cbs_t *cbs = (menu_file_list_cbs_t*)
             file_list_get_actiondata_at_offset(list, i);
 
-         if (cbs->enum_idx == (kiosk_mode 
-                  ? MENU_ENUM_LABEL_CONTENT_SHOW_SETTINGS 
+         if (cbs->enum_idx == (kiosk_mode
+                  ? MENU_ENUM_LABEL_CONTENT_SHOW_SETTINGS
                   : MENU_ENUM_LABEL_CONTENT_SHOW_EXPLORE))
          {
             tabs_begin = i;
@@ -1015,7 +1065,7 @@ QWidget *ViewsPage::widget()
    leftLayout->addRow(mainMenu);
    leftLayout->addRow(settings);
    leftLayout->addRow(startScreen);
-   leftLayout->add(MENU_ENUM_LABEL_MENU_SHOW_SUBLABELS);
+   leftLayout->add(menu_setting_find_enum(MENU_ENUM_LABEL_MENU_SHOW_SUBLABELS));
 
    rightLayout->addWidget(tabs);
    rightLayout->addWidget(quickMenu);
@@ -1058,8 +1108,8 @@ QWidget *AppearancePage::widget()
    menu_displaylist_build_list(
          list, settings, DISPLAYLIST_MENU_SETTINGS_LIST, true);
 
-   /* TODO/FIXME - we haven't yet figured out how to 
-    * put a radio button setting next to another radio 
+   /* TODO/FIXME - we haven't yet figured out how to
+    * put a radio button setting next to another radio
     * button on the same row */
 
    for (i = 0; i < list->size; i++)
@@ -1069,12 +1119,13 @@ QWidget *AppearancePage::widget()
 
       switch (cbs->enum_idx)
       {
-         /* TODO/FIXME - this is a dirty hack - if we 
+         /* TODO/FIXME - this is a dirty hack - if we
           * detect this setting, we instead replace it with a
-          * color button and ignore the other two font color 
+          * color button and ignore the other two font color
           * settings since they are already covered by this one
           * color button */
          case MENU_ENUM_LABEL_MENU_FONT_COLOR_RED:
+            /* TODO/FIXME - localize */
             layout->addUIntColorButton("Menu Font Color: ",
                   MENU_ENUM_LABEL_MENU_FONT_COLOR_RED,
                   MENU_ENUM_LABEL_MENU_FONT_COLOR_GREEN,
@@ -1084,7 +1135,7 @@ QWidget *AppearancePage::widget()
          case MENU_ENUM_LABEL_MENU_FONT_COLOR_BLUE:
             break;
          default:
-            layout->add(cbs->enum_idx);
+            layout->add(menu_setting_find_enum(cbs->enum_idx));
             break;
       }
    }
@@ -1145,8 +1196,8 @@ QWidget *UserPage::widget()
    QWidget *widget    = new QWidget;
    FormLayout *layout = new FormLayout;
 
-   layout->add(MENU_ENUM_LABEL_NETPLAY_NICKNAME);
-   layout->add(MENU_ENUM_LABEL_USER_LANGUAGE);
+   layout->add(menu_setting_find_enum(MENU_ENUM_LABEL_NETPLAY_NICKNAME));
+   layout->add(menu_setting_find_enum(MENU_ENUM_LABEL_USER_LANGUAGE));
 
    widget->setLayout(layout);
 
@@ -1161,13 +1212,13 @@ AccountsPage::AccountsPage(QObject *parent) :
 
 QWidget *AccountsPage::widget()
 {
-   QWidget *widget             = new QWidget;
-   QVBoxLayout *layout         = new QVBoxLayout;
-   SettingsGroup *youtubeGroup = new SettingsGroup(msg_hash_to_str(MENU_ENUM_LABEL_VALUE_ACCOUNTS_YOUTUBE));
-   SettingsGroup *twitchGroup  = new SettingsGroup(msg_hash_to_str(MENU_ENUM_LABEL_VALUE_ACCOUNTS_TWITCH));
+   QWidget *widget              = new QWidget;
+   QVBoxLayout *layout          = new QVBoxLayout;
+   SettingsGroup *youtubeGroup  = new SettingsGroup(msg_hash_to_str(MENU_ENUM_LABEL_VALUE_ACCOUNTS_YOUTUBE));
+   SettingsGroup *twitchGroup   = new SettingsGroup(msg_hash_to_str(MENU_ENUM_LABEL_VALUE_ACCOUNTS_TWITCH));
    SettingsGroup *facebookGroup = new SettingsGroup(msg_hash_to_str(MENU_ENUM_LABEL_VALUE_ACCOUNTS_FACEBOOK));
 #ifdef HAVE_CHEEVOS
-   SettingsGroup *cheevosGroup = new SettingsGroup(msg_hash_to_str(MENU_ENUM_LABEL_VALUE_ACCOUNTS_RETRO_ACHIEVEMENTS));
+   SettingsGroup *cheevosGroup  = new SettingsGroup(msg_hash_to_str(MENU_ENUM_LABEL_VALUE_ACCOUNTS_RETRO_ACHIEVEMENTS));
 
    cheevosGroup->add(MENU_ENUM_LABEL_CHEEVOS_USERNAME);
    cheevosGroup->add(MENU_ENUM_LABEL_CHEEVOS_PASSWORD);
@@ -1321,8 +1372,10 @@ QWidget *VideoPage::widget()
 
    windowedGroup->add(MENU_ENUM_LABEL_VIDEO_WINDOW_SHOW_DECORATIONS);
    windowedGroup->add(MENU_ENUM_LABEL_UI_MENUBAR_ENABLE);
-   
+
    vSyncGroup->add(MENU_ENUM_LABEL_VIDEO_SWAP_INTERVAL);
+   vSyncGroup->add(MENU_ENUM_LABEL_VIDEO_SHADER_SUBFRAMES);
+   vSyncGroup->add(MENU_ENUM_LABEL_VIDEO_SCAN_SUBFRAMES);
    vSyncGroup->add(MENU_ENUM_LABEL_VIDEO_BLACK_FRAME_INSERTION);
    vSyncGroup->add(MENU_ENUM_LABEL_VIDEO_ADAPTIVE_VSYNC);
    vSyncGroup->add(MENU_ENUM_LABEL_VIDEO_FRAME_DELAY);
@@ -1500,10 +1553,10 @@ QWidget *CrtSwitchresPage::widget()
    m_crtSuperResolutionCombo->addItem("2560", 2560);
    m_crtSuperResolutionCombo->addItem("3840", 3840);
 
-   layout->add(MENU_ENUM_LABEL_CRT_SWITCH_RESOLUTION);
+   layout->add(menu_setting_find_enum(MENU_ENUM_LABEL_CRT_SWITCH_RESOLUTION));
    layout->addRow(msg_hash_to_str(MENU_ENUM_LABEL_VALUE_CRT_SWITCH_RESOLUTION_SUPER), m_crtSuperResolutionCombo);
-   layout->add(MENU_ENUM_LABEL_CRT_SWITCH_X_AXIS_CENTERING);
-   layout->add(MENU_ENUM_LABEL_CRT_SWITCH_RESOLUTION_USE_CUSTOM_REFRESH_RATE);
+   layout->add(menu_setting_find_enum(MENU_ENUM_LABEL_CRT_SWITCH_X_AXIS_CENTERING));
+   layout->add(menu_setting_find_enum(MENU_ENUM_LABEL_CRT_SWITCH_RESOLUTION_USE_CUSTOM_REFRESH_RATE));
 
    connect(m_crtSuperResolutionCombo, SIGNAL(currentIndexChanged(int)), this, SLOT(onCrtSuperResolutionComboIndexChanged(int)));
 
@@ -1524,7 +1577,7 @@ void CrtSwitchresPage::onCrtSuperResolutionComboIndexChanged(int index)
    settings_t *settings = config_get_ptr();
    Q_UNUSED(index)
 
-   settings->uints.crt_switch_resolution_super = 
+   settings->uints.crt_switch_resolution_super =
    m_crtSuperResolutionCombo->currentData().value<unsigned>();
 }
 
