@@ -28,11 +28,13 @@
 #include "cocoa/cocoa_common.h"
 #include "cocoa/apple_platform.h"
 #include "../ui_companion_driver.h"
+#include "../../audio/audio_driver.h"
 #include "../../configuration.h"
 #include "../../frontend/frontend.h"
 #include "../../input/drivers/cocoa_input.h"
 #include "../../input/drivers_keyboard/keyboard_event_apple.h"
 #include "../../retroarch.h"
+#include "../../tasks/task_content.h"
 #include "../../verbosity.h"
 
 #ifdef HAVE_MENU
@@ -40,11 +42,13 @@
 #endif
 
 #import <AVFoundation/AVFoundation.h>
+#import <CoreFoundation/CoreFoundation.h>
+
+#import <MetricKit/MetricKit.h>
+#import <MetricKit/MXMetricManager.h>
 
 #if defined(HAVE_COCOA_METAL) || defined(HAVE_COCOATOUCH)
-#if TARGET_OS_IOS
 #import "JITSupport.h"
-#endif
 id<ApplePlatform> apple_platform;
 #else
 static id apple_platform;
@@ -54,6 +58,46 @@ static CFRunLoopObserverRef iterate_observer;
 static void ui_companion_cocoatouch_event_command(
       void *data, enum event_command cmd) { }
 
+static struct string_list *ui_companion_cocoatouch_get_app_icons(void)
+{
+   static struct string_list *list = NULL;
+   static dispatch_once_t onceToken;
+
+   dispatch_once(&onceToken, ^{
+         union string_list_elem_attr attr;
+         attr.i = 0;
+         NSDictionary *iconfiles = [[NSBundle mainBundle] objectForInfoDictionaryKey:@"CFBundleIcons"];
+         NSString *primary;
+#if TARGET_OS_TV
+         primary = iconfiles[@"CFBundlePrimaryIcon"];
+#else
+         primary = iconfiles[@"CFBundlePrimaryIcon"][@"CFBundleIconName"];
+#endif
+         list = string_list_new();
+         string_list_append(list, [primary cStringUsingEncoding:kCFStringEncodingUTF8], attr);
+
+         NSArray<NSString *> *alts;
+#if TARGET_OS_TV
+         alts = iconfiles[@"CFBundleAlternateIcons"];
+#else
+         alts = [iconfiles[@"CFBundleAlternateIcons"] allKeys];
+#endif
+         NSArray<NSString *> *sorted = [alts sortedArrayUsingSelector:@selector(localizedCaseInsensitiveCompare:)];
+         for (NSString *str in sorted)
+            string_list_append(list, [str cStringUsingEncoding:kCFStringEncodingUTF8], attr);
+      });
+
+   return list;
+}
+
+static void ui_companion_cocoatouch_set_app_icon(const char *iconName)
+{
+   NSString *str;
+   if (!string_is_equal(iconName, "Default"))
+      str = [NSString stringWithCString:iconName encoding:NSUTF8StringEncoding];
+   [[UIApplication sharedApplication] setAlternateIconName:str completionHandler:nil];
+}
+
 static void rarch_draw_observer(CFRunLoopObserverRef observer,
     CFRunLoopActivity activity, void *info)
 {
@@ -67,6 +111,7 @@ static void rarch_draw_observer(CFRunLoopObserverRef observer,
       ui_companion_cocoatouch_event_command(
             NULL, CMD_EVENT_MENU_SAVE_CURRENT_CONFIG);
       main_exit(NULL);
+      exit(0);
       return;
    }
 
@@ -75,6 +120,27 @@ static void rarch_draw_observer(CFRunLoopObserverRef observer,
       CFRunLoopWakeUp(CFRunLoopGetMain());
 }
 
+void rarch_start_draw_observer(void)
+{
+   if (iterate_observer && CFRunLoopObserverIsValid(iterate_observer))
+       return;
+
+   if (iterate_observer != NULL)
+      CFRelease(iterate_observer);
+   iterate_observer = CFRunLoopObserverCreate(0, kCFRunLoopBeforeWaiting,
+                                              true, 0, rarch_draw_observer, 0);
+   CFRunLoopAddObserver(CFRunLoopGetMain(), iterate_observer, kCFRunLoopCommonModes);
+}
+
+void rarch_stop_draw_observer(void)
+{
+    if (!iterate_observer || !CFRunLoopObserverIsValid(iterate_observer))
+        return;
+    CFRunLoopObserverInvalidate(iterate_observer);
+    CFRelease(iterate_observer);
+    iterate_observer = NULL;
+}
+
 apple_frontend_settings_t apple_frontend_settings;
 
 void get_ios_version(int *major, int *minor)
@@ -90,6 +156,7 @@ void get_ios_version(int *major, int *minor)
 /* Input helpers: This is kept here because it needs ObjC */
 static void handle_touch_event(NSArray* touches)
 {
+#if !TARGET_OS_TV
    unsigned i;
    cocoa_input_data_t *apple = (cocoa_input_data_t*)
       input_state_get_ptr()->current_data;
@@ -110,6 +177,7 @@ static void handle_touch_event(NSArray* touches)
          apple->touches[apple->touch_count ++].screen_y = coord.y * scale;
       }
    }
+#endif
 }
 
 #ifndef HAVE_APPLE_STORE
@@ -273,7 +341,7 @@ enum
 - (void)sendEvent:(UIEvent *)event
 {
    [super sendEvent:event];
-    if (@available(iOS 13.4, *)) {
+    if (@available(iOS 13.4, tvOS 13.4, *)) {
         if (event.type == UIEventTypeHover)
             return;
     }
@@ -311,11 +379,20 @@ enum
 
 @end
 
+#if TARGET_OS_IOS
+@interface RetroArch_iOS () <MXMetricManagerSubscriber>
+
+@end
+#endif
+
 @implementation RetroArch_iOS
 
 #pragma mark - ApplePlatform
 -(id)renderView { return _renderView; }
--(bool)hasFocus { return YES; }
+-(bool)hasFocus
+{
+    return [[UIApplication sharedApplication] applicationState] == UIApplicationStateActive;
+}
 
 - (void)setViewType:(apple_view_type_t)vt
 {
@@ -361,6 +438,7 @@ enum
    [[_renderView.bottomAnchor constraintEqualToAnchor:rootView.bottomAnchor] setActive:YES];
    [[_renderView.leadingAnchor constraintEqualToAnchor:rootView.leadingAnchor] setActive:YES];
    [[_renderView.trailingAnchor constraintEqualToAnchor:rootView.trailingAnchor] setActive:YES];
+   [_renderView layoutIfNeeded];
 }
 
 - (apple_view_type_t)viewType { return _vt; }
@@ -378,7 +456,15 @@ enum
 }
 
 - (void)setCursorVisible:(bool)v { /* no-op for iOS */ }
-- (bool)setDisableDisplaySleep:(bool)disable { /* no-op for iOS */ return NO; }
+- (bool)setDisableDisplaySleep:(bool)disable
+{
+#if TARGET_OS_TV
+   [[UIApplication sharedApplication] setIdleTimerDisabled:disable];
+   return YES;
+#else
+   return NO;
+#endif
+}
 + (RetroArch_iOS*)get { return (RetroArch_iOS*)[[UIApplication sharedApplication] delegate]; }
 
 -(NSString*)documentsDirectory
@@ -395,9 +481,42 @@ enum
    return _documentsDirectory;
 }
 
+- (void)handleAudioSessionInterruption:(NSNotification *)notification
+{
+   NSNumber *type = notification.userInfo[AVAudioSessionInterruptionTypeKey];
+   if (![type isKindOfClass:[NSNumber class]])
+      return;
+
+   if ([type unsignedIntegerValue] == AVAudioSessionInterruptionTypeBegan)
+   {
+      RARCH_LOG("AudioSession Interruption Began\n");
+      audio_driver_stop();
+   }
+   else if ([type unsignedIntegerValue] == AVAudioSessionInterruptionTypeEnded)
+   {
+      RARCH_LOG("AudioSession Interruption Ended\n");
+      audio_driver_start(false);
+   }
+}
+
+- (NSData *)pngForIcon:(NSString *)iconName
+{
+    UIImage *img;
+    NSData *png;
+    img = [UIImage imageNamed:iconName];
+    if (!img)
+        NSLog(@"could not load %@\n", iconName);
+    else
+    {
+        png = UIImagePNGRepresentation(img);
+        if (!png)
+            NSLog(@"could not get png for %@\n", iconName);
+    }
+    return png;
+}
+
 - (void)applicationDidFinishLaunching:(UIApplication *)application
 {
-   NSError *error;
    char arguments[]   = "retroarch";
    char       *argv[] = {arguments,   NULL};
    int argc           = 1;
@@ -409,16 +528,45 @@ enum
    self.window        = [[UIWindow alloc] initWithFrame:[[UIScreen mainScreen] bounds]];
    [self.window makeKeyAndVisible];
 
-   [[AVAudioSession sharedInstance] setCategory:AVAudioSessionCategoryAmbient error:&error];
+   [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(handleAudioSessionInterruption:) name:AVAudioSessionInterruptionNotification object:[AVAudioSession sharedInstance]];
 
    [self refreshSystemConfig];
    [self showGameView];
 
+   jb_start_altkit();
+
    rarch_main(argc, argv, NULL);
 
-   iterate_observer = CFRunLoopObserverCreate(0, kCFRunLoopBeforeWaiting,
-         true, 0, rarch_draw_observer, 0);
-   CFRunLoopAddObserver(CFRunLoopGetMain(), iterate_observer, kCFRunLoopCommonModes);
+   uico_driver_state_t *uico_st     = uico_state_get_ptr();
+   rarch_setting_t *appicon_setting = menu_setting_find_enum(MENU_ENUM_LABEL_APPICON_SETTINGS);
+   struct string_list *icons;
+   if (appicon_setting && uico_st->drv && uico_st->drv->get_app_icons && (icons = uico_st->drv->get_app_icons()) && icons->size > 1)
+   {
+      appicon_setting->default_value.string = icons->elems[0].data;
+      int len = 0, i = 0;
+      const char *iconName = [[application alternateIconName] cStringUsingEncoding:kCFStringEncodingUTF8]; // need to ask uico_st for this
+      for (; i < (int)icons->size; i++)
+      {
+         len += strlen(icons->elems[i].data) + 1;
+         if (string_is_equal(iconName, icons->elems[i].data))
+            appicon_setting->value.target.string = icons->elems[i].data;
+      }
+      char *options = (char*)calloc(len, sizeof(char));
+      string_list_join_concat(options, len, icons, "|");
+      if (appicon_setting->values)
+         free((void*)appicon_setting->values);
+      appicon_setting->values = options;
+   }
+
+   rarch_start_draw_observer();
+
+#if TARGET_OS_TV
+   update_topshelf();
+#endif
+
+#if TARGET_OS_IOS
+   [MXMetricManager.sharedManager addSubscriber:self];
+#endif
 
 #ifdef HAVE_MFI
    extern void *apple_gamecontroller_joypad_init(void *data);
@@ -426,25 +574,85 @@ enum
 #endif
 }
 
-- (void)applicationDidEnterBackground:(UIApplication *)application { }
+- (void)applicationDidEnterBackground:(UIApplication *)application
+{
+#if TARGET_OS_TV
+   update_topshelf();
+#endif
+   rarch_stop_draw_observer();
+}
 
 - (void)applicationWillTerminate:(UIApplication *)application
 {
-   CFRunLoopObserverInvalidate(iterate_observer);
-   CFRelease(iterate_observer);
-   iterate_observer = NULL;
+   rarch_stop_draw_observer();
+   retroarch_main_quit();
+}
+
+- (void)applicationWillResignActive:(UIApplication *)application
+{
+   self.bgDate = [NSDate date];
+   rarch_stop_draw_observer();
 }
 
 - (void)applicationDidBecomeActive:(UIApplication *)application
 {
+   rarch_start_draw_observer();
+   NSError *error;
    settings_t *settings            = config_get_ptr();
    bool ui_companion_start_on_boot = settings->bools.ui_companion_start_on_boot;
 
+   if (settings->bools.audio_respect_silent_mode)
+       [[AVAudioSession sharedInstance] setCategory:AVAudioSessionCategoryAmbient error:&error];
+   else
+       [[AVAudioSession sharedInstance] setCategory:AVAudioSessionCategoryPlayback error:&error];
+
    if (!ui_companion_start_on_boot)
       [self showGameView];
+
+#ifdef HAVE_CLOUDSYNC
+   if (self.bgDate)
+   {
+      if (   [[NSDate date] timeIntervalSinceDate:self.bgDate] > 60.0f
+          && (   !(runloop_get_flags() & RUNLOOP_FLAG_CORE_RUNNING)
+              || retroarch_ctl(RARCH_CTL_IS_DUMMY_CORE, NULL)))
+         task_push_cloud_sync();
+      self.bgDate = nil;
+   }
+#endif
+}
+
+-(BOOL)openRetroArchURL:(NSURL *)url
+{
+   if ([url.host isEqualToString:@"topshelf"])
+   {
+      NSURLComponents *comp = [[NSURLComponents alloc] initWithURL:url resolvingAgainstBaseURL:NO];
+      NSString *ns_path, *ns_core_path;
+      char path[PATH_MAX_LENGTH];
+      char core_path[PATH_MAX_LENGTH];
+      content_ctx_info_t content_info = { 0 };
+      for (NSURLQueryItem *q in comp.queryItems)
+      {
+         if ([q.name isEqualToString:@"path"])
+            ns_path = q.value;
+         else if ([q.name isEqualToString:@"core_path"])
+            ns_core_path = q.value;
+      }
+      if (!ns_path || !ns_core_path)
+         return NO;
+      fill_pathname_expand_special(path, [ns_path UTF8String], sizeof(path));
+      fill_pathname_expand_special(core_path, [ns_core_path UTF8String], sizeof(core_path));
+      RARCH_LOG("TopShelf told us to open %s with %s\n", path, core_path);
+      return task_push_load_content_with_new_core_from_companion_ui(core_path, path,
+                                                                    NULL, NULL, NULL,
+                                                                    &content_info, NULL, NULL);
+   }
+   return NO;
 }
 
 -(BOOL)application:(UIApplication *)app openURL:(NSURL *)url options:(NSDictionary<UIApplicationOpenURLOptionsKey, id> *)options {
+    if ([[url scheme] isEqualToString:@"retroarch"])
+        return [self openRetroArchURL:url];
+
    NSFileManager *manager = [NSFileManager defaultManager];
    NSString     *filename = (NSString*)url.path.lastPathComponent;
    NSError         *error = nil;
@@ -475,9 +683,9 @@ enum
 #if TARGET_OS_IOS
    [self setToolbarHidden:true animated:NO];
    [[UIApplication sharedApplication] setStatusBarHidden:true withAnimation:UIStatusBarAnimationNone];
+   [[UIApplication sharedApplication] setIdleTimerDisabled:true];
 #endif
 
-   [[UIApplication sharedApplication] setIdleTimerDisabled:true];
    [self.window setRootViewController:[CocoaView get]];
 
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, 1.0 * NSEC_PER_SEC), dispatch_get_main_queue(), ^{
@@ -502,8 +710,49 @@ enum
 }
 
 - (void)supportOtherAudioSessions { }
+
+#if TARGET_OS_IOS
+- (void)didReceiveMetricPayloads:(NSArray<MXMetricPayload *> *)payloads
+{
+    for (MXMetricPayload *payload in payloads)
+    {
+        NSString *json = [[NSString alloc] initWithData:[payload JSONRepresentation] encoding:kCFStringEncodingUTF8];
+        RARCH_LOG("Got Metric Payload:\n%s\n", [json cStringUsingEncoding:kCFStringEncodingUTF8]);
+    }
+}
+
+- (void)didReceiveDiagnosticPayloads:(NSArray<MXDiagnosticPayload *> *)payloads
+{
+    for (MXDiagnosticPayload *payload in payloads)
+    {
+        NSString *json = [[NSString alloc] initWithData:[payload JSONRepresentation] encoding:kCFStringEncodingUTF8];
+        RARCH_LOG("Got Diagnostic Payload:\n%s\n", [json cStringUsingEncoding:kCFStringEncodingUTF8]);
+    }
+}
+#endif
+
 @end
 
+ui_companion_driver_t ui_companion_cocoatouch = {
+   NULL, /* init */
+   NULL, /* deinit */
+   NULL, /* toggle */
+   ui_companion_cocoatouch_event_command,
+   NULL, /* notify_refresh */
+   NULL, /* msg_queue_push */
+   NULL, /* render_messagebox */
+   NULL, /* get_main_window */
+   NULL, /* log_msg */
+   NULL, /* is_active */
+   ui_companion_cocoatouch_get_app_icons,
+   ui_companion_cocoatouch_set_app_icon,
+   NULL, /* browser_window */
+   NULL, /* msg_window */
+   NULL, /* window */
+   NULL, /* application */
+   "cocoatouch",
+};
+
 int main(int argc, char *argv[])
 {
 #if TARGET_OS_IOS
